JSEPackage("Summary");

JSEImport("Summary.Trad");

Summary = function () {
	this.template = document.getElementById('summaryTpl').innerHTML;
	this.container = document.getElementById('summaryContainer');
	this.advertising = document.getElementById('advertising');
};

Summary.prototype = {
	advertisingBaseUrl: 'http://download.viamichelin.com/mobile/static/ad/vmm/',
	summaryDefaultData: {
		title: '',
		vehiculeType: 0,
		time: {
			total: 0,
			motorway: 0,
			traffic: 0
		},
		distance: {
			total: 0,
			motorway: 0,
			unit: 'km'
		},
		cost: {
			total: 0,
			toll: 0,
			ccz: 0,
			currency: 'EUR'
		},
		departure: 'Departure',
		arrival: 'Arrival'
	},

	init: function (conf) {
		this.lang = conf.lang;
		this.device = conf.device;
		this.model = conf.model;
		this.useExternalLinksInAd = conf.useExternalLinksInAd;
	},

	clean: function () {
		this.container.innerHTML = "";
	},
	render: function (summaryData, networkAvailable) {
		var $, that = this;
		var adIframe, currency;
		var summary = this.extend(
			this.clone(this.summaryDefaultData),
			{
				title: summaryData.title,
				vehiculeType: summaryData.vehiculeType,
				departure: summaryData.departure,
				arrival: summaryData.arrival
			}
		);
		this.extend(summary.time, summaryData.time || {});
		this.extend(summary.distance, summaryData.distance || {});
		this.extend(summary.cost, summaryData.cost || {});

		if (typeof networkAvailable === 'undefined') {
			networkAvailable = true;
		}

		switch (summary.cost.currency) {
		case ('EUR') : currency = '&euro;'; break;
		case ('GBP') : currency = '&pound;'; break;
		case ('USD') : currency = '&dollar;'; break;
		default: currency = summary.cost.currency;
		}

		this.clean();

		this.container.innerHTML = this.template;
		$ = function (selector) { return that.container.querySelector.call(that.container, selector) };

		// Data
		$('#iti-bis-desc-title').innerHTML = summaryData.title || '';
		// Time
		$('#iti-bis-desc-time-total').innerHTML = this.formatTime(summary.time.total);
		$('#iti-bis-desc-time-motorway').innerHTML = this.formatTime(summary.time.motorway);
		$('#iti-bis-desc-time-traffic').innerHTML = this.formatTime(summary.time.traffic);
		// Distance
		$('#iti-bis-desc-dist-total').innerHTML = this.formatDist(summary.distance.total, summary.distance.unit);
		$('#iti-bis-desc-dist-motorway').innerHTML = this.formatDist(summary.distance.motorway, summary.distance.unit);
		// Cost
		$('#iti-bis-desc-cost-total').innerHTML = this.formatCost(summary.cost.total, currency);
		$('#iti-bis-desc-cost-toll').innerHTML = this.formatCost(summary.cost.toll, currency);
		$('#iti-bis-desc-cost-ccz').innerHTML = this.formatCost(summary.cost.ccz, currency);


		if (summary.vehiculeType !== 0 && summary.vehiculeType !== 4) {
			// on foot or bicycle
			this.hide($('#iti-bis-time-motorway'));
			this.hide($('#iti-bis-time-traffic'));
			this.hide($('#iti-bis-dist-motorway'));
			this.hide($('#iti-bis-desc-cost'));
		} else {
			summary.time.traffic <= 60 && this.hide($('#iti-bis-time-traffic'));
			summary.cost.ccz <= 0 && this.hide($('#iti-bis-cost-ccz'));
			if (summary.time.motorway <= 0 && summary.distance.motorway <= 0) {
				this.hide($('#iti-bis-time-motorway'));
				this.hide($('#iti-bis-dist-motorway'));
				this.hide($('#iti-bis-cost-toll'));
			}
		}

		//Trad
		traduction = Summary.Trad[this.lang];
		for (id in traduction) {
			if (traduction.hasOwnProperty(id)) {
				$('#' + id).innerHTML = traduction[id];
			}
		}

		this.container.style.opacity = 1;

		// Ad
		if (networkAvailable) {
			adIframe = document.createElement('iframe');
			adIframe.src = this.advertisingBaseUrl +
				"?distance=" + window.encodeURIComponent(summary.distance.total) +
				"&distanceUnit=" + window.encodeURIComponent(summary.distance.unit) +
				"&currency=" + window.encodeURIComponent(summary.cost.currency) +
				"&departure=" + window.encodeURIComponent(JSON.stringify(summary.departure)) +
				"&arrival=" + window.encodeURIComponent(JSON.stringify(summary.arrival)) +
				"&lang=" + window.encodeURIComponent(this.lang) +
				"&vehicule=" + window.encodeURIComponent(summary.vehiculeType) +
				"&device=" + window.encodeURIComponent(this.device) +
				"&model=" + window.encodeURIComponent(this.model) +
				"&fuelCost=" + window.encodeURIComponent((summary.cost.total - summary.cost.ccz - summary.cost.toll)) +
				(this.useExternalLinksInAd ? "&useExternal=true" : "");

			while (this.advertising.firstChild) {
				this.advertising.removeChild(this.advertising.firstChild);
			}
			this.advertising.appendChild(adIframe);
			this.advertising.style.display = "";
		} else {
			// Remove ad container !
			this.advertising.style.display = "none";
		}
	},

	formatDist: function (distance, distanceUnit) {
		formatedDistance = distance >= 9.99 ? distance.toFixed(0) : distance.toFixed(2);
		
		return formatedDistance + ' ' + distanceUnit;
	},
	
	formatTime: function (seconds) {
		var hours = Math.floor(seconds / 3600);
		var minutes = Math.floor((seconds - hours * 3600) / 60);
		
		return this.pad(hours) + 'h' + this.pad(minutes);
	},
	
	formatCost: function (cost, currency) {
		var units = Math.floor(cost);
		var cents = Math.round( (cost - units) * 100 );
		
		return units + '.' + this.pad(cents) + ' ' + currency;
	},
	
	/**
	* Pad n with zero : 1 => 01, 9 => 09, 10 => 10
	*/
	pad: function (n) {
		return (n >= 0 && n < 10 ? '0' : '') + n;
	},


	/**
	* Extend the destination object with the properties of the source one
	*/
	extend: function (destination, source) {
		var property;
		for (property in source) {
			destination[property] = source[property];
		}
		return destination;
	},

	clone: function (object) {
		var theClone = {};
		this.extend(theClone, object);
		return theClone;
	},

	hide: function (element) {
		if (element && element.style) {
			element.style.display = 'none';
		}
	}
};
