/*
JSE - JavaScript Engine < http://www.jsengine.org >
Copyright (c) 2006-2011 Stephane Maccari < http://stephane.maccari.fr >

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

Credits:
	- Browser detection taken from MooTools (c) 2007 Valerio Proietti
*/


//Default path to Javascript Packages
if (typeof(JSEPath) == "undefined") JSEPath = "";

/*
 * Object augmentation
 * 	- Inheritance	
 * 		- cast(string JSEtype)
 * 		- extend(object,n arguments) 
 * 	- Events & Wires
 * 		- plugWire(JSEwire)
 * 		- unplugWire(JSEwire)
 * 		- stimulate(string event,n arguments)
 * 	- Object isolation
 * 		- isolate()
 * 	- Object preparation 
 * 		- prepare() 		
 */

JSEObject = function() {
	
};

JSEObject.prototype.cast = function(JSEtype)
{
	// Returns an object containing source object members 
	// augmented by JSEtype instanciated object members
	var result = new (eval(JSEtype))();
	var item;
	for (item in this) 
	{
		result[item] = this[item];
	}
	return result;
};
JSEObject.prototype.extend = function(object)
{
	// Prototype ou Object inheritance
	var source = null;
	if (typeof(object) != "function") source = object;
	if (object.prototype) source = object.prototype;
	
	if (source != null) {
		var item;
		for (item in source) 
		{
			if (!this[item]) this[item] = source[item];
		}
	}
	
	if (typeof(object) == "function") {
		// Constructor inheritance, applying arguments
		if (arguments.length == 1) object.call(this);
		else object.apply(this,  Array.prototype.slice.apply(arguments, [1]));
	}
};
JSEObject.prototype.isolate = JSEObject.prototype.removeEventListeners = function()
{
	// Delete all listening wires
	if (this.JSEwireHub) {
		for (var item in this.JSEwireHub) 
		{
			if ((typeof(this.JSEwireHub[item]) == "object")&&(!JSEObject.prototype[item])) {
				while (this.JSEwireHub[item].length > 0)
				{
					try {
						this.unplugWire(this.JSEwireHub[item][0]);
					} catch(e) {}
				}
			}
		}
	}
	// Delete thisObj references from JSEWires in external JSEWireHubs
	if (this.JSEExtRef) {
		for (var i=0; i < this.JSEExtRef.length; i++)
		{
			var obj = this.JSEExtRef[i];
			if (obj.JSEwireHub) {
				for (var item in obj.JSEwireHub) 
				{
					if ((typeof(obj.JSEwireHub[item]) == "object")&&(!JSEObject.prototype[item])) {
						for (var k=0; k < obj.JSEwireHub[item].length; k++)
						{
							try {
								if (obj.JSEwireHub[item][k].thisObj==this) {
									obj.unplugWire(obj.JSEwireHub[item][k]);
									k--;
								}
							} catch(e) {}
						}
					}
				}
			}
		}
		
		this.JSEExtRef = new Array();
	}
};
JSEObject.prototype.plugWire = JSEObject.prototype.addEventListener = function(wire)
{
	wire = JSE.buildWire.apply(JSE,arguments);
	if ((wire != null) && (wire.method)) {
		// own JSEWireHub
		if (!this.JSEwireHub) this.JSEwireHub = new Array();
		if (!this.JSEwireHub[wire.event]) this.JSEwireHub[wire.event] = new Array();
		this.JSEwireHub[wire.event].push(wire);
		
		// JSEExtRef on destination thisObj (for isolation)
		if (wire.thisObj) {
			try {
			if (!wire.thisObj.JSEExtRef) wire.thisObj.JSEExtRef = new Array();
			wire.thisObj.JSEExtRef.push(this);
			} catch(e) {
			}
		}
	} 
};
JSEObject.prototype.unplugWire = JSEObject.prototype.removeEventListener = function(wire)
{
	wire = JSE.buildWire.apply(JSE,arguments);
	if (wire != null) {
		// JSEWire deletion in own JSEWireHub
		if (!this.JSEwireHub) this.JSEwireHub = new Array();
		if (this.JSEwireHub[wire.event]) 
		{
			for (var i=0; i < this.JSEwireHub[wire.event].length; i++)
			{
				if ((wire.method == this.JSEwireHub[wire.event][i].method)&&((wire.thisObj == this.JSEwireHub[wire.event][i].thisObj))) 
				{
						this.JSEwireHub[wire.event].splice(i,1);
						i--;
				}
			}
		}
		// JSEWire deletion in the JSEExtRef of the destination thisObj (for isolation)
		if ((wire.thisObj)&&(wire.thisObj.JSEExtRef)&&(JSE.arrayIndexOf(wire.thisObj.JSEExtRef, this) != -1)) {
			wire.thisObj.JSEExtRef.splice(JSE.arrayIndexOf(wire.thisObj.JSEExtRef, this),1);	
		}
	}
};
JSEObject.prototype.stimulate = JSEObject.prototype.triggerEvent = function(strEvent)
{
	if (!this.JSEwireHub) this.JSEwireHub = new Array();
	if (!this.JSEwireHub[strEvent]) this.JSEwireHub[strEvent] = new Array();
	if (this.JSEwireHub[strEvent]) {
		var tmpArray = new Array();
		for (var i=0; i < this.JSEwireHub[strEvent].length; i++) tmpArray.push(this.JSEwireHub[strEvent][i]);
		for (var i=0; i < tmpArray.length; i++) {
			var thisObj = ((tmpArray[i].thisObj) ? tmpArray[i].thisObj : this); 
			if (tmpArray[i].method && (typeof(tmpArray[i].method) == "function")) tmpArray[i].method.apply(thisObj, Array.prototype.slice.apply(arguments, [1]));
		}
	}
};
JSEObject.prototype.prepare = function()
{
	//Instanciate member objects and build wires
	var object = this;
	if ((JSEObject.prototype.prepare.caller != JSE.componentLoaded)&&(JSEObject.prototype.prepare.caller)&&(JSEObject.prototype.prepare.caller.prototype)) object = JSEObject.prototype.prepare.caller.prototype;
	var _wireToTreat = new Object();
	if (!this._isPrepared) {
		var item;
		for (item in object)
		{
			//Member objects instanciation
			if ((object[item] != null)&&(item != "JSEwireHub")&&(item != "JSEEvents")) {
				if (object[item].JSEtype || object[item].JSEType) 
				{
					var wires = object[item].JSEwires || object[item].JSEEvents;
					var hint = object[item].JSEhint;
					var init = object[item].JSEinit;
					var type = object[item].JSEtype || object[item].JSEType;
					var typeParams = object[item].JSEtypeParams || object[item].JSETypeParams;
					if (typeParams) {
						var tmpParams = new Array();
						for (var i=0; i < typeParams.length; i++)
						{
							if ((typeof(typeParams[i]) == "string")&&(typeParams[i].indexOf("JSEInstance") != -1)) {
								var members = typeParams[i].split(".");
								var member = this;
								for (var j=1; j < members.length; j++) {
									member = member[members[j]];
								}
								tmpParams[i] = member;
							} else tmpParams[i] = typeParams[i];
						}
						typeParams = tmpParams;
						this[item] = new JSEObject();
						typeParams.unshift(eval(type));
						this[item].extend.apply(this[item], typeParams);
					} else {
						try{
							if (typeof(eval(type)) == "function")
								this[item] = new (eval(type))();
							else {
								var obj = (eval(type));
								if(typeof(obj) != "undefined")
									this[item] = obj;
								else
									throw type+" is undefined";
							}
						} catch (e) {
							JSE.stimulate("onObjectPreparationError", ((!navigator.isIE) ? e.toString() : e.message));
						}
					}
					this[item].JSEhint = hint;
					if (wires) {
						//Wiring preparation
						object[item].JSEwires = wires;
						_wireToTreat[item] = object[item];
					}
					if (init) {
						this[item].init.apply(this[item],init);
					}
					
				}
			}
			
			
		}
		//Member objects pre-wiring
		if (_wireToTreat)
		{
			for (item in _wireToTreat)
			{
				if (_wireToTreat[item].JSEwires) 
				{
					var wires = _wireToTreat[item].JSEwires;
					
					for (var i=0; i < wires.length; i++) {
						var wire = new JSEWire(wires[i].event, wires[i].method, wires[i].thisObj);
						if (typeof(wire.method) == "string") {
							var members = wire.method.split(".");
							if (members[0] == "JSEInstance") {
								var method = this;
								for (var j=1; j < members.length; j++) {
									method = method[members[j]];
								}
								wire.method = method;
							}
							else wire.method = eval(wire.method);
						}
						if (typeof(wire.thisObj) == "string") {
							var members = wire.thisObj.split(".");
							if (members[0] == "JSEInstance") {
								var thisObj = this;
								for (var j=1; j < members.length; j++) {
									thisObj = thisObj[members[j]];
								}
								wire.thisObj = thisObj;
							}
							else wire.thisObj = eval(wire.thisObj);
						}
						this[item].plugWire(wire);
					}
				}
			}
			
		}
		//Auto wiring
		if (object["JSEwireHub"] || object["JSEEvents"]) {
			var item = "JSEwireHub";
			if (object["JSEEvents"] && !object["JSEwireHub"]) item = "JSEEvents";
			var tmpHub = new Array();
			var hubItem;
			for (hubItem in object[item]) {
				if (!JSEObject.prototype[item])
				{
					tmpHub[hubItem] = new Array();
					for (var i=0; i < object[item][hubItem].length; i++)
					{
						if (object[item][hubItem][i] && object[item][hubItem][i].method) {
							var wire = new JSEWire( object[item][hubItem][i].event,  object[item][hubItem][i].method,  object[item][hubItem][i].thisObj);
							if (typeof(wire.method) == "string") {
								var members = wire.method.split(".");
								if (members[0] == "JSEInstance") {
									var method = this;
									for (var j=1; j < members.length; j++) {
										method = method[members[j]];
									}
									wire.method = method;
								}
								else wire.method = eval(wire.method);
							}
							if (wire.method) {
								if (typeof(wire.thisObj) == "string") {
									var members = wire.thisObj.split(".");
									if (members[0] == "JSEInstance") {
										var thisObj = this;
										for (var j=1; j < members.length; j++) {
											thisObj = thisObj[members[j]];
										}
										wire.thisObj = thisObj;
									}
									else wire.thisObj = eval(wire.thisObj);
								}
								tmpHub[hubItem].push(wire);
							}
						}
					}
				}
			}
			this.JSEwireHub = tmpHub;
		}
	}
};



/*
 * JSE System Object  
 * 
 */

JSE = {
	JSEwireHub : {
		"onLoadingNamespace" : [],
		"onNamespaceLoadingError" : [],
		"onNamespaceLoaded" : [],
		"onImportDone" : [],
		"onInit" : [],
		"onInitError" : [],
		"onObjectPreparationError" : []
	},
	version : "1.75",
	debug : false,
	namespaces : [],
	currentPackMaster : null,
	lastNamespacePrepared : 0,
	namespaceLoadIndex : 0,
	isLoadingApplication : false,
	applicationConfiguration : null,
	applicationWires : null,
	nextApplications : [],
	arrayIndexOf : function(source, match)
	{
		for (var i=0; i < source.length; i++)
		{
			if (source[i] == match) return i;
		}
		return -1;
	},
	systemInit : function()
	{
		//Init the JSELaunched application
		try {
			if (JSE.currentJSEPath != null) {
				JSEPath = JSE.currentJSEPath;
				JSE.currentJSEPath = null;
			}
			var loadedApplication = null;
			//Create a new instance of the JSELaunched application if needed
			if (typeof(eval(this.currentPackMaster)) == "function") loadedApplication = eval("new "+this.currentPackMaster+"()");
			else loadedApplication = eval(this.currentPackMaster);
			//Callbacks wiring
			if (this.applicationWires) {
				if ((typeof(this.applicationWires) == "function")||(this.applicationWires.method && this.applicationWires.event && this.applicationWires.event == "onInit")) {
					//Case one single onInit JSEWire
					var method = this.applicationWires.method || this.applicationWires;
					JSE.addEventListener("onInit", method, this.applicationWires.thisObj);
				} else {
					//Case list of wiring items
					for (var item in this.applicationWires)
					{
						
						if ((item != "onInit")&&(item != "onInitError")) {
							if (typeof(this.applicationWires[item]) == "function")
								loadedApplication.addEventListener(item, this.applicationWires[item]);
							else if (typeof(this.applicationWires[item]) != "undefined" && this.applicationWires[item].method) 
								loadedApplication.addEventListener(item, this.applicationWires[item].method, this.applicationWires[item].thisObj);
						} else {
							if (typeof(this.applicationWires[item]) == "function")
								this.addEventListener(item, this.applicationWires[item]);
							else if (typeof(this.applicationWires[item]) != "undefined" && this.applicationWires[item].method) 
								this.addEventListener(item, this.applicationWires[item].method, this.applicationWires[item].thisObj);
						}
					}
				}
			}
			//Call init method of the JSELaunched application
			if (loadedApplication.init) loadedApplication.init(this.applicationConfiguration);		
			
			this.applicationWires = null;
			this.applicationConfiguration = null;
			JSE.currentPackMaster = null;
			this.isLoadingApplication = false;
			//Stimulate the onInit JSE event
			this.triggerEvent("onInit", loadedApplication);
			//loadedApplication.triggerEvent("onInit", loadedApplication);
			
		} catch (e) {
			this.applicationConfiguration = null;
			JSE.currentPackMaster = null;
			this.isLoadingApplication = false;
			//Stimulate the onInitError JSE event
			this.triggerEvent("onInitError", loadedApplication, e);
		}
	},
	loadComponent : function(namespace)
	{
		this.namespaces.push(namespace);
		this.appendComponent(false);
	},
	appendComponent : function(isFromComponentLoad)
	{
		if (isFromComponentLoad) {
		try {
			if ((eval("typeof("+this.namespaces[this.namespaceLoadIndex].name+")") != "undefined")&&(!eval(this.namespaces[this.namespaceLoadIndex].name+".JSE"))) {
				JSE.namespaceLoadIndex++;
				if (JSE.namespaceLoadIndex < JSE.namespaces.length) JSE.appendComponent(true);
				return;
			}
		} catch (e)
		{
			
		}}
		
		if ((!this.isLoadingModule)||((isFromComponentLoad)))	
		{
			this.isLoadingModule = true;
			var domScript = document.createElement("script");
			domScript.setAttribute("type","text/javascript");
			if (navigator.isIE) domScript.onreadystatechange = JSE.componentReady;
			else domScript.onload = JSE.componentLoaded;
			domScript.onerror = JSE.componentLoadingError;
			try {
				if (eval("typeof("+this.namespaces[this.namespaceLoadIndex].name+") != 'undefined'")) {
					var JSEClone = eval(this.namespaces[this.namespaceLoadIndex].name);
					domScript.JSEClone = JSEClone;
				}
			} catch (e) {
				
			}
			domScript.setAttribute("src",this.namespaces[this.namespaceLoadIndex].url+((JSE.debug) ? "?debug="+(new Date()).getTime() : ""));
			this.stimulate("onLoadingNamespace", this.namespaces[this.namespaceLoadIndex].name);
			var body = document.getElementsByTagName("body")[0];
			var head = document.getElementsByTagName("head")[0];
			if (body) body.appendChild(domScript);
			else if (head) head.appendChild(domScript);
		}
	},
	componentLoadingError : function()
	{
		JSE.stimulate("onNamespaceLoadingError", JSE.namespaces[JSE.namespaceLoadIndex].name);
	},
	componentLoaded : function()
	{
		if (this.JSEClone) {
			var item;
			for (item in this.JSEClone) {
				eval(JSE.namespaces[JSE.namespaceLoadIndex].name)[item] = this.JSEClone[item];
			}
			this.JSEClone = "";
		}
		JSE.extend(eval(JSE.namespaces[JSE.namespaceLoadIndex].name),JSEObject);
		var obj = JSE.namespaces[JSE.namespaceLoadIndex].name.split(".");
		obj = obj[obj.length - 1];
		if (JSE.namespaces[JSE.namespaceLoadIndex].alias) eval(JSE.namespaces[JSE.namespaceLoadIndex].alias + "=" + JSE.namespaces[JSE.namespaceLoadIndex].name);
		if (eval("typeof("+obj+") == 'undefined'")) eval(obj + "=" + JSE.namespaces[JSE.namespaceLoadIndex].name);
		JSE.stimulate("onNamespaceLoaded", JSE.namespaces[JSE.namespaceLoadIndex].name);
		JSE.namespaceLoadIndex++;
		if (JSE.namespaceLoadIndex < JSE.namespaces.length) JSE.appendComponent(true);
		else {
			//Objects preparation
			for (var i=JSE.lastNamespacePrepared; i < JSE.namespaces.length; i++) {
				if (typeof(eval(JSE.namespaces[i].name)) != "function")
				eval(JSE.namespaces[i].name).prepare();
			}
			JSE.lastNamespacePrepared = JSE.namespaces.length - 1;
			//Controller initialization
			JSE.isLoadingModule = false;
			JSE.triggerEvent("onImportDone");
	
		}
		this.onload = "";
	},
	componentReady : function()
	{
		if ((this.readyState == "loaded")||(this.readyState == "complete")) {
			this.onreadystatechange = "";
			JSE.componentLoaded.call(this);
		}
	},
	hasNamespace : function(namespace)
	{
		for (var i=0; i < this.namespaces.length; i++) {
			if (this.namespaces[i].equals(namespace)) return true;
		}
		//For non JSEImported objects
		try {
			if ((typeof(eval(namespace.name)) != "undefined")&&(!eval(namespace.name).JSE)) return true;
		} catch (e) {
			return false;
		}
		return false;
	},
	extend : function(src, dest)
	{
		var item;
		if (src.prototype) src = src.prototype;
		for (item in dest.prototype) {
			if (typeof(src[item]) == "undefined") src[item] = dest.prototype[item];
		}
	},
	buildWire : function()
	{
		if ((arguments.length == 1)&&(arguments[0].method)) return arguments[0];
		if (arguments.length > 1) {
			return new JSEWire(arguments[0], arguments[1], arguments[2]);
		}
		return null;
	},
	alertMode : function()
	{
		JSE.unplugWire("onInitError", this._alert);
		JSE.plugWire("onInitError", this._alert);
	},
	_alert : function(a,b)
	{
		alert(b.message);
	}
};
JSE.extend(JSE, JSEObject);

JSE.addEventListener("onImportDone", JSE.systemInit, JSE);




function JSENamespace(name, url, alias)
{
	this.name = name;
	var basePath = this.name.replace(/\./g,"/");
	basePath += "/" + this.name.split(".")[this.name.split(".").length - 1] + ".js";
	if ((url)&&(url != "")) this.url = url + basePath;
	else this.url = JSEPath + basePath;
	if (alias) this.alias = alias;
}
JSENamespace.prototype.equals = function(namespace)
{
	if (this.name == namespace.name) return true;
	return false;
}; 

function JSEImport(packageName, packageURL, aliasName)
{
	if (JSE.currentPackMaster == null) JSE.currentPackMaster = packageName;
	var namespace = new JSENamespace(packageName, packageURL, aliasName);
	if (!JSE.hasNamespace(namespace)) JSE.loadComponent(namespace);
}
//JSEPackage
function JSEPackage(name)
{
	var names = name.split(".");
	var object = window;
	for (var i=0; i < names.length; i++) { 
            if (!object[names[i]]) { 
                    object[names[i]] = new JSEObject(); 
                    object[names[i]].JSE = true;
            } 
            object = object[names[i]]; 
    } 
}
function JSEWire(event, method, thisObj)
{
	this.event = event;
	this.method = method;
	this.thisObj = thisObj;
}
function JSEEvent(event, method, thisObj)
{
	this.event = event;
	this.method = method;
	this.thisObj = thisObj;
}


function $_id(id)
{
	return document.getElementById(id);
}

/*
 * JSE Application launcher
 * 
 * 		- JSELaunch(object, configuration object, callBackWires, rootURL)
 */
function JSELaunch(application, conf, callBackWires, rootURL)
{
	if (!JSE.isLoadingApplication) {
		JSE.isLoadingApplication = true;
		JSE.JSEwireHub["onInit"] = new Array();
		JSE.JSEwireHub["onInitError"] = new Array();
		if (callBackWires) {
			/*if (typeof(callBackWire) == "function") JSE.plugWire(new JSEWire("onInit", callBackWire));
			else {
				JSE.plugWire(callBackWire);
			}*/
			JSE.applicationWires = callBackWires;
			
		}
		if (conf) JSE.applicationConfiguration = conf;
		else JSE.applicationConfiguration = null;
		JSE.currentPackMaster = application;
		
		var namespace = new JSENamespace(application);
		if (!JSE.hasNamespace(namespace)) {
			JSE.currentJSEPath = JSEPath;
			
			if (rootURL) JSEPath = rootURL;
			JSEImport(application);
		}
		else JSE.systemInit();
		
	} else {
		JSE.nextApplications.push([application, conf, callBackWires, rootURL]);
		JSE.plugWire(new JSEWire("onInit",function() {
			
			if (JSE.nextApplications.length > 0) {
				var tmp = JSE.nextApplications[0];
				JSE.nextApplications.splice(0,1);
				JSELaunch(tmp[0], tmp[1], tmp[2], tmp[3]);
			}	
		},JSE));
	}
} 

/*
 * Browser detection
 * 
 * 		- navigator.isIE
 * 		- navigator.isIE6
 * 		- navigator.isIE7
 * 		- navigator.isSafari
 * 		- navigator.isSafari2
 * 		- navigator.isSafari3
 * 		- navigator.isMozilla
 * 		- navigator.isOpera
 * 		- navigator.isJSECompatible
 */
window.xpath = !!(document.evaluate);
//if (window.ActiveXObject && !window.opera) navigator.isIE = navigator[window.XMLHttpRequest ? 'isIE7' : 'isIE6'] = true;
if (document.childNodes && !document.all && !navigator.taintEnabled) navigator.isSafari = navigator[window.xpath ? 'isSafari3' : 'isSafari2'] = true;
else if (document.getBoxObjectFor != null) navigator.isMozilla = true;
else if (window.mozInnerScreenX != null) navigator.isMozilla = true;
navigator.isOpera = window.opera;
navigator.isChrome = navigator.userAgent.toLowerCase().indexOf('chrome') > -1;

// Test IE
navigator.isIE = navigator.isIE6 = navigator.isIE7 = navigator.isIE8 = navigator.isIE9 = false;
var msie = (/MSIE/i).test(navigator.userAgent);
var versionIE = navigator.userAgent.match(/MSIE (\d+)./i);
if (msie) {
	navigator.isIE = navigator["isIE"+versionIE[1]] = true;
}

navigator.isJSECompatible = navigator.isIE || navigator.isMozilla || navigator.isOpera || navigator.isSafari3 || navigator.isChrome;

//////////////////////////////
//Add-ons By VM
//////////////////////////////

function $isNotEmpty(object){
	if (typeof(object) == "undefined"){
		return false;
	}
	if (object == null) {
		return false;
	}
	if (typeof(object) == "string") {
		if (object.replace(/^\s+/,'').replace(/\s+$/,'') == "" || object == "null") {
			return false;
		}
	}
	return true;
}

function $isEmpty(object){
	return !$isNotEmpty(object);
}

/* 
 * Recopie en provenance du JSE Unique
 * Détermine si une variable est utilisable ou non 
 */
function $isDefined(object){
	return ((typeof(object) != "undefined") && (object != null));
}

function $isTrue(object){
	return ((typeof(object) == "boolean") && object);
}

function $isFalse(object){
	return ((typeof(object) == "boolean") && !object);
}

function $intVal(str) {
	var reponse;
    if( typeof(str) == "boolean"){
    	reponse = (str) ? 1 : 0;
    } else if(typeof(str) == "string"){
    	reponse = str * 1;
    	reponse = parseInt(str, 10);
    	if(isNaN(reponse) || !isFinite(reponse)){
        	reponse =  0;
        }
    } else if(typeof(str) == 'number'){
        reponse = str;
    } else{
        reponse = 0;
    }
    return reponse;
}

function $isArray(object) {
    return (object instanceof Array);
}

function $isDomElement(object) {
	return object && object.tagName && object.nodeName && object.ownerDocument && object.removeAttribute;
}

/**
 * Remplace "str" par "replacement" dans la chaine de caractères.
 * @param {String} str
 * @param {String} replacement
 */
String.prototype.replaceAll = function(str, replacement)
{
	return this.split(str).join(replacement);
};
/*
 * Array augmentation
 * 		
 * 		- indexOf(object)
 * 
 */
Array.prototype.indexOf = function(obj)
{
	for (var i=0; i < this.length; i++)
	{
		if (this[i] == obj) return i;
	}
	return -1;
};


ToJSEObject = function(object) {
	JSE.extend(object, JSEObject);
};

/* Convenient method to import trads. */
JSEImportTrad = function (str) {
	JSEImport(str, ViaMichelin.Api.Constants.System.ApiTradPath);
};

JSEImportApi = function (str) {
	JSEImport(str, ViaMichelin.Api.Constants.System.ApiJSEPath);
};
/*
if(window && typeof(window.console) == "undefined"){
	window.console = {
		log : function(value){}
	};
}
var navigatorHost = location.hostname;
if (navigatorHost.indexOf("127.0.0.1") < 0 && navigatorHost.indexOf("localhost") < 0){
	window.console.log = function(value){};
}*/