JSEPackage("ViaMichelin.Api.Constants.Itinerary");

/**
 * Constantes d'Itineraire
 * @class ViaMichelin.Api.Constants.Itinerary
 */
ViaMichelin.Api.Constants.Itinerary = {
	/** 
	 * Unités pour le choix d'un itinéraire.
	 * <ul>
 	 * 	<li><b>Details :</b></li>
 	 * 	<li>METERS : Indique que l'unité est en mètre.
 	 *  <li>MILES : Indique que l'unité est en miles.</li>
 	 * </ul> 
	 * @type Object
	 */
	DIST_UNIT		: {
		METERS			: "m",
		MILES			: "mi"
	},
	/** 
	 * CURRENCY 
	 * <ul>
 	 * 	<li><b>Details :</b></li>
 	 *  <li>EUR</li>
 	 *  <li>CHF</li>
 	 *  <li>CZK</li>
 	 *  <li>DKK</li>
 	 *  <li>GBP</li>
 	 *  <li>HRK</li>
 	 *  <li>HUF</li>
 	 *  <li>NOK</li>
 	 * 	<li>PLN</li>
 	 *  <li>SEK</li>
 	 *  <li>SIT</li>
 	 *  <li>SKK</li>
 	 *  <li>UAH</li>
 	 *  <li>USD</li>
 	 * </ul> 
	 * @type Object
	 * */
	CURRENCY		: {
		EUR : "EUR",
		CHF : "CHF",
		CZK : "CZK",
		DKK : "DKK",
		GBP : "GBP",
		HRK : "HRK",
		HUF : "HUF",
		NOK : "NOK",
		PLN : "PLN",
		SEK : "SEK",
		SIT : "SIT",
		SKK : "SKK",
		UAH : "UAH",
		USD : "USD"
	},
	/** 
	 * DATA
	 * <ul>
 	 * 	<li><b>Details :</b></li>
 	 *  <li>HEADER</li>
 	 *  <li>ROADSHEET</li>
 	 *  <li>GEOM</li>
 	 *  <li>MAP</li>
 	 * </ul>
 	 * @type Object
	 */
	DATA			: {
		HEADER			: "header",
		ROADSHEET		: "roadsheet",
		GEOM			: "geom",
		MAP				: "map"
	},
	/**
	 * Type de carburant
	 * <ul>
 	 * 	<li><b>Details :</b></li>
 	 *  <li>PETROL</li>
 	 *  <li>LPG</li>
 	 *  <li>DIESEL</li>
 	 * </ul>
	 * @type {Object}
	 */
	FUEL_TYPE : {
		PETROL  : "TYPE PETROL",
		LPG		: "TYPE LPG",
		DIESEL	: "TYPE DIESEL"
	},
	/** 
	 * Type d'itinéraire 
	 * <ul>
 	 * 	<li><b>Details :</b></li>
 	 *  <li>RECOMMENDED : Conseillé par Michelin.</li>
 	 * 	<li>QUICK : Le plus rapide.</li>
 	 *  <li>SHORT : Le plus court.</li>
 	 *  <li>DISCOVERY : Le plus touristique.</li>
 	 *  <li>ECONOMIC : Le plus économique.</li>
 	 * </ul> 
	 * @type Object
	 */
	ITINERARY_TYPE	: {
		RECOMMENDED		: 0,
		QUICK			: 1,
		SHORT			: 2,
		DISCOVERY		: 3,
		ECONOMIC		: 4
	},
	/**
	 * STEP_TYPE
	 * <ul>
 	 * 	<li><b>Details :</b></li>
 	 *  <li>COORDS</li>
 	 *  <li>POI</li>
 	 *  <li>LOCID</li>
 	 * </ul>
	 * @type Object
	 */
	LOCATION_TYPE		: {
		COORDS				: 1,
		POI					: 2,
		LOCID				: 3
	},
	/**
	 * Nombre maximal d'étape (tout confondu : départ, (n)étapes, arrivée).
	 * @type Number
	 */
	MAX_STEPS_NUMBER : 8,
	/** 
	 * STEP_KIND
	 * <ul>
 	 * 	<li><b>Details :</b></li>
 	 *  <li>STEP_KIND_STEP</li>
 	 *  <li>STEP_KIND_CROSS</li>
 	 * </ul>
	 * @type Object
	 */
	STEP_TYPE		: {
		STEP_KIND_STEP		: "e",
		STEP_KIND_CROSS		: "p"
	},
	/** 
	 * TOLL_CAT
	 * <ul>
 	 * 	<li><b>Details :</b></li>
 	 *  <li>CAR</li>
 	 *  <li>CARAVAN</li>
 	 *  <li>MOTO</li>
 	 *  <li>PL2</li>
 	 *  <li>PL3</li>
 	 *  <li>PL4</li>
 	 *  <li>PL5</li>
 	 * </ul>
	 * @type Object
	 */ 
	TOLL_CAT		: {
		CAR 				: "car", 
		CARAVAN 			: "caravan",
		MOTO 				: "moto",
		PL2 				: "pl2",
		PL3 				: "pl3",
		PL4 				: "pl4",
		PL5 				: "pl5"
	},
	/**
	 * Catégoris de transport
	 * <ul>
	 * <li>ALL : toute catégorie de véhicule</li>
	 * <li>PUBLIC : transport en commun</li>
	 * <li>COMMODITY : transport de marchandise</li>
	 * <li>EXPLOSIVES : transport de produits explosifs</li>
	 * <li>POLLUTANTS : transport de produits polluants</li>
	 * <li>DANGEROUS_GOODS : transport de produits dangereux</li>
	 * </ul>
	 */
	TRUCK_CATEGORY_TRANSPORT : {
		"ALL"				: 0,
		"PUBLIC"			: 4,
		"COMMODITY"			: 5,
		"EXPLOSIVES"		: 6,
		"POLLUTANTS"		: 7,
		"DANGEROUS_GOODS"	: 8
	},
	
	/** 
	 * Type de véhicule pour un itinéraire.
	 * <ul>
 	 * 	<li><b>Details :</b></li>
 	 *  <li>CAR : Itinéraire en voiture.</li>
 	 * 	<li>TRUCK : Itinéraire en camion.</li>
 	 *  <li>ON_FOOT : Itinéraire à pied.</li>
 	 *  <li>CYCLE : Itinéraire en vélo.</li>
 	 * </ul> 
	 * @type Object
	 */
	VEHICULE_TYPE	: {
		CAR				: 0,
		TRUCK			: 1,
		ON_FOOT			: 2,
		CYCLE			: 3,
		MOTO			: 4
	}
};
