JSEPackage("ViaMichelin.Api.Itinerary.RoadSheetView");

JSEImportApi("ViaMichelin.Api.Util");
JSEImportApi("ViaMichelin.Api.Util.HTML");
JSEImportApi("ViaMichelin.Api.Util.Other");
JSEImportApi("ViaMichelin.Api.UI.Drawing");
JSEImportApi("ViaMichelin.Api.UI.Drawing.ItineraryBuilder");

/**
 * @class ViaMichelin.Api.Itinerary.RoadSheetView
 * @param {ViaMichelin.Api.Model.Itinerary.Roadsheet} roadsheet
 */
ViaMichelin.Api.Itinerary.RoadSheetView = function(roadsheet){
	this._roadsheet = roadsheet;
	this.uniqueIdSuffix = (new Date()).getTime() + '' + parseInt(Math.random()*1000000);
};

ViaMichelin.Api.Itinerary.RoadSheetView.prototype = {
	_withSchema : null,
	_graphic	: null,
	_roadsheet	: null,
	_schemaHeight : 300,
	_display 	: null,
	_orientation: null,
	_version	: null,
	_customizedPictos : null,
	/**
	 * 3 modes possibles : 
	 * - "NORMAL"
	 * - "NAVIGATION"
	 * - "SCHEMA"
	 * @type String
	 */
	_mode : "NORMAL",
	/**
	 * Défini les éléments a afficher sur la feuille de route.
	 * Ensemble de clé/valeur
	 * <ul>
	 * <li>radar 	: les radars</li>
	 * <li>toll		: les péages</li>
	 * <li>proximity: les lieux à proximité</li>
	 * <li>bridge	: les ponts</li>
	 * <li>tunnel	: les tunnels</li>
	 * <li>security : comprends les limitations, les radars, les pentes</li>
	 * <li>arrow	: les fleches de direction</li>
	 * <li>map		: les cartes</li>
	 * </ul>
	 * @type Object
	 */
	customItemsDrawable: {
		"radar":true,"toll":true,"proximity":true,"bridge":true,"tunnel":true,"security":true,"arrow":true,"map":false
	},
	/**
	 * @description
	 * Dessine la feuille de route + schema <br/>
	 * <b>Details :</b>
	 * <ul>
	 * 		<li><span class="light">{HTMLElement}</span> <b>container</b> : container de la feuille de route. </li>
	 * 		<li><span class="light">{Boolean}</span> <b>withSchema</b>	: avec/sans schema (seulement pour le mode NORMAL).</li>
	 * 		<li><span class="light">{String}</span> <b>mode</b>	: mode utilisé pour le rendu (SCHEMA, NORMAL, NAVIGATION).</li>
	 * 		<li><span class="light">{String}</span> <b>display</b>		: impression/web.</li>
	 * 		<li><span class="light">{Object}</span> <b>customizedItems</b> : affichage/masquage de certains éléments graphique.
	 * 		<br/>Voir ViaMichelin.Api.Itinerary.RoadSheetView.customItemsDrawable
	 * 		</li>
	 * </ul>
	 * 
	 * @param {Object} options
	 * @return {HTMLElement} Le conteneur de la feuille de route
	 */
	addTo : function (options) {
		var target = options.container;
		this._target = target;
		var version = 2; //$isDefined(options.version) ? options.version : 1;
		this._version = version;
		this._withSchema = withSchema = $isDefined(options.withSchema) ? options.withSchema : false;
		//var orientation = $isDefined(options.orientation) ? options.orientation : "N";
		//this._orientation = orientation;
		this._display = $isDefined(options.display) ? options.display : "WEB";
		this._mode = $isDefined(options.mode) ? options.mode : "NORMAL";
		
		for(var item in options.customizedItems){
			this.customItemsDrawable[item] = options.customizedItems[item];
		}
		
		if($isDefined(options.customizedPictos)){
			this._customizedPictos = this._roadsheet._customizedPictos = options.customizedPictos;
		}else{
			this._customizedPictos = this._roadsheet._customizedPictos;
		}
		
		// Parametrage
		if(this._mode=="NORMAL"){
			this._orientation = "N";
		}
		if(this._mode=="SCHEMA"){
			this._orientation = "C";
			this._withSchema = withSchema = true;
		}
		
		// Dessin de la partie textuelle
		if (this._mode=="NORMAL") {
			this._drawRoadSheet(options);
		} else if (this._mode=="SCHEMA") {
			this._drawRoadSheetCap(options);
		} else if(this._mode=="NAVIGATION"){
			this._drawNavigationRoadSheet(options);
		}
		
		// Spécial pour les éléments customisable de la feuille de route et du schema
		this._customizeRoadsheet(this.customItemsDrawable);
		
		if (this._mode=="NORMAL" && this._orientation=="N") {
			// On redimentionne la fdr en fonction du schema, des éléments visibles (radar, péage, etc.)
			this._resizeBackGround(target, this._orientation, version, withSchema);
		}
		
		// Draw schema
		//if(this._roadsheet._schema != null && version >1 ) {
		//normalement n'est plus utilisé, jusqu'à nouvel ordre
			if (this._mode=="NORMAL" && withSchema) {
				this._drawLinearSchema();
			} else if (this._mode=="SCHEMA" && this._roadsheet._schema != null){
				this._createSchema(this._orientation);
				this._drawSchema();
			}
		//}
		
		this.plugPoiCorridorClick();
		
		return target;
	},
	/**
	 * Branchement des evenements de click sur les POI CORRIDOR
	 */
	plugPoiCorridorClick : function(){
		var list = ViaMichelin.Api.Util.HTML.getElementsByClassName(this._target, "poiToClick");
		
		for(var i=0;i<list.length;i++){
			var poiDOM = list[i];
			var poiParam = poiDOM.getAttribute("param");
			var poiId 			= poiParam.split("-")[0];
			var poiProductId 	= poiParam.split("-")[1];
			var poipko 			= poiParam.split("-")[2];
			var self = this;
			
			for(var j=0; j<this._roadsheet._poiList.length; j++){
				var item = this._roadsheet._poiList[j];
				if(($isDefined(poiProductId) && item.id == poiId && item.productId == poiProductId) || (item.poipko == poipko)){
					poiDOM.poi = item;
					poiDOM.onclick = function(){
						self._roadsheet.stimulate("onClickPOI", this.poi);
						return false;
					};
					break;
				}
			}
		}
	},
	
	/**
	 * Suite à un changement de la feuille de route (ajout d'elements dans le DOM), 
	 * cette méthode permet de redessinner la feuille de route/schema
	 */
	resize : function(){
		/*
		if (this._version>1) {
			// Enleve la taille des flèches
			var list = ViaMichelin.Api.Util.HTML.getElementsByClassName(this._target, "arrowDirection");
			for(var item in list){
				list[item].style.height = "";
				// image
				var DOMImgList = ViaMichelin.Api.Util.HTML.getElementsByClassName(list[item], "arrowLine");
				if(DOMImgList.length==1){
					DOMImgList[0].style.height="";
				}
			}
		}*/
		if(this._target != null){
			this._resizeBackGround(this._target, this._orientation, this._version, this._withSchema);
			
			// Dessine le schema
			if (this._mode=="SCHEMA" || (this._mode=="NORMAL" && this._withSchema)) { //$isDefined(this._withSchema) && this._withSchema) {
				//this.clearSchema();
				if (this._orientation == "N")
					this._drawLinearSchema();
				else
					this._drawSchema();
			}
		}
	},
	/**
	 * Affiche/cache un élément de la feuille de route / schema
	 * @param {String} item
	 * @param {Boolean} value
	 */
	setCustomizeItem : function(item, value){
		if ($isDefined(item) && $isDefined(value)) {
			this.customItemsDrawable[item] = value;
			this._customizeRoadsheet(this.customItemsDrawable);
			this.resize();
		}
	},
	/**
	 * TODO: a revoir quand les spéc seront définitives.
	 * 
	 * @param {Object} element
	 * @param {String} orientation
	 * @param {int} version
	 * @param {boolean} withSchema
	 */
	_resizeBackGround : function(element, orientation, version, withSchema){
		var elementImages = this._target.getElementsByTagName("img"),
			len = elementImages.length;
		for (var i = 0; i<len; i++){
			var elementImage = elementImages[i];
			
			if (elementImage.className == "fdr_bgImg"){
				
				//Calcul de la taille de l element parent
				if(elementImage.nextSibling.nodeType == 1) {
					elementImage.style.height = elementImage.nextSibling.offsetHeight+"px";
				}
				else {
					elementImage.style.height = elementImage.nextSibling.nextSibling.offsetHeight+"px";
				}
			}
		}
		
		// Spécifique à la v2
		if(this._version > 1){
			// Alignement des puces avec le schema
			/* TODO : la validité de ce code dépendra des choix produits
			if(this._withSchema){
				var elements = ViaMichelin.Api.Util.HTML.getElementsByClassName(this._target, "fdr_stepBlock"),
					len = elements.length, i, heightTotal=0;
				
				var steps = this._roadsheet.steps, lens = steps.length, i = 0;
				
				for (var i=0; i < lens; i++) {
					var item = elements[i];
					heightTotal+= item.offsetHeight;
					
					var nextIndice;
					if(this._orientation == "C"){
						nextIndice = lens - (i+1);
					} else {
						nextIndice = i+1;
					}
					
					if(i>=1 && i<len){
						if (nextIndice < len && nextIndice>0 && $isDefined(this._roadsheet.steps[nextIndice].schema)) {
							var nextPointHeight = (this._orientation=="C") ? (this._roadsheet._schema.size[1] - this._roadsheet.steps[nextIndice].schema[0][1]) : this._roadsheet.steps[nextIndice].schema[0][1];
							var delta = nextPointHeight - heightTotal;
							
							if(delta>0){
								var blocSize = nextPointHeight - ( heightTotal - item.offsetHeight);
								
								heightTotal+= delta;
								var endElement = ViaMichelin.Api.Util.HTML.getElementsByClassName(item, "fdr_totalDistance");
								
								if (endElement){
									endElement[0].style.height = delta+20+"px";
									var child = item.firstChild;
									if (ViaMichelin.Api.Util.HTML.isClassNameExistInHTMLElement(child, "fdr_bgImg")) {
										// Modification de la taille de l'image
										item.firstChild.style.height = parseInt(item.firstChild.style.height)+delta+"px";
									}
								}
								var tableBloc = item.getElementsByTagName("table")[0];
								var realBlocSize = tableBloc.offsetHeight;
								
								var trIndice = (this._orientation=="C") ? 1 : 0;
								if(blocSize > realBlocSize){
									var firstTR = tableBloc.getElementsByTagName("tr")[trIndice];
									firstTR.style.height = (20 + blocSize - realBlocSize)+"px";
								}
							}
						}
					}
					
					// Gestion du résumé de l'étape (km)
					var stepResume = ViaMichelin.Api.Util.HTML.getElementsByClassName(item, "stepResume");
					
					if(item.offsetHeight > 62 && $isDefined(stepResume[0])){
						stepResume[0].style.display = "block";
						stepResume[0].style.top = (item.offsetHeight/2) - (stepResume[0].offsetHeight/2) +"px";
					}
				}
			}
			*/
			// Dessine les fleches
			var list = ViaMichelin.Api.Util.HTML.getElementsByClassName(this._target, "arrowDirection"),
				len = list.length;
			
			for (var i=0; i<len; i++) {
				var item = list[i];
				//	item.style.height = item.parentNode.offsetHeight+"px";
				var tableParent = item.parentNode.parentNode.parentNode;
				item.style.height = tableParent.offsetHeight+"px";
				
				var arrow = ViaMichelin.Api.Util.HTML.getElementsByClassName(item, "arrowLine");
				
				if ($isDefined(arrow[0])) {
					arrow[0].style.height = tableParent.offsetHeight+"px";
				}
			}
		}
	},
	/**
	 * Créee la feuille de route en HTML : 
	 * Utilisé pour la v1
	 * @param {Object} options
	 * @deprecated
	 */
	_getFeuilleDeRouteHTML : function(options) {
		var withMap = $isDefined(options.withMap) ? options.withMap : false;
		var display = $isDefined(options.display) ? options.display : "WEB";
		var roadsheetClass = "VMRoadSheetNew";
		
		if (display == "WEB") roadsheetClass+= " VMRoadSheetWeb";
		else roadsheetClass+= " VMRoadSheetPrint";
		
		var strHTML = '<div id="idRoadSheet'+this.uniqueIdSuffix+'" class="'+ roadsheetClass +'"><div class="roadsheetContent">';
		var steps = this._roadsheet.steps;
		
		for (var i=0; i < steps.length; i++) {
			//Definition parité
			var isEtapeImpair = (i%2 == 0 || i == 0);
			
			strHTML += '<div'+(isEtapeImpair ? ' class="fdr_stepBlock fdr_bg"' : ' class="fdr_stepBlock"')+'>'
			+ (isEtapeImpair ? '<img class="fdr_bgImg" src="' + ViaMichelin.Api.Constants.System.ApiImgPath + 'all/iti/fdrBg.png" alt="" />' : '')
			+ '<table cellpadding="0" cellspacing="0">' 
			+ '<tr>';
			if (withMap) {
				strHTML += ((isNaN(steps[i].number)||steps[i].number=="")	? "<td class='fdr_stepNumber' " + (((steps[i].details)&&(steps[i].details.length > 0)) ? "rowspan='"+(steps[i].details.length+1)+"'" : "" ) + ">"+steps[i].number+"</td>"
						: "<td class='fdr_stepNumber' " + (((steps[i].details)&&(steps[i].details.length > 0)) ? "rowspan='"+(steps[i].details.length+1)+"'" : "" ) + ">" + ViaMichelin.Api.Util.drawMiniPastilleBleue(steps[i].number) + "</td>");
			} else {
				if (isNaN(steps[i].number)||steps[i].number==""){
					
					strHTML += "<td class='fdr_stepNumber' ";
					if ((steps[i].details)&&(steps[i].details.length > 0)){
						strHTML +="rowspan='"+(steps[i].details.length+1)+"'";
					}
					var imgTxt = steps[i].number;
					if (imgTxt.indexOf("src=\'\'")!=-1){
						imgTxt = "";
					} 
					else{
						if (steps[i].type == "C" && imgTxt.indexOf("Flag")==-1){
							imgTxt = imgTxt.replaceAll("<img", "<img class='fdr_poi'");
						}
					}
					strHTML +=">"+imgTxt+"</td>";
				}
				else{
					strHTML += "<td class='fdr_stepNumber' " + (((steps[i].details)&&(steps[i].details.length > 0)) ? "rowspan='"+(steps[i].details.length+1)+"'" : "" ) + ">" + ViaMichelin.Api.Util.drawMiniPastilleBleue(steps[i].number) + "</td>";
				}
			}
			strHTML += '<td class="fdr_stepWays" ' + (((steps[i].details)&&(steps[i].details.length > 0)) ? "rowspan='"+(steps[i].details.length+1)+"'" : "" ) + ">" + steps[i].panels + '<span>&nbsp;</span></td>'
			+ '<td class="fdr_stepDetails">'+ steps[i].description + ((withMap) ? this._getMapBlock(steps[i].mapURL) : "") + '</td>'
			+ '<td class="fdr_lightGrey">'+((steps[i].distPleasant)?"<img class='itifdr18' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/tourism.png' alt='' />":"")+steps[i].partialDistance+"</td>"
			+ '<td></td>'
			+ '<td></td>'
			+ '</tr>'
			+ (((steps[i].details)&&(steps[i].details.length > 0)) ? this._getDetailHTML(steps[i].details) : "")
			+ '<tr class="fdr_totalDistance">'
			+ '<td></td>'
			+ '<td></td>'
			+ '<td></td>'
			+ '<td></td>'
			+ '<td>' + steps[i].totalDistance + '</td>'
			+ '<td>' + steps[i].totalTime + '</td>'
			+ '</tr>'
			+ '</table>'
			+ '</div>';
		}
		strHTML += '</div>';
		strHTML += '</div>';
		
		options.container.innerHTML = strHTML;
		
		return strHTML;
	},
	/**
	 * Dessine la feuille de route en mode Navigation
	 * @param {Object} options
	 */
	_drawNavigationRoadSheet : function(options){
		var container = options.container;
		var roadsheetElement = document.createElement("div");
			roadsheetElement.id = "idRoadSheet"+this.uniqueIdSuffix;
			roadsheetElement.className = "VMRoadSheetNew ";
			roadsheetElement.expando = this;
		
		container.appendChild(roadsheetElement);
	},
	/**
	 * Affiche la feuille de route v2 textuelle
	 * @param {Object} 	options
	 * @param {String} 	options.display
	 * @param {Boolean} options.withMap
	 * @param {String} 	options.orientation
	 */
	_drawRoadSheet : function(options){
		var display = $isDefined(options.display) ? options.display : "WEB";
		var container = options.container;
		var withMap = $isDefined(options.withMap) ? options.withMap : true;
		var withSchema = $isDefined(options.withSchema) ? options.withSchema : false;
		var orientation = $isDefined(options.orientation) ? options.orientation : "N";
		
		var classOrientation = "normalOrientation";
		if(orientation == "C"){
			classOrientation = "capOrientation";
		}
		
		var roadsheetElement = document.createElement("div");
			roadsheetElement.id = "idRoadSheet"+this.uniqueIdSuffix;
			roadsheetElement.className = "VMRoadSheetNew "+ classOrientation;
			roadsheetElement.expando = this;
		var roadsheetContentElement = document.createElement("div");
			roadsheetContentElement.className = "roadsheetContent";
		var schemaElement = document.createElement("div");
			schemaElement.id = "idSchemaContainer"+this.uniqueIdSuffix;
			schemaElement.className = "schemaContainer";
		
		// Determinaison d'un nom de classe unique décrivant l'affichage, à cause d'IE6 qui ne prends 
		// pas en compte la multiimbrication des classes
		var roadsheetClassName = " ";
		if (orientation == "N") roadsheetClassName+= "normal";
		else roadsheetClassName+= "cap";
		
		if (display == "WEB") roadsheetClassName+= "W"+"eb";
		else roadsheetClassName+= "Print";
		
		if (withSchema == true) roadsheetClassName+= "WithSchema";
		else roadsheetClassName+= "NoSchema";
		
		roadsheetElement.className+= roadsheetClassName;
		if (display=="PRINT") roadsheetElement.className+= " display-printable";
		
		roadsheetElement.appendChild(roadsheetContentElement);
		roadsheetElement.appendChild(schemaElement);
		container.appendChild(roadsheetElement);
		
		if(withSchema) roadsheetElement.className+= " withSchema";
		
		if(!this.customItemsDrawable["map"]) roadsheetElement.className+= " mapOff";
		
		var strHTML = "";
		var steps = this._roadsheet.steps, len = steps.length;
		
		for (var iter=0; iter < len; iter++) {
			if(orientation == "C"){
				i = len - (iter+1);
				//classOrientation = "capOrientation printableRoadsheet";
			} else {
				i = iter;
			}
			var checkpoint = steps[i];
			
			if (orientation == "C" && withSchema==true ) {
				strHTML += '<div class="fdr_stepBlock"><table cellpadding="0" cellspacing="0">';
			} else {
				//Definition parité
				var isEtapeImpair = (i%2 == 0 || i == 0);
				
				strHTML += '<div'+(isEtapeImpair ? ' class="fdr_stepBlock fdr_bg"' : ' class="fdr_stepBlock"')+'>'
				+ (isEtapeImpair ? '<img class="fdr_bgImg" src="' + ViaMichelin.Api.Constants.System.ApiImgPath + 'all/iti/fdrBg.png" alt="" />' : '')
				+ '<table cellpadding="0" cellspacing="0">';
			}
			
			var checkpointType = "";
			if (orientation == "C"){
				if (i==0) {
					checkpointType = "start";
				} else if(i==len-1){
					checkpointType = "end";
				} else checkpointType = "step";
			} else if (orientation == "N"){
				if(i==0 && checkpoint.type=="M"){
					checkpointType = "step";
				} else if(i==len-1) {
					checkpointType = "end";
				} else if(i==0 || (i==1 && checkpoint.picto=="start")){
					checkpointType = "start";
				} else checkpointType = "step";
			}
			
			strHTML += this._drawCheckpoint(checkpoint, checkpointType, display, orientation, withSchema, withMap);
			
			strHTML += '</table>'
					+ '</div>';
		}
		roadsheetContentElement.innerHTML = strHTML;
	},
	/**
	 * Dessine un checkpoint
	 * @param {Object} checkpoint
	 * @param {String} type : start, end,
	 * @param {String} display : WEB / PRINT
	 * @param {String} orientation : N / C
	 * @param {String} withSchema : true / false
	 * 
	 */
	_drawCheckpoint : function(checkpoint, type, display, orientation, withSchema, withMap){
		var strHTML = "";
		var rowspan = ((checkpoint.details)&&(checkpoint.details.length > 0)) ? checkpoint.details.length+2 : 2;
		var rowspanTxt = " rowspan=\""+ rowspan +"\"";
		
		// Traitement du bloc de début
		var classFirstTR = "", classLastTR = "";
		if (orientation == "C"){
			if (type == "start") {
				classFirstTR = "rowStart";
				classLastTR = "rowStart";
			} else if(type == "end"){
				classFirstTR = "rowEnd";
				classLastTR = "rowEnd";
			}
		} else if (orientation == "N"){
			if(type == "end") {
				classFirstTR = "rowEnd";
				classLastTR = "rowEnd";
			} else if(type == "start"){
				classFirstTR = "rowStart";
				classLastTR = "rowStart";
			}
		}
		
		var colspan = "";
		var contentfirstCol		= "";
		var contentSecondCol	= "";
		var contentThirdCol		= "";
		var contentFourthCol	= "";
		var contentFifthCol		= "";
		
		// Orientation au cap
		if (orientation == "C"){
			classFirstTR = "fdr_totalDistance";
			
			if(type=="start" || type=="end") {
				// Cas d'un checkpoint de départ.
				contentfirstCol		= "";
				contentSecondCol	= "";
				
				if (type=="start") {
					var arrowHeaderSrc = ViaMichelin.Api.Constants.System.ApiImgPath +"all/iti/direction_arrow_header_N.png";
					if (orientation == "C"){
						arrowHeaderSrc = ViaMichelin.Api.Constants.System.ApiImgPath +"all/iti/direction_arrow_header_C.png";
					}
					contentThirdCol = checkpoint.number;
				} else {
					contentThirdCol		= checkpoint.number;
				}
				
				contentFourthCol	= ""; 
				contentFifthCol		= "";
				
			} else if (type == "step") {
				// Ligne de résumé du checkpoint
				contentfirstCol = checkpoint.totalDistance;
				contentSecondCol = checkpoint.totalTime;
				
				if ($isDefined(rowspan)){
					// Spécial pour les drapeaux, pour les passages aux frontières 
					if(checkpoint.type=="F"){
						var imgTxt = checkpoint.number;
						contentThirdCol = imgTxt;
					} else {
						var value = (!isNaN(checkpoint.number) && checkpoint.number!="") ? this._drawPastilleCheckpoint(checkpoint, true, orientation) : "";
						contentThirdCol = value;
					}
					contentFourthCol= checkpoint.panels;
				}
				contentFifthCol = "";
				
				if(withSchema){
					var description = checkpoint.description, nDesc = "";
					var indice = description.indexOf("<div");
					if(indice != -1){
						nDesc = description.substring(indice, description.length-1);
					}
					
					contentFourthCol = "";//nDesc;
				}
			}
		// Orientation en mode Normal
		} else {
			if (type=="start" || type=="end") {
				contentThirdCol = checkpoint.number;
				contentFifthCol = checkpoint.description + ((withMap) ? this._getMapBlock(checkpoint.mapURL) : "");
				
			} else if (type == "step") {
				contentfirstCol  = "";//checkpoint.totalDistance;
				contentSecondCol = "";//checkpoint.totalTime;
				
				if ($isDefined(rowspan)) {
					// Spécial pour les drapeaux, pour les passages aux frontières 
					if(checkpoint.type=="F"){
						var imgTxt = checkpoint.number;
						contentThirdCol = imgTxt;
					} else {
						// si étape
						var value = "";
						if(checkpoint.type=="C" && checkpoint.picto=="step"){
							value = checkpoint.number;
						} else {
							value = (!isNaN(checkpoint.number) && checkpoint.number!="") ? this._drawPastilleCheckpoint(checkpoint, true, orientation) : "";
						}
						contentThirdCol = value;
					}
					contentFourthCol= checkpoint.panels;
				}
				contentFifthCol = checkpoint.description + ((withMap) ? this._getMapBlock(checkpoint.mapURL) : "");
			}
		}
		
		strHTML+= '<tr class="'+ classFirstTR +'">';
		
		if ((display == "WEB" && orientation=="N") || (!withSchema) || (orientation=="C" && !withSchema)){
			strHTML += '<td class="colTotalDistance">'+ contentfirstCol +'</td>'
					+ '<td class="colTotalTime">'+ contentSecondCol +'</td>';
		}
		
		strHTML += '<td class="fdr_stepNumber" rowspan="'+ rowspan +'">'+ contentThirdCol +'</td>'
				+ '<td class="fdr_stepWays" rowspan="'+ rowspan +'">'+ contentFourthCol +'</td>'
				+ '<td class="fdr_stepDetails">'+ contentFifthCol +'</td>';
		
		if(this._mode=="NORMAL" && withSchema){
			strHTML += '<td class="fdr_schema" rowspan="'+ rowspan +'"><div class="miniSchemaContainer"></div></td>';
		}
		
		strHTML += '</tr>';
		
		// Traitement des détails du checkpoint
		strHTML+= (((checkpoint.details)&&(checkpoint.details.length > 0)) ? this._getDetailHTML2(checkpoint.details, orientation, display, withSchema) : "");
		
		// Traitement du bloc de fin
		if (orientation == "C"){
			contentfirstCol		= "";
			contentSecondCol	= "";
			contentFifthCol		= "";
			
			if (withSchema){
				if (type=="start" || type=="end") {
					contentfirstCol		= "";
					contentSecondCol	= "";
					contentThirdCol		= "";
					contentFourthCol	= "";
					contentFifthCol		= checkpoint.description;
					
				} else if (type == "step") {
					contentfirstCol		= "";//checkpoint.totalDistance;
					contentSecondCol	= "";//checkpoint.totalTime;
					contentFifthCol		= "";
					
					var description = checkpoint.description, nDesc = "";
					var indice = description.indexOf("<div");
					if(indice != -1){
						nDesc = description.substring(indice, description.length-1);
					}
					contentFifthCol = nDesc;
				}
			} else {
				if (type=="start" || type=="end") {
					contentfirstCol		= "";
					contentSecondCol	= "";
					contentThirdCol		= "";
					contentFourthCol	= ""; 
					contentFifthCol		= ((withMap) ? this._getMapBlock(checkpoint.mapURL) : "") + checkpoint.description;
					
				} else if (type == "step") {
					contentfirstCol		= "";//checkpoint.totalDistance;
					contentSecondCol	= "";//checkpoint.totalTime;
					contentFifthCol		= ((withMap) ? this._getMapBlock(checkpoint.mapURL) : "") + checkpoint.description;
				}
			}
		} else {
			classLastTR = "fdr_totalDistance";
			
			if (type=="start" || type=="end") {
				contentfirstCol		= "";
				contentSecondCol	= "";
				contentFifthCol		= "";
			} else if (type == "step") {
				contentfirstCol		= checkpoint.totalDistance;
				contentSecondCol	= (checkpoint.totalTime!="00h00") ? checkpoint.totalTime : "";
				contentFifthCol		= "";
			}
		}
		
		strHTML+= '<tr class="'+ classLastTR +'">';
		
		if ((display == "WEB" && orientation=="N") || (!withSchema) || (orientation=="C" && !withSchema)){
			strHTML += '<td class="colTotalDistance">'+ contentfirstCol +'</td>'
					+ '<td class="colTotalTime">'+ contentSecondCol +'</td>';
		}
		
		strHTML += '<td class="fdr_stepDetails">'+ contentFifthCol +'</td>'
				+ '</tr>';
		
		return strHTML;
	},
	/**
	 * @param {Object} checkpoint
	 * @param {Boolean} isDrawed
	 * @param {String} orientation
	 */
	_drawPastilleCheckpoint : function(checkpoint, isDrawed, orientation){
		if (!$isDefined(isDrawed)) isDrawed = false;
		if (isDrawed){
			var arrowHeaderSrc = ViaMichelin.Api.Constants.System.ApiImgPath +"all/iti/direction_arrow_header_N.png";
			if (orientation=="C"){
				arrowHeaderSrc = ViaMichelin.Api.Constants.System.ApiImgPath +"all/iti/direction_arrow_header_C.png";
			}
			
			var imgArrowLine="", imgArrowHead="", urlArrowLine=ViaMichelin.Api.Constants.System.ApiImgPath +"all/iti/direction_arrow_bg.png";
			if(navigator.isIE6){
				imgArrowLine = "<img class=\"arrowLine\" src=\""+ ViaMichelin.Api.Constants.System.ApiImgPath +"/all/s.gif\" style=\"filter:progid:DXImageTransform.Microsoft.AlphaImageLoader(src='"+ urlArrowLine +"',sizingMethod='scale')\" />";
				imgArrowHead = "<img class=\"arrowHead\" src=\""+ ViaMichelin.Api.Constants.System.ApiImgPath +"/all/s.gif\" style=\"filter:progid:DXImageTransform.Microsoft.AlphaImageLoader(src='"+ arrowHeaderSrc +"',sizingMethod='scale')\"/>";
			} else {
				imgArrowLine = "<img class=\"arrowLine\" src=\""+ urlArrowLine +"\" />";
				imgArrowHead = "<img class=\"arrowHead\" src=\""+ arrowHeaderSrc +"\"/>";
			}
			
			/*
			if(orientation=="C"){
				return "<img class='printPastille' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath +"/all/data/itisteps/e"+ checkpoint.number +".png'  alt='' />"
					+ ViaMichelin.Api.Util.drawMiniPastilleBleue(checkpoint.number)
					+ "<div class=\"arrowDirection\">"
						+ imgArrowLine
						+ imgArrowHead
						+ "<div class=\"stepResume\" style=\"display:none;\">"+ checkpoint.partialDistance.replace("&nbsp;", "<br/>") +"</div>"
					+ "</div>";
					
			} else {
				return "<img class='printPastille' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath +"/all/data/itisteps/e"+ checkpoint.number +".png'  alt='' />"
					+ ViaMichelin.Api.Util.drawMiniPastilleBleue(checkpoint.number)
					+ "<div class=\"arrowDirection\">"
						+ imgArrowLine
						+ imgArrowHead
						+ "<div class=\"stepResume\" style=\"display:none;\">"+ checkpoint.partialDistance.replace("&nbsp;", "<br/>") +"</div>"
					+ "</div>";
			}
			*/
			
			return ViaMichelin.Api.Util.drawMiniPastilleBleue(checkpoint.number);
				
		} else return "";
	},
	/**
	 * Format de détail des checkpoint utilisé pour la v1
	 * @param {Array} steps
	 */
	_getDetailHTML : function(steps) {
		var strHTML = "";
		for (var i=0; i < steps.length; i++)
		{
			strHTML += '<tr';
			if(typeof(steps[i].className) != "undefined"){
				strHTML += ' class="'+steps[i].className+'"'; 
			}
			strHTML += '>'
			+ ( (steps[i].partialDistance != "") ? (
				'<td class="fdr_underStep">' + steps[i].picto + " " + steps[i].description + '</td>'
				+ '<td class="fdr_lightGrey">'+((steps[i].partialDistance != "") ? "("+steps[i].partialDistance+")" : "")+'</td>'
				+ '<td></td>'
				+ '<td></td>')
			: (
				'<td class="fdr_underStep">' + steps[i].picto + " " + steps[i].description +'</td>'
				+ '<td></td>'
				+ '<td>'+steps[i].totalDistance+'</td>'
				+ '<td>'+steps[i].totalTime+'</td>'))
			+ '</tr>';
		}
		return strHTML;
	},
	/**
	 * Retourne le HTML des détails des checkpoints
	 * Utilisé pour la v2
	 * @param {Array} steps
	 * @param {String} orientation
	 * @param {String} display
	 * @param {Boolean} withSchema
	 */
	_getDetailHTML2 : function(steps, orientation, display, withSchema) {
		var strHTML = "";
		var len = steps.length;
		for (var iter=0; iter < len; iter++) {
			
			if(orientation == "C"){
				i = len - (iter+1);
			} else {
				i = iter;
			}
			
			// On n'affiche pas les franchissements
			if(steps[i].type != "fra"){
				var checkpoint = steps[i];
				
				strHTML += '<tr';
				if(typeof(steps[i].className) != "undefined"){
					strHTML += ' class="'+steps[i].className+'"';
				}
				strHTML += '>';
				
				var totalDistance = 0;
				
				// Affichage qu'en mode WEB ou s'il n'y a pas de schema
				if ((display == "WEB" && orientation=="N") || (!withSchema) || (orientation=="C" && !withSchema)){
					var totalTime = (steps[i].totalTime=="00h00") ? "" : steps[i].totalTime;
					strHTML += '<td class="colTotalDistance">'+ steps[i].totalDistance +'</td>'
							+ '<td class="colTotalTime">'+ totalTime +'</td>';
				}
				
				if (orientation=="C" && withSchema ) { //&& (steps[i].type=="V" || steps[i].type=="S")) {
					//strHTML += '<td class="fdr_underStep">' + steps[i].picto + " " + steps[i].description + '</td>';
					// on extrait le panneaux
					var description = steps[i].description, nDesc = "";
					var indice = description.indexOf("<div");
					
					if(indice != -1){
						nDesc = description.substring(indice, description.length-1);
					}
					
					strHTML += '<td class="fdr_underStep">'+ nDesc +'</td>';
				} else {
					var picto = steps[i].picto;
					
					var customPicto = null;
					if ($isDefined(steps[i].poi) && $isDefined(this._customizedPictos)) {
						if ($isDefined(this._customizedPictos[steps[i].poi.productId])) {
							customPicto = this._customizedPictos[steps[i].poi.productId];
						} else if ($isDefined(this._customizedPictos["default"])){
							customPicto = this._customizedPictos["default"];
						}
					}
					
					if(customPicto != null){
						var size = " style=\"height:25px\" ";
						if($isDefined(customPicto.size)){
							size = " style=\"width:"+ customPicto.size.width +"px;height:"+ customPicto.size.height +"px\" ";
						}
						picto = "<img class=\"poi\" src=\""+ customPicto.url + "\""+ size +"/>";
					} else if(steps[i].type == "CORRPOI"){
						var picto = "<img class=\"poi\" src=\""+steps[i].pictoUrl+"\" style=\"height:25px;\"/>";
					}
					strHTML += '<td class="fdr_underStep">' + picto + " " + steps[i].description + '</td>';
				}
				
				strHTML += '</tr>';
			}
		}
		return strHTML;
	},
	/**
	 * Dessine la feuille de route au cap
	 * @param {Object} options
	 */
	_drawRoadSheetCap : function(options){
		var display = $isDefined(options.display) ? options.display : "WEB";
		var container = options.container;
		var withMap = $isDefined(options.withMap) ? options.withMap : true;
		var withSchema = $isDefined(options.withSchema) ? options.withSchema : false;
		var orientation = "C";
		var classOrientation = "capOrientation";
		
		var sizeSchema = { 
			height : parseInt(this._roadsheet._schema.size[1]),
			width  : parseInt(this._roadsheet._schema.size[0])
		};
		
		var roadsheetElement = document.createElement("div");
			roadsheetElement.id = "idRoadSheet"+this.uniqueIdSuffix;
			roadsheetElement.className = "VMRoadSheetNew "+ classOrientation;
			roadsheetElement.expando = this;
			
		var schemaContainer 			= document.createElement("div");
			schemaContainer.id 			= "idSchemaContainer"+this.uniqueIdSuffix;
			schemaContainer.className 	= "schemaContainer";
			schemaContainer.style.height = sizeSchema.height+"px";
			
		var panelsContainer = document.createElement("div");
			panelsContainer.id 			= "idPanelsContainer";
			panelsContainer.className 	= "panelsContainer";
			//panelsContainer.style.width  = sizeSchema.width+"px";
			panelsContainer.style.height = sizeSchema.height+"px";
			
		// Determinaison d'un nom de classe unique décrivant l'affichage, à cause d'IE6
		var roadsheetClassName = "cap";
		
		if (display == "WEB") roadsheetClassName+= "W"+"eb";
		else roadsheetClassName+= "Print";
		
		if (withSchema == true) roadsheetClassName+= "WithSchema";
		else roadsheetClassName+= "NoSchema";
		
		roadsheetElement.className+= roadsheetClassName;
		if (display=="PRINT") roadsheetElement.className+= " display-printable";
		
		roadsheetElement.appendChild(schemaContainer);
		roadsheetElement.appendChild(panelsContainer);
		container.appendChild(roadsheetElement);
		
		if(withSchema) roadsheetElement.className+= " withSchema";
		
		if(!this.customItemsDrawable["map"]) roadsheetElement.className+= " mapOff";
		
		var strHTML = "";
		var steps = this._roadsheet.steps, len = steps.length;
		var tmpPanelsContainer = document.createElement("div");
		panelsContainer.appendChild(tmpPanelsContainer);
		
		var lastPanelPosition = 0;
		
		for (var i=0; i<len; i++) {
			var item = steps[i];
			
			if (item.details.length > 0) {
				for (var j=0; j<item.details.length; j++) {
					var detail = item.details[j];
					var description = detail.description, nDesc = "";
					var indice = description.indexOf("<div");
					
					if (indice != -1 && detail.type!="CORRPOI") {
						nDesc = description.substring(indice, description.length-1);
						tmpPanelsContainer.innerHTML = nDesc;
						
						var position = tmpPanelsContainer.firstChild.getAttribute("schema").split(",");
						var panelPosition = parseInt(position[1]);
						var panelHeight = parseInt(tmpPanelsContainer.firstChild.offsetHeight);
						panelPosition-= panelHeight;
						
						if ((panelPosition - panelHeight) < lastPanelPosition) {
							var delta = panelHeight - (panelPosition - lastPanelPosition);
							panelPosition = lastPanelPosition +10;
						}
						lastPanelPosition = panelPosition+panelHeight;
						
						var panel = tmpPanelsContainer.firstChild;
							panel.className = "panelSchema dirPan";
							panel.style.bottom = panelPosition +"px";
							//panel.style.left = "10px";
							panel.style.position = "absolute";
						panelsContainer.appendChild(panel);
					}
				}
			}
		}
	},
	/**
	 * Crée l'objet support pour le schema
	 * @param {String} orientation
	 */
	_createSchema : function(orientation){
		this._graphic = new ViaMichelin.Api.UI.Drawing.ItineraryBuilder(
			"idSchemaContainer"+this.uniqueIdSuffix,
			parseInt(this._roadsheet._schema.size[0]),
			parseInt(this._roadsheet._schema.size[1]),
			orientation
		);
	},
	/**
	 * Dessine le schema
	 */
	_drawSchema : function(){
		if(this._graphic != null){
			var size = {width:this._roadsheet._schema.size[0], height: this._roadsheet._schema.size[1]};
			// Draw adjacent roads			
			this._graphic.drawAdjacentRoads(this._roadsheet._schema.sections.adj);
			
			// Draw itinerary roads
			this._graphic.drawItineraryRoads(this._roadsheet._schema.sections.iti);
			
			// Draw roads indications, pastilles, crossings, etc.
			this._graphic.drawPuces(this._roadsheet.steps, this.customItemsDrawable);
			
			// Draw instructions arrows
			if(this.customItemsDrawable["arrow"]){
				this._graphic.drawInstructions(this._roadsheet._schema.sections.ins);
			}
			
			//graphic.drawRoadsLabel(this._roadsheet._schema.roadsLabel);
			// Dessine les pointillet entre les panneaux et le schema
			var panneaux = ViaMichelin.Api.Util.HTML.getElementsByClassName(document, "dirPan");
			var delta = JSE.HTML.findPosition($_id("idRoadSheet"+this.uniqueIdSuffix));
			
			for(var i=0;i<panneaux.length;i++){
				var panneau = panneaux[i], startX = 10, panneauH = panneau.offsetHeight;
				if (this._orientation=="C" && this._graphic._orientation=="C" ) startX = 290;
				
				var point2 = [startX, (JSE.HTML.findPosition(panneau)[1] - delta[1]) + (panneauH*30/100) ];
				var point1 = panneau.getAttribute("schema").split(",");
				var orientation = "left";
				
				this._graphic.drawDashPath(point2, [parseInt(point1[0]),parseInt(point1[1])] , { "stroke-dasharray":"3,4", "stroke-width":1 }, orientation);
			}
		}
	},
	/**
	 * Dessine le schema linéaire disponible en mode NORMAL
	 */
	_drawLinearSchema : function(){
		var checkpointsDOM = ViaMichelin.Api.Util.HTML.getElementsByClassName($_id("idRoadSheet"+this.uniqueIdSuffix), "fdr_stepBlock"), 
			len = checkpointsDOM.length;
		var Xgap = 150;
		
		for (var i=0; i<=len-1; i++) {
			if(i!=0 && i!=len-1){
				try{
				var checkpointDOM = checkpointsDOM[i], 
					details = this._roadsheet.steps[i].details, 
					detailLen = details.length,
					checkpoint = this._roadsheet.steps[i],
					startPoint = 0, 
					endPoint = checkpointDOM.offsetHeight,
					partialDistance = "",
					totalDistance = "";
					 
				// numPartialDistance: Taille de l'instruction
				// numTotalDistance  : Distance entre le début du parcours et la fin de l'instruction
				var numPartialDistance = 0, numTotalDistance = 0;
				var schemaDOM = ViaMichelin.Api.Util.HTML.getElementsByClassName(checkpointDOM, "miniSchemaContainer")[0];
				
				while(schemaDOM.childNodes.length>0)
					schemaDOM.removeChild(schemaDOM.firstChild);
				
				var graphic = new ViaMichelin.Api.UI.Drawing.ItineraryBuilder(
					schemaDOM,
					parseInt(this._schemaHeight),
					parseInt(checkpointDOM.offsetHeight),
					"N"//, {rendering:"CANVAS"}
				);
				
				var infoContainer = graphic._infoContainer;
				var arrowContainer = document.createElement("div");
					arrowContainer.className = "arrowContainer";
					arrowContainer.style.marginLeft = (Xgap-70)+"px";
				infoContainer.appendChild(arrowContainer);
				
				// Gérer le type de route
				var coords = [[Xgap, startPoint],[Xgap, endPoint]];
				var isoCountry = "FRA"
				var value = [[
					[[Xgap, startPoint],[Xgap, endPoint]],
					"FRA",
					0,
					9
				]];
				
				if ($isDefined(checkpoint.roadInfo)){
					value = [[
						[[Xgap, startPoint],[Xgap, endPoint]],
						checkpoint.roadInfo[0],
						checkpoint.roadInfo[1],
						checkpoint.roadInfo[2]
					]];
				}
				
				graphic.drawItineraryRoads(value);
				
				if(checkpoint.partialDistance != "") {
					partialDistance = checkpoint.partialDistance.replace("&nbsp;", "<br/>");
					numPartialDistance = parseFloat(partialDistance.replace(/([^0-9.]*)/g, ""));
					if(numPartialDistance<1) numPartialDistance = 1;
				}
				if(checkpoint.totalDistance != "") {
					totalDistance = checkpoint.totalDistance.replace("&nbsp;", "<br/>");
					numTotalDistance = parseFloat(totalDistance.replace(/([^0-9.]*)/g, ""));
					if(numTotalDistance<1) numTotalDistance = 1;
				}
				
				// Dessin des flêches
				var isEmpty = false;
				// on affiche pas la flêche pour les passages de frontières
				if (this._roadsheet.steps[i].type == "F") {
					//isEmpty = true;
				}
				var arrow = this._drawLinearArrow(endPoint-startPoint, partialDistance, isEmpty);
				arrowContainer.appendChild(arrow);	
				
				var lastCity = null;
				
				// Boucle sur les détails d'un checkpoint
				for (var j=0; j<detailLen; j++){
					var detail = details[j], city=detail.city, fontSize=12;
					// detailPosition : position courante du detail
					var schema = [], pointX=Xgap+40, pointY=0, detailPosition=detail.totalDistance.replace(/([^0-9.]*)/g, "");
					
					// Dessine les passages à proximités
					if (detail.type=="c" && detail.model=="proximity" &&  this.customItemsDrawable["proximity"]) {
						var pixelHeight = (numPartialDistance-(numTotalDistance-detailPosition));
						
						if (numPartialDistance == 0) pointY = 0;
						else pointY =  parseInt(( endPoint * pixelHeight ) / numPartialDistance ) - 1;
						
						//pointY = parseInt((checkpointDOM.offsetHeight * (detailPosition-lastTotalDistance)) / numPartialDistance) + startPoint;
						var nbLine=1;
						
						// Alignement par le bas.
						if(pointY + 16 >= endPoint) pointY = endPoint - 16; 
						
						// Découpage du nom de la ville
						// Ex: Villefranche-sur-Saône sur deux ligne
						// Villefranche
						// sur-Saône
						if(nbLine>1){
							var separator = " ";
							//if(detail.city.indexOf("-")!=-1) separator = "-";
							//var re = new RegExp(separator, 'i');
							//for(var i=1;i<nbLine;i++){
							//	city = detail.city.replace(re, "<br/>");
							//}
						}
						
						city = detail.city;
						
						if(!$isNotEmpty(lastCity) || ($isNotEmpty(lastCity) && pointY > lastCity.top + 15)){
							var proximityDOM = document.createElement("div");
								proximityDOM.className 	= "cityProximity";
								proximityDOM.style.top	= pointY +"px";
								proximityDOM.style.left	= pointX +"px";
								proximityDOM.innerHTML 	= city;
							
							infoContainer.appendChild(proximityDOM);
							ViaMichelin.Api.Util.HTML.adaptableDescription(proximityDOM);
							
							lastCity = {
								left	: pointX,
								top		: pointY,
								width	: proximityDOM.clientWidth,
								height	: proximityDOM.clientHeight
							};
						}
						
					// Dessine les franchissements
					} else if (detail.type=="fra") {
						
						
					// Dessine les péages
					} else if (detail.type=="P" && this.customItemsDrawable["toll"]) {
						
						var startX=Xgap-10, startY, endX=Xgap+10, endY, detailPosition=detail.totalDistance.replace(/([^0-9.]*)/g, "");;
						var style = {
							stroke: "#000"
						};
						//startY = endY = parseInt((checkpointDOM.offsetHeight * (detailPosition-lastTotalDistance)) / numPartialDistance) + startPoint;
						var pixelHeight = (numPartialDistance-(numTotalDistance-detailPosition));
						startY = endY =  parseInt(( endPoint * pixelHeight ) / numPartialDistance ) - 1;
						
						graphic.drawPath([[startX,startY], [endX,endY]], style);
					
					// Dessine les POIS CORRIDOR
					} else if (detail.type=="CORRPOI") {						
						var detailPosition = parseFloat(detail.totalDistance.replace(/([^0-9.]*)/g, ""));
						//console.log("> > ", detail , detail.name, detail.totalDistance +" >>> "+ detailPosition +", offsetHeight: "+ checkpointDOM.offsetHeight +", lastTotalDistance: "+ lastTotalDistance +", partialDistance: "+ partialDistance);
						//console.log("> > ("+ numPartialDistance +"-("+ numTotalDistance +"-"+ detailPosition +"))");
						var pixelHeight = (numPartialDistance-(numTotalDistance-detailPosition));
						var coordsY = parseInt(( endPoint * pixelHeight ) / numPartialDistance ) - 1;
						
						// sinon il est caché
						if(coordsY==0 || typeof(coordsY)==NaN) coordsY=3;
						
						if ($isNotEmpty(detail.poi.primaryType)){
							detail.poi.principalType = detail.poi.primaryType;
						}
						var type = "poi-"+ detail.poi.principalType;
						var coords = [[Xgap, coordsY],[ Xgap+15, coordsY-5, Xgap+25, coordsY+5]];
						//graphic.drawIndication(type, coords);
						
						// on recherche si le poi existe dans la table de correspondance
						var url = detail.pictoUrl;
						var size = {};
						
						var customPicto = "";
						if($isDefined(this._customizedPictos) && $isDefined(detail.poi)){
							if ($isDefined(this._customizedPictos[detail.poi.productId])){
								customPicto = this._customizedPictos[detail.poi.productId];
							} else if ($isDefined(this._customizedPictos["default"])){
								customPicto = this._customizedPictos["default"];
							}
						}
						
						if($isDefined(customPicto)){
							url = customPicto.url;
							if($isDefined(customPicto.size))
								size = customPicto.size;
						}
						graphic.drawPicto(url, coords, size);
					}
				}
				
				// Dessine les caissons des routes
				if ($isDefined(checkpoint.voies) && checkpoint.voies.length > 0 && checkpoint.voies[0][2]!="") {
					var firstPanel	= checkpoint.voies[0];
					var number		= firstPanel[0];
					var type		= firstPanel[1];
					var country		= checkpoint.isoCountry; 
					var width_text	= Math.abs(coords[2] - coords[0]);
					var height_text = Math.abs(coords[3] - coords[1]);
					var roadLabel = document.createElement("div");
						roadLabel.style.position = "absolute";
						roadLabel.style.right	= (this._schemaHeight-Xgap) +"px";
						roadLabel.style.top		= (startPoint+5) +"px";
						roadLabel.className		= "roadLabel";
					
			  		var txt = document.createElement("span");
			  			txt.className = "caisson_white caisson_default_"+ type +" caisson_"+ country +"_rte"+ type +"_carte";
			  			txt.style.padding = "2px";
					var txtNode = document.createTextNode(number);
					txt.appendChild(txtNode);
					roadLabel.appendChild(txt);
					infoContainer.appendChild(roadLabel);
				}
				
				
				// Dessine les pointillet entre les panneaux et le schema
				var panneaux = ViaMichelin.Api.Util.HTML.getElementsByClassName(checkpointDOM, "dirPan");
				var delta = JSE.HTML.findPosition($_id("idRoadSheet"+this.uniqueIdSuffix));
				var lastTotalDistance = 0;
				
				for(var k=0;k<panneaux.length;k++){
					var panneau = panneaux[k], startX=10, endX=Xgap, endY=2, middleX=0, middleY=0, panneauH = panneau.offsetHeight;
					var detailPosition = parseFloat(panneau.getAttribute("distance").replace(/([^0-9.]*)/g, ""));
					
					//console.log(">"+ k, panneau +", checkpointDOM.offsetHeight: "+ endPoint +", detailPosition: "+ detailPosition);
					// Pour les panneaux n'étant pas issus des details des étapes, nous n'avons pas leur position
					// en km sur le parcours. Leur position est donc renseigné au début de l'étape.
					if (k==0 && panneau.className.indexOf("isDetail")==-1){
						endY = 1;//startPoint;
					} else {
						//endY = parseInt((endPoint * (detailPosition-lastTotalDistance)) / numPartialDistance) + startPoint - 1;
						var pixelHeight = (numPartialDistance-(numTotalDistance-detailPosition));
						//console.log("= parseInt(( "+ endPoint +" * "+ pixelHeight +" ) / "+numPartialDistance+") - 1");
						if(detailPosition == numTotalDistance){
							endY = endPoint - 1;
						} else {
							endY =  parseInt(( endPoint * pixelHeight ) / numPartialDistance ) - 1; 
							if(endPoint < (endY-startPoint)) endY = endPoint + startPoint - 1; // -1 pour être dans la zone de dessin.
						}
					}
					
					//var startY = JSE.HTML.findPosition(panneau)[1] - delta[1] + (panneauH*30/100);
					var startY = JSE.HTML.findPosition(panneau)[1] - JSE.HTML.findPosition(checkpointDOM)[1] + (panneauH*30/100);
					//console.log("pos: "+ JSE.HTML.findPosition(checkpointDOM)[1] +" - "+ JSE.HTML.findPosition(panneau)[1] +" + "+ (panneauH*30/100) +", startY: "+ startY +", endY:"+ endY);
					middleX = 40;
					middleY = endY;
					
					graphic.drawPath([[startX, startY], [middleX, middleY], [endX, endY]] , { "stroke-dasharray":"3,4", "stroke-width":1 });
				}
				
				if (i!=(len-2)) {
					//startPoint = endPoint;
				}
				}catch(exc){
					
				}
			}
		}
	},
	/**
	 * Dessine le schema sous une forme linéaire
	 */
	_drawLinearSchemaBak : function(){
		try{
			var roadSheetHeight = ViaMichelin.Api.Util.HTML.getElementsByClassName($_id("idRoadSheet"+this.uniqueIdSuffix), "roadsheetContent")[0].offsetHeight;
		}catch(e){
			return;
		}
		
		this._graphic = new ViaMichelin.Api.UI.Drawing.ItineraryBuilder(
			"idSchemaContainer"+this.uniqueIdSuffix,
			parseInt(this._schemaHeight),
			parseInt(roadSheetHeight),
			"N"
		);
		
		var Xgap = 150;
		var infoContainer = this._graphic._infoContainer;
		var arrowContainer = document.createElement("div");
			arrowContainer.className = "arrowContainer";
			arrowContainer.style.marginLeft = (Xgap-70)+"px";
		infoContainer.appendChild(arrowContainer);
		
		var checkpointsDOM = ViaMichelin.Api.Util.HTML.getElementsByClassName($_id("idRoadSheet"+this.uniqueIdSuffix), "fdr_stepBlock"), len = checkpointsDOM.length;
		
		var startPoint=0, endPoint=0;
		var partialDistance = "";
		var numPartialDistance = 0;
		
		for (var i=0; i<=len-1; i++) {
			var checkpointDOM = checkpointsDOM[i], details = this._roadsheet.steps[i].details, detailLen = details.length,
				checkpoint = this._roadsheet.steps[i];
			
			if(checkpoint.partialDistance != "") {
				partialDistance = checkpoint.partialDistance.replace("&nbsp;", "<br/>");
				numPartialDistance = parseFloat(partialDistance.replace(/([^0-9.]*)/g, ""));
				if(numPartialDistance<1) numPartialDistance = 1;
			}
			
			endPoint+= checkpointDOM.offsetHeight;
			
			if (i!=0) { // && i!=(len-2)) {
				var lastTotalDistance = parseFloat(this._roadsheet.steps[i-1].totalDistance.replace(/([^0-9.]*)/g, ""));
				if (i==1) lastTotalDistance = 0;
				
				// Gérer le type de route
				var coords = [[Xgap, startPoint],[Xgap, endPoint]];
				var isoCountry = "FRA"
				var value = [[
					[[Xgap, startPoint],[Xgap, endPoint]],
					"FRA",
					0,
					9
				]];
				
				if ($isDefined(checkpoint.roadInfo)){
					value = [[
						[[Xgap, startPoint],[Xgap, endPoint]],
						checkpoint.roadInfo[0],
						checkpoint.roadInfo[1],
						checkpoint.roadInfo[2]
					]];
				}
				
				this._graphic.drawItineraryRoads(value);
				
				if (i!=(len-2)) {
					// Dessin des flêches
					var isEmpty = false;
					// on affiche pas la flêche pour les passages de frontières
					if (this._roadsheet.steps[i].type == "F") {
						//isEmpty = true;
					}
					var arrow = this._drawLinearArrow(endPoint-startPoint, partialDistance, isEmpty);
					arrowContainer.appendChild(arrow);				
				}
				
				// Boucle sur les détails d'un checkpoint
				for (var j=0; j<detailLen; j++){
					var detail = details[j], city=detail.city, fontSize=12;
					var schema = [], pointX=Xgap+40, pointY=0, detailPosition=detail.totalDistance.replace(/([^0-9.]*)/g, "");
					
					// Dessine les passages à proximités
					if (detail.type=="c" && detail.model=="proximity" &&  this.customItemsDrawable["proximity"]) {
						pointY = parseInt((checkpointDOM.offsetHeight * (detailPosition-lastTotalDistance)) / numPartialDistance) + startPoint;
						var nbLine=1;
						
						// Découpage du nom de la ville
						// Ex: Villefranche-sur-Saône sur deux ligne
						// Villefranche
						// sur-Saône
						if(nbLine>1){
							var separator = " ";
							if(detail.city.indexOf("-")!=-1) separator = "-";
							var rexp = new RegExp(separator, 'i');
							for(var i=1;i<nbLine;i++){
								city = detail.city.replace(rexp, "<br/>");
							}
						}
						
						var container = document.createElement("div");
							container.className = "cityProximity";
							container.style.top	= pointY +"px";
							container.style.left= pointX +"px";
							container.innerHTML = city;
						
						infoContainer.appendChild(container);
					
					// Dessine les franchissements
					} else if (detail.type=="fra") {
						
						
						
					// Dessine les péages
					} else if (detail.type=="P" && this.customItemsDrawable["toll"]) {
						
						var startX=Xgap-10, startY, endX=Xgap+10, endY, detailPosition=detail.totalDistance.replace(/([^0-9.]*)/g, "");;
						var style = {
							stroke: "#000"
						};
						startY = endY = parseInt((checkpointDOM.offsetHeight * (detailPosition-lastTotalDistance)) / numPartialDistance) + startPoint;
						
						this._graphic.drawPath([[startX,startY], [endX,endY]], style);
					
					// Dessine les POIS CORRIDOR
					} else if (detail.type=="CORRPOI") {						
						var detailPosition = parseInt(detail.totalDistance.replace(/([^0-9.]*)/g, ""));
						//console.log("> > ", detail , detail.name, detail.totalDistance +" >>> "+ detailPosition +", offsetHeight: "+ checkpointDOM.offsetHeight +", lastTotalDistance: "+ lastTotalDistance +", partialDistance: "+ partialDistance);
						//console.log("> > ("+ checkpointDOM.offsetHeight+" * ("+detailPosition+"-"+lastTotalDistance+")) / "+ numPartialDistance +" ) + "+ startPoint);
						
						var coordsY = parseInt((checkpointDOM.offsetHeight * (detailPosition-lastTotalDistance)) / numPartialDistance) + startPoint;
						//console.log("> > ("+ checkpointDOM.offsetHeight+" * "+ numPartialDistance +") / ("+detailPosition+"-"+lastTotalDistance+")) + "+ startPoint);
						//var coordsY = parseInt((checkpointDOM.offsetHeight * numPartialDistance) / (detailPosition-lastTotalDistance)) + startPoint;
						
						// sinon il est caché
						if(coordsY==0 || typeof(coordsY)==NaN) coordsY=3;
						
						if ($isNotEmpty(detail.poi.primaryType)){
							detail.poi.principalType = detail.poi.primaryType;
						}
						var type = "poi-"+ detail.poi.principalType;
						var coords = [[Xgap, coordsY],[ Xgap+15, coordsY-5, Xgap+25, coordsY+5]];
						this._graphic.drawIndication(type, coords);
					}
				}
				
				// Dessine les caissons des routes
				if ($isDefined(checkpoint.voies) && checkpoint.voies.length > 0 && checkpoint.voies[0][2]!="") {
					var firstPanel	= checkpoint.voies[0];
					var number		= firstPanel[0];
					var type		= firstPanel[1];
					var country		= checkpoint.isoCountry; 
					var width_text	= Math.abs(coords[2] - coords[0]);
					var height_text = Math.abs(coords[3] - coords[1]);
					var roadLabel = document.createElement("div");
						roadLabel.style.position = "absolute";
						roadLabel.style.right	= (this._schemaHeight-Xgap) +"px";
						roadLabel.style.top		= (startPoint+5) +"px";
						roadLabel.className		= "roadLabel";
					
			  		var txt = document.createElement("span");
			  			txt.className = "caisson_white caisson_"+ country +"_rte"+ type +"_carte";
			  			txt.style.padding = "2px";
					var txtNode = document.createTextNode(number);
					txt.appendChild(txtNode);
					roadLabel.appendChild(txt);
					infoContainer.appendChild(roadLabel);
				}
				
//				// Dessin des pictos
//				for (var j=0; j<detailLen; j++){
//					var detail = details[j];
//					// calcul des coordonnées
//					var detailPosition = parseInt(detail.totalDistance.replace(/([^0-9.]*)/g, ""));
//					/*
//					var coordsY = parseInt((checkpointDOM.offsetHeight * (detailPosition-lastTotalDistance)) / parseInt(numPartialDistance)) + startPoint;
//					
//					if (detail.type == "vir" || detail.type == "dec") {
//						if (this.customItemsDrawable["security"]) {
//							var type = detail.type +"_"+ detail.num;
//							var coords = [[Xgap, coordsY],[ Xgap+10, coordsY-5, Xgap+20, coordsY+5]];
//							this._graphic.drawIndication(type, coords);
//						}
//					}
//				}
				
				// Dessine les pointillet entre les panneaux et le schema
				var panneaux = ViaMichelin.Api.Util.HTML.getElementsByClassName(checkpointDOM, "dirPan");
				var delta = JSE.HTML.findPosition($_id("idRoadSheet"+this.uniqueIdSuffix));
				
				for(var k=0;k<panneaux.length;k++){
					var panneau = panneaux[k], startX=10, endX=Xgap, endY=0, middleX=0, middleY=0, panneauH = panneau.offsetHeight;
					var detailPosition = parseFloat(panneau.getAttribute("distance").replace(/([^0-9.]*)/g, ""));
					
					// Pour les panneaux n'étant pas issus des details des étapes, nous n'avons pas leur position
					// en km sur le parcours. Leur position est donc renseigné au début de l'étape.
					if (k==0 && panneau.className.indexOf("isDetail")==-1){
						endY = startPoint;
					} else {
						endY = parseInt((checkpointDOM.offsetHeight * (detailPosition-lastTotalDistance)) / numPartialDistance) + startPoint;
						if(checkpointDOM.offsetHeight < (endY-startPoint)) endY = checkpointDOM.offsetHeight + startPoint;
					}
					
					var startY = JSE.HTML.findPosition(panneau)[1] - delta[1] + (panneauH*30/100);
					
					middleX = 40;
					middleY = endY;
					
					this._graphic.drawPath([[startX, startY], [middleX, middleY], [endX, endY]] , { "stroke-dasharray":"3,4", "stroke-width":1 });
				}
				
				if (i!=(len-2)) {
					startPoint = endPoint;
				}
			}
		}
	},
	clearSchema : function(){
		this._graphic.clear();
	},
	/**
	 * Retourne le code HTML pour dessiner une flêche entre deux étape.
	 * @param {int} 	height	: 
	 * @param {String} 	text	: 
	 * @param {Boolean} isEmpty	: 
	 * @return {HTMLElement}
	 */
	_drawLinearArrow : function(height, text, isEmpty){
		var gap=20, textAligned="", content="&nbsp;";
		isEmpty = ($isDefined(isEmpty)) ? isEmpty : false;
		
		if (height > 40){
			textAligned = "<div style=\"display: block; top: "+ (parseInt(height / 2) - gap) +"px;\" class=\"stepResume\">"+ text +"<br/></div>";
		}
		
		if (!isEmpty){
			content = "<img src=\""+ ViaMichelin.Api.Constants.System.ApiImgPath +"all/iti/direction_arrow_bottom_N.png\" class=\"arrowBottom\" />" 
				+ "<img src=\""+ ViaMichelin.Api.Constants.System.ApiImgPath +"all/iti/direction_arrow_bg.png\" class=\"arrowLine\" style=\"height: "+ height +"px;\" />"
				+ "<img src=\""+ ViaMichelin.Api.Constants.System.ApiImgPath +"all/iti/direction_arrow_header_N_2.png\" class=\"arrowHead\" />"
				+ textAligned;
		}
		
		var element = document.createElement("div");
			element.className = "arrowDirection";
			element.style.height = height+"px";
			element.style.width = "15px";
			element.innerHTML = content;
		
		return element;
	},
	/**
	 * @param {String} url
	 */
	_getMapBlock : function(url) {
		var size="";
		
		if($isDefined(this._roadsheet._customizedMap) && $isDefined(this._roadsheet._customizedMap.step)){
			var stepMap = this._roadsheet._customizedMap.step;
			size = "width:"+stepMap.size.width+"px;height:"+stepMap.size.height+"px;";
		}
		
		if (url == "") return "";
		
		url = ViaMichelin.Api.Util.Other.protocolAdapter(url);
		
		var text = '<div class="mapBlock">';
		text += '<span class="txtLink" onclick="javascript:this.parentNode.getElementsByTagName(\'img\')[0].style.display = \'block\';this.parentNode.getElementsByTagName(\'img\')[0].src = \''+url+'\';this.style.display = \'none\'; var cElement=document.getElementById(\'idRoadSheet'+ this.uniqueIdSuffix +'\'); cElement.expando.resize();">'+ this._roadsheet.labels['lab_map'] +'</span>';
		text += '<img src="'+ ViaMichelin.Api.Constants.System.ApiImgPath + 'all/s.gif" onclick="this.parentNode.getElementsByTagName(\'span\')[0].style.display = \'inline\';this.style.display = \'none\';var cElement=document.getElementById(\'idRoadSheet'+ this.uniqueIdSuffix +'\'); cElement.expando.resize();" style="display:none;'+size+'" />';
		text += '</div>';
		return text;
	},
	/**
	 * Rajoute des classes dans la feuille de route pour masquer certains éléments définis dans "list".
	 * @param {Array} list
	 */
	_customizeRoadsheet : function(list){
		for(var item in list){
			switch(item){
				case "radar" :
					if(list[item]){ 
						JSE.HTML.removeClass($_id('idRoadSheet'+this.uniqueIdSuffix),'radarOff','');
					} else {
						JSE.HTML.addClass($_id('idRoadSheet'+this.uniqueIdSuffix),'radarOff');
					}
					break;
				case "toll" :
					if(list[item]){ 
						JSE.HTML.removeClass($_id('idRoadSheet'+this.uniqueIdSuffix),'tollOff','');
					} else {
						JSE.HTML.addClass($_id('idRoadSheet'+this.uniqueIdSuffix),'tollOff');
					}
					break;
				case "proximity" :
					if(list[item]){ 
						JSE.HTML.removeClass($_id('idRoadSheet'+this.uniqueIdSuffix),'proximityOff','');
					} else {
						JSE.HTML.addClass($_id('idRoadSheet'+this.uniqueIdSuffix),'proximityOff');
					}
					break;
				case "bridge" :
					if(list[item]){ 
						JSE.HTML.removeClass($_id('idRoadSheet'+this.uniqueIdSuffix),'bridgeOff','');
					} else {
						JSE.HTML.addClass($_id('idRoadSheet'+this.uniqueIdSuffix),'bridgeOff');
					}
					break;
				case "tunnel" :
					if(list[item]){ 
						JSE.HTML.removeClass($_id('idRoadSheet'+this.uniqueIdSuffix),'tunnelOff','');
					} else {
						JSE.HTML.addClass($_id('idRoadSheet'+this.uniqueIdSuffix),'tunnelOff');
					}
					break;
				case "security" :
					if(list[item]){ 
						JSE.HTML.removeClass($_id('idRoadSheet'+this.uniqueIdSuffix),'securityOff','');
					} else {
						JSE.HTML.addClass($_id('idRoadSheet'+this.uniqueIdSuffix),'securityOff');
					}
					break;
				case "map" : 
					if(list[item]){ 
						JSE.HTML.removeClass($_id('idRoadSheet'+this.uniqueIdSuffix),'mapOff','');
					} else {
						JSE.HTML.addClass($_id('idRoadSheet'+this.uniqueIdSuffix),'mapOff');
					}
					break;
			}
		}
	},
	toString : function(){
		return "[ViaMichelin.Api.Itinerary.RoadSheetView]";
	}
};
