JSEPackage("ViaMichelin.Api.Map.Layer");

JSEImportApi("ViaMichelin.Api.Model.GeoCoordinates");
JSEImportApi("ViaMichelin.Api.Map.ExpandLayer");
JSEImportApi("JSE.HTML.UIComponent");

/**
 * @class ViaMichelin.Api.Map.Layer
 * @extends {JSE.HTML.UIComponent} 
 * @param {ViaMichelin.Api.Model.GeoCoordinates} oVMLonLat
 * @param {String} html
 * @param {int} offsetX
 * @param {int} offsetY
 * @param {Object} jsObject
 */
ViaMichelin.Api.Map.Layer = function(oVMLonLat,html,offsetX,offsetY,jsObject){
	this.extend(JSE.HTML.UIComponent);
	this.prepare();
	
	this.uniqueIdSuffix = (new Date()).getTime() + '' + parseInt(Math.random()*1000000);
	
	this.coords = oVMLonLat;
	if (offsetX) this.offsetX = offsetX;
	if (offsetY) this.offsetY = offsetY;
	
	this.HTMLElement.id = "VMLayer" + this.uniqueIdSuffix;
	this.HTMLElement.style.position = "absolute";
	if (html) this.HTMLElement.innerHTML = html;
	if (jsObject) this.jsObject = jsObject;
	else this.jsObject = null;
	this.setZindex(2);
	//this.setVisibility(true);
};

ViaMichelin.Api.Map.Layer.prototype = {
	JSEwireHub : {
		"HTMLEvent_click" : [
			{event:"HTMLEvent_click", method:"JSEInstance.cancelBubbling", thisObj:"JSEInstance"},
			{event:"HTMLEvent_click", method:"JSEInstance._triggerClickEvent", thisObj:"JSEInstance"}
		],
		"HTMLEvent_mouseover" : [
			{event:"HTMLEvent_mouseover", method:"JSEInstance.cancelBubbling", thisObj:"JSEInstance"},
			{event:"HTMLEvent_mouseover", method:"JSEInstance._triggerMouseoverEvent", thisObj:"JSEInstance"}
		],
		"HTMLEvent_mouseout" : [
			{event:"HTMLEvent_mouseout", method:"JSEInstance.cancelBubbling", thisObj:"JSEInstance"},
			{event:"HTMLEvent_mouseout", method:"JSEInstance._triggerMouseoutEvent", thisObj:"JSEInstance"}
		 ],
		"HTMLEvent_mousedown" : [],
		"HTMLEvent_mouseup" :  [{event:"HTMLEvent_mouseup", method:"JSEInstance._checkMouseUp", thisObj:"JSEInstance"}],
		"HTMLEvent_touchend" :  [{event:"HTMLEvent_touchend", method:"JSEInstance._checkMouseUp", thisObj:"JSEInstance"}],
		"HTMLEvent_mousemove" :  [],
		"HTMLEvent_mousewheel" : [],
		"HTMLEvent_contextmenu" : [
			{event:"HTMLEvent_contextmenu", method:"JSEInstance.cancelBubbling", thisObj:"JSEInstance"},
			{event:"HTMLEvent_contextmenu", method:"JSEInstance.preventDefAction", thisObj:"JSEInstance"}
			
		],
		"onMouseOver" : [],
		"onMouseOut" : [],
		"onClick" : [{event:"onClick", method:"JSEInstance._showExpand", thisObj:"JSEInstance"}],
		"onExpandOpen" : [],
		"onExpandClose" : []
	},
	offsetX			: 0,
	offsetY			: 0,
	zIndex			: 2,
	visibility		: true,
	expandOffsetX	: 32,
	expandOffsetY	: 15,
	expandLayer		: null,
	needClipping	: false,
	mapObject		: null,
	jsObject		: null,
	contextMenuTemplate	: "",
	contextMenuControl	: false,
	_clickTriggable		: true,
	/**
	 * @event {HTMLEvent_mouseup}
	 */
	_checkMouseUp : function(e){
		if(typeof(this.mapObject) != "undefined" && this.mapObject!=null){
			if (this.mapObject.hasMoved) {
				this._clickTriggable = false;
			} else {
				this._clickTriggable = true;
				this.cancelBubbling(e);
				this.mapObject.isDragging = false;
				this.mapObject.isZooming = false;
				
				if(navigator.isIE){
					document.detachEvent("onmousemove", this.mapObject._docmove);
					document.detachEvent("onmouseup", this.mapObject._docup);
				} else {
					document.removeEventListener("mousemove", this.mapObject._docmove, false);
					document.removeEventListener("mouseup", this.mapObject._docup, false);
				}
				
				document.tiledmap = "";
				this.mapObject.HTMLElement.style.cursor = "";
			}
		}
	},
	/**
	 * Indique si le contextMenu doit actif ou non
	 * @param {Boolean} value
	 */
	setContextMenuControl : function(value){
		//this._loadContextMenuTemplate();
		this.contextMenuControl = value;
		if (value == true) {
			this.plugWire(new JSEWire("HTMLEvent_contextmenu", this._displayContextMenu, this));
		} else {
			this.unplugWire(new JSEWire("HTMLEvent_contextmenu", this._displayContextMenu, this));
		}
	},
	/**
	 * Met à jour la valeur du contextMenuTemplate
	 * @param {String} value
	 */
	setContextMenuTemplate : function(value){
		this.contextMenuTemplate = value;
	},
	/**
	 * Appel suite a la levé de l'evenement "HTMLEvent_contextmenu"
	 * @return {void}
	 */
	_contextMenu : function(e){
		this.stimulate("onContextMenu",e);
		this.preventDefAction(e);
		this.cancelBubbling(e);
	},
	/**
	 * Appel de la méthode suite à la levé de l'evenement "onContextMenu"
	 * Evenement stimulé des qu'un click droit survient
	 * Affichage du contextMenu
	 * @param {Event} e
	 * @return {void}
	 */
	_displayContextMenu : function(e){
		if(this.contextMenuControl){
			e = window.event ? window.event : e;	
			this.mouseStartX = (navigator.isMozilla ? e.pageX : e.clientX + document.documentElement.scrollLeft);
			this.mouseStartY = (navigator.isMozilla ? e.pageY : e.clientY + document.documentElement.scrollTop);
			
			if (this.contextMenu == null){
				this._loadContextMenuTemplate();
			} else {
				//alert("Positionnement:"+this.mouseStartX+","+this.mouseStartY);
				this.moveContextMenu();
			}
		}
	},
	_loadContextMenuTemplate : function () {
		var req = new JSE.HTTP.XMLHTTPRequest(this.contextMenuTemplate);
		req.plugWire(new JSEWire("responseLoaded", this._contextMenuTemplateLoaded, this));
		req.request();
	},
	/**
	 * Callback du chargement du menu contextuel
	 * @param {String} template
	 * @return {void}
	 */
	_contextMenuTemplateLoaded : function(template){
		var tpl = template.responseText;
		
		if (this.contextMenu != null) {
			this.HTMLElement.removeChild(this.contextMenu.container);
		}
		
		var contextMenuDiv = document.createElement("div");
		contextMenuDiv.className = "sContextMenu";
		this.mapObject.container.appendChild(contextMenuDiv);
		
		var menuConf = {
			container: contextMenuDiv,
			template: tpl,
			application: this.mapObject,
			layer : this
		};
		menuConf.application = new Object();
		menuConf.application.mapObject = this.mapObject;
		menuConf.application.layer = this;
		JSELaunch("ViaMichelin.Api.UI.MenuComponent", menuConf, new JSEWire("onInit", this.contextMenuLoaded, this), ViaMichelin.Api.Constants.System.ApiJSEPath);
		
		//this.displayedOnMap = '';
		this.stimulate("contextMenuTemplateLoaded");
	},
	/**
	 * Instance du contextMenu
	 * @param {ViaMichelin.Api.UI.MenuComponent} menu
	 * @return {void}
	 */
	contextMenuLoaded : function(menu) {
		this.contextMenu = menu;
		
		this.mapObject.plugWire(new JSEWire("HTMLEvent_mousewheel", this._closeContextMenu, this));
		this.mapObject.plugWire(new JSEWire("HTMLEvent_mousedown", this._closeContextMenu, this));
		this.mapObject.plugWire(new JSEWire("onBubbleClosed", this._closeContextMenu, this));
		this.mapObject.plugWire(new JSEWire("HTMLEvent_mouseup", this._closeContextMenu, this));
		
		this.moveContextMenu();
	},
	/**
	 * Déplace le contextMenu
	 */
	moveContextMenu : function() {
		
		if(typeof(this.contextMenu)!="undefined"){
			
			this.contextMenu.container.style.display = "block";
			this.contextMenuX = this.mouseStartX - JSE.HTML.findPosition(this.mapObject.container)[0];
			this.contextMenuY = this.mouseStartY - JSE.HTML.findPosition(this.mapObject.container)[1];
			
			if ((parseInt(this.mapObject.container.style.width) - ( parseInt(this.contextMenuX) + parseInt(this.contextMenu.container.offsetWidth) )) > 0) {
				this.contextMenu.container.style.left = this.contextMenuX + "px";
			} else {
				this.contextMenu.container.style.left = ( this.contextMenuX - this.contextMenu.container.offsetWidth ) + "px";
			}
			
			if (( parseInt(this.mapObject.container.style.height ) - ( parseInt(this.contextMenuY) + parseInt(this.contextMenu.container.offsetHeight) )) > 0) {
				this.contextMenu.container.style.top = this.contextMenuY + "px";
			} else {
				this.contextMenu.container.style.top = ( this.contextMenuY - this.contextMenu.container.offsetHeight ) + "px";
			}			
		}
	},
	/**
	 * Ferme le contextMenu
	 */
	_closeContextMenu : function(){
		if(typeof(this.contextMenu)!="undefined") {
			this.contextMenu.container.style.display = "none";
		}
	},
	_triggerClickEvent : function(e){
		if (this._clickTriggable) this.stimulate("onClick", e, this);
		this._clickTriggable = true;
	},
	_triggerMouseoverEvent : function(e){
		this.stimulate("onMouseover", e, this);
	},
	_triggerMouseoutEvent : function(e){
		this.stimulate("onMouseout", e, this);
	},
	setHTML : function(html){
		this.HTMLElement.innerHTML = html;
	},
	getVisibility : function(){
		return this.visibility;
	},
	setVisibility : function(visibility){
		this.HTMLElement.style.visibility = (visibility === true ? "visible": "hidden");
		if ($isDefined(this.expandLayer) && $isDefined(this.expandLayer.HTMLElement)){
			this.expandLayer.HTMLElement.style.visible = (visibility === true ? "visible": "hidden");
		}
		this.visibility = visibility;
	},
	getZindex : function(){
		if($isDefined(this.HTMLElement)){
			return this.HTMLElement.style.zIndex;
		}else{
			return NaN;
		}
	},
	setZindex : function(zIndex){
		this.HTMLElement.style.zIndex = zIndex;
		if ($isDefined(this.expandLayer) && $isDefined(this.expandLayer.HTMLElement)){
			this.expandLayer.HTMLElement.style.zIndex = zIndex;
		}
	},
	getGeoBoundaries : function(){
		return [new ViaMichelin.Api.Model.GeoCoordinates(this.coords.lon,this.coords.lat),new ViaMichelin.Api.Model.GeoCoordinates(this.coords.lon,this.coords.lat)];
	},
	convertToPoints : function(obj,func){
		var oVMPoint = func.call(obj, this.coords);
		this.HTMLElement.style.left = oVMPoint.x + this.offsetX + "px";
		this.HTMLElement.style.top = oVMPoint.y + this.offsetY + "px";
		
		this.mapObject = obj;
		if ((this.expandLayer != null)&&(this.expandLayer.mapObject == null)) this.expandLayer.mapObject = obj;
	},
	setExpandLayer : function(htm){
		this.expandLayer = new ViaMichelin.Api.Map.ExpandLayer(this.coords, htm, this.offsetX, this.offsetY);
		
		//this.unplugWire(new JSEWire("onClick",this._showExpand,this));
		//this.plugWire(new JSEWire("onClick",this._showExpand,this));
		
		this.HTMLElement.style.cursor = "hand";		
		this.HTMLElement.style.cursor = "pointer";
	},
	/**
	 * Indique si le layer affiche une bubble au click
	 * @return {Boolean}
	 */
	isExpandLayer : function(){
		if(this.expandLayer!=null) return true;
		return false;
	},
	_showExpand : function(e){
		this.openExpandLayer();
		if(e){
			this.cancelBubbling(e);
		}
	},
	setBubbleContent : function(htm){
		this.expandLayer = new ViaMichelin.Api.Map.ExpandLayer(this.coords, htm, this.offsetX, this.offsetY); 
		this.openExpandLayer(this.expandLayer, this);
	},
	openExpandLayer : function(){
		if (this.expandLayer != null && this.mapObject != null) {
			this.unplugWire(new JSEWire("onClick",this._showExpand,this));
			this.unplugWire(new JSEWire("onClick",this.closeExpandLayer,this));
			this.plugWire(new JSEWire("onClick",this.closeExpandLayer,this));
			this.mapObject.showBubble(this.expandLayer, this);
			this.stimulate("onExpandOpen",this);
			this.mapObject.unplugWire(new JSEWire("onBubbleClosed",this._triggerExpandClose,this));
			this.mapObject.plugWire(new JSEWire("onBubbleClosed",this._triggerExpandClose,this));
		}		
	},
	closeExpandLayer : function(){
		if (this.mapObject != null) {
			this.mapObject.unplugWire(new JSEWire("onBubbleClosed",this._triggerExpandClose,this));
			this.unplugWire(new JSEWire("onClick",this.closeExpandLayer,this));
			this.unplugWire(new JSEWire("onClick",this._showExpand,this));
			this.plugWire(new JSEWire("onClick",this._showExpand,this));
			this.mapObject.closeBubble.call(this.mapObject.bubble);
			this.stimulate("onExpandClose", this);
		}
	},
	_triggerExpandClose : function(){
		this.mapObject.unplugWire(new JSEWire("onBubbleClosed",this._triggerExpandClose,this));
		this.unplugWire(new JSEWire("onClick",this.closeExpandLayer,this));
		this.unplugWire(new JSEWire("onClick",this._showExpand,this));
		this.plugWire(new JSEWire("onClick",this._showExpand,this));
		this.stimulate("onExpandClose", this);	
	},
	release : function(){
		this.expandLayer = "";
		this.clearDOM();
	},
	toString : function(){
		return "[ViaMichelin.Api.Map.Layer]";
	}
};