JSEPackage("ViaMichelin.Api.Map.PolyLine");

JSEImportApi("ViaMichelin.Api.Map.Layer");
JSEImportApi("ViaMichelin.Api.Map.ExpandLayer");
JSEImportApi("ViaMichelin.Api.Util");
JSEImportApi("ViaMichelin.Api.Model.GeoCoordinates");
JSEImportApi("JSE.HTML");
JSEImportApi("ViaMichelin.Api.Util");
JSEImportApi("ViaMichelin.Api.Util.HTML");

/**
 * Crée une polyline affichable sur la carte
 * 
 * @name ViaMichelin.Api.Map.PolyLine
 * @class ViaMichelin.Api.Map.PolyLine
 * @extends ViaMichelin.Api.Map.Layer
 * @param {Object} config
 * @param {ViaMichelin.Api.Model.GeoCoordinates} config.coords
 * @param {String} config.strokeColor
 * @param {Number} config.strokeOpacity
 * @param {Number} config.strokeWeight
 * @param {String} config.fillColor
 * @param {Number} config.fillOpacity
 * @param {String} config.type
 * @param {Boolean} config.needClipping
 * 
 * @example
 * config : {
 *    coords		: [ [lon] , [lat] ],
 *    strokeColor 	: "#000",
 *    strokeOpacity : 1,
 *    strokeWeight 	: 2,
 *    fillColor 	: "",
 *    fillOpacity 	: 0,
 *    type			: "polyLine",
 *    needClipping 	: true/false
 * }
 * 
 */
ViaMichelin.Api.Map.PolyLine = function(config){
	this.extend(ViaMichelin.Api.Map.Layer);
	this.HTMLElement.oncontextmenu = "";
	this.prepare();	

	this.uniqueIdSuffix = (new Date()).getTime() + '' + parseInt(Math.random()*1000000);
	
	this.initialConf = new Object();
	for (var item in ViaMichelin.Api.Map.PolyLine.prototype.initialConf) {
		if(item == "coords"){
			this.initialConf["coords"] = new Array();
		} else
			this.initialConf[item] = ViaMichelin.Api.Map.PolyLine.prototype.initialConf[item];
	}
	for(var item in config){
		this.initialConf[item] = config[item];
	}
	
	this.initialConf.coords = this._convertCoordsStructure(this.initialConf.coords, "[[lon],[lat]]");
	
	if (this.initialConf.coords.length > 1 && this.initialConf.coords[0][0].length != 0 && this.initialConf.coords[1][0] != 0) {
		this.coords = new ViaMichelin.Api.Model.GeoCoordinates(this.initialConf.coords[0][0], this.initialConf.coords[1][0]);
	}
	this.needClipping = this.initialConf.needClipping;
	this.psp = 0;
	this.aSimplifiedPoints = new Array();
	/*if($isDefined(window.zIndexVMShape)){
		this.zIndex = ++window.zIndexVMShape;
	}else{
		window.zIndexVMShape = 2;
		this.zIndex =	window.zIndexVMShape;
	}
	this.HTMLElement.style.zIndex = "";*/
	
	if ($isNotEmpty(this.initialConf.htm)) {
		this.setExpandLayer(this.initialConf.htm);
		this.HTMLElement.style.cursor = "";
	}
	this._unplugEventListeners(this.HTMLElement, this.reportHTMLEvent);
	
	if (navigator.isIE){
		this.convertToPoints = this._traceVML;	
	} else if (ViaMichelin.Api.Util.HTML.isSupportSVG()){
		this.convertToPoints = this._traceSVG;
	} else {
		this.convertToPoints = this._traceCanvas;
	}
};

ViaMichelin.Api.Map.PolyLine.prototype = {
	JSEwireHub : {
		"HTMLEvent_click" : [
			{event:"HTMLEvent_click", method:"JSEInstance.cancelBubbling", thisObj:"JSEInstance"},
			{event:"HTMLEvent_click", method:"JSEInstance._triggerClickEvent", thisObj:"JSEInstance"}
		],
		"HTMLEvent_mouseover" : [
		                         {event:"HTMLEvent_mouseover", method:"JSEInstance.cancelBubbling", thisObj:"JSEInstance"},
		                         {event:"HTMLEvent_mouseover", method:"JSEInstance._triggerMouseoverEvent", thisObj:"JSEInstance"}
		],
		"HTMLEvent_mouseout" : [
		                         {event:"HTMLEvent_mouseout", method:"JSEInstance.cancelBubbling", thisObj:"JSEInstance"},
		                         {event:"HTMLEvent_mouseout", method:"JSEInstance._triggerMouseoutEvent", thisObj:"JSEInstance"}
		 ],
		"HTMLEvent_mousedown" : [],
		"HTMLEvent_mouseup" :  [{event:"HTMLEvent_mouseup", method:"JSEInstance._checkMouseUp", thisObj:"JSEInstance"}],
		"HTMLEvent_mousemove" :  [],
		"HTMLEvent_mousewheel" : [],
		"HTMLEvent_contextmenu" : [
			{event:"HTMLEvent_contextmenu", method:"JSEInstance.cancelBubbling", thisObj:"JSEInstance"},
			{event:"HTMLEvent_contextmenu", method:"JSEInstance.preventDefAction", thisObj:"JSEInstance"}
		],
		"onClick" : [
			          {event:"onClick", method:"JSEInstance._bubblePosition", thisObj:"JSEInstance"},
			          {event:"onClick", method:"JSEInstance._showExpand", thisObj:"JSEInstance"}
		],
		"onMouseOver" : [],
		"onMouseOut" : [],
		"onExpandOpen" : [],
		"onExpandClose" : []
	},
	initialConf : {
		coords 			: [],
		strokeColor 	: "#000",
		strokeOpacity 	: 1,
		strokeWeight 	: 2,
		fillColor 		: "",
		fillOpacity 	: 1,
		type			: "polyLine",
		needClipping 	: true,
		htm				: ""
	},
	_sensivity	: 6, //in px
	_mouseOut	: false,
	_mouseOver	: false,
	_pathPx		: [],
	getGeoBoundaries : function(){
		var x0, y0, x1, y1;
		for (var i=0; i < this.initialConf.coords[0].length; i++) {
			if (i == 0) {
				x0 = this.initialConf.coords[0][i];
				y0 = this.initialConf.coords[1][i];
				x1 = x0;
				y1 = y0;
			} else {
				if (this.initialConf.coords[0][i] < x0) x0 = this.initialConf.coords[0][i];//min lon (west)
				if (this.initialConf.coords[1][i] < y0) y0 = this.initialConf.coords[1][i];//min lat (south)
				if (this.initialConf.coords[0][i] > x1) x1 = this.initialConf.coords[0][i];//max lon (east)
				if (this.initialConf.coords[1][i] > y1) y1 = this.initialConf.coords[1][i];//max lat (north)
			}
		}
		//upperLeftCorner, lowerRightCorner
		return new Array(new ViaMichelin.Api.Model.GeoCoordinates(x0,y1),new ViaMichelin.Api.Model.GeoCoordinates(x1,y0));
	},
	/**
	 * Getter de la couleur du trait
	 * @return {String} 
	 */
	getStrokeColor : function(){
		var divId = "idPoly_"+ this.uniqueIdSuffix;
		if ($isDefined($_id(divId))){
			if (!navigator.isIE) {
				return $_id(divId).getElementsByTagName('path')[0].getAttribute('stroke');
			}else{
				return $_id(divId).strokeColor;
			}
		}else{
			return "";
		}
	},
	/**
	 * Setter de la couleur du trait
	 * @param {String} newColor
	 */
	setStrokeColor : function(newColor){
		var divId = "idPoly_"+ this.uniqueIdSuffix;
		if($isDefined($_id(divId)) &&  $isDefined(newColor)) {			
			if (!navigator.isIE) {
				$_id(divId).getElementsByTagName('path')[0].setAttribute('stroke', newColor);
			}else{
				$_id(divId).strokeColor = newColor;
			}	
		}
	},
	/**
	 * Getter de l'opacité du trait
	 * @return {float} 
	 */
	getStrokeOpacity : function(){
		var divId = "idPoly_"+ this.uniqueIdSuffix;
		if ($isDefined($_id(divId))){
			if (!navigator.isIE) {
				return $_id(divId).getElementsByTagName('path')[0].getAttribute('stroke-opacity');
			} else {
				var element=null;
				if(navigator.isIE6 || navigator.isIE7 || navigator.isIE8){
					element = $_id(divId).getElementsByTagName('stroke');
				} else {
					element = $_id(divId).getElementsByTagName('v:stroke');
				}
				return (element && element.length) ? element.getAttribute("opacity") : 1;
			}
		}else{
			return "";
		}
	},
	/**
	 * Setter de l'opacité du trait
	 * @param {float} newOpacity
	 */
	setStrokeOpacity : function(newOpacity){
		var divId = "idPoly_"+ this.uniqueIdSuffix;
		if($isDefined($_id(divId)) && $isDefined(newOpacity)) {
			if (!navigator.isIE) {
				$_id(divId).getElementsByTagName('path')[0].setAttribute('stroke-opacity', parseFloat(newOpacity));
			} else {
				var element = null;
				var value = "'"+parseFloat(newOpacity)+"'";
				
				if(navigator.isIE6 || navigator.isIE7){
					element = $_id(divId).getElementsByTagName('stroke');
				} else {
					element = $_id(divId).getElementsByTagName('v:stroke');
				}
				
				if (element && element.length) {
					//element[0].setAttribute("opacity", value);
					element[0].opacity = parseFloat(newOpacity);
				}
			}
		}
	},
	/**
	 * Getter de l'épaisseur du trait
	 * @return {integer} 
	 */
	getStrokeWeight : function(){
		var divId = "idPoly_"+ this.uniqueIdSuffix;
		if ($isDefined($_id(divId))){
			if (!navigator.isIE) {
				return parseInt($_id(divId).getElementsByTagName('path')[0].getAttribute('stroke-width'));
			}else{
				return parseInt($_id(divId).strokeWeight);
			}
		}else{
			return "";
		}
	},
	/**
	 * Setter de l'épaisseur du trait
	 * @param {float} newWidth
	 */
	setStrokeWeight : function(newWidth){
		var divId = "idPoly_"+ this.uniqueIdSuffix;
		if($isDefined($_id(divId)) &&  $isDefined(newWidth)) {			
			if (!navigator.isIE) {
				$_id(divId).getElementsByTagName('path')[0].setAttribute('stroke-width', parseInt(newWidth));
			}else{
				$_id(divId).strokeWeight = parseInt(newWidth) + "px";
			}	
		}
	},
	/**
	 * retourne le chemin courant
	 * @return {Array}
	 */
	getPath : function(){
		return this._convertCoordsStructure(this.initialConf.coords, "[{lon,lat}]");
	},	
	/**
	 * remplace le chemin courant
	 * @param {Array} tab
	 */	
	setPath : function(coords){
		this.initialConf.coords = this._convertCoordsStructure(coords, "[[lon],[lat]]");
		this.psp = 0;
		this.aSimplifiedPoints = new Array();

		//Pour mettre à jour la polyline sur la carte
		if (this.mapObject != null) {
			this.mapObject.removeLayer(this);
			this.mapObject.addLayer(this);	
		}
	},
	/**
	 * Convertie si besoin un simple tableau de GeoCoordinates en deux tableau : un de longitude et l'autre de latitude
	 * @param {Array} tab
	 * @return {Array}
	 */
	_convertCoordsStructure : function(coords, outFormat){
		var newTab = new Array();
		
		if("[[lon],[lat]]" == outFormat){
			if(coords.length > 0 && typeof(coords[0].lon)!="undefined" && coords[0].lon != null){
				newTab[0] = new Array();
				newTab[1] = new Array();		
				for(var i=0; i<coords.length; i++){
					newTab[0].push(coords[i].lon);
					newTab[1].push(coords[i].lat);
				}
			} else {
				newTab = coords;
			}
			//Close the polygon
			if( (this.initialConf.type == "polygon") 
					&& ( (newTab[0][0] != newTab[0][newTab[0].length-1]) 
					|| (newTab[1][0] != newTab[1][newTab[1].length-1])) ){
				newTab[0].push(newTab[0][0]);
				newTab[1].push(newTab[1][0]);
			}
		}else{//[{lon,lat}]
			if(coords.length > 0 && typeof(coords[0].lon)!="undefined" && coords[0].lon != null){
				newTab = coords;
			} else {
				for(var i=0; i<coords[0].length; i++){
					newTab.push({lon: coords[0][i], lat: coords[1][i]});
				}
			}	
			//Close the polygon
			if( (this.initialConf.type == "polygon") 
					&& (newTab[0] != newTab[newTab.length-1])){
				newTab.push(newTab[0]);
			}
		}
		return newTab;
	},
	/**
	 * @param {double} pVMLon
	 * @param {double} pVMLat
	 * @param {double} oVMLon
	 * @param {double} oVMLat
	 * @param {int} psp
	 * @return {boolean}
	 */
	_validatePoint : function(pVMLon,pVMLat,oVMLon,oVMLat,psp){
		if ((Math.abs(pVMLon-oVMLon)*Math.pow(10,5) > Math.ceil(psp))||(Math.abs(pVMLat-oVMLat)*Math.pow(10,5) > Math.ceil(psp))) return true;
		else return false;
		return true;
	},
	_simplifyPoints : function(psp)
	{
		var arrPointsLon = new Array();
		var arrPointsLat = new Array();
		var i = 0;
		var prevPointLon = null;
		var prevPointLat = null;
		//alert(this.initialConf.coords[0][0]+","+ this.initialConf.coords[1][0])
		while (i < this.initialConf.coords[0].length) {
			var longi = this.initialConf.coords[0][i];
			var lati = this.initialConf.coords[1][i];
			if ( (0 == psp) || (i == 0) || (i == this.initialConf.coords[0].length-1) 
					|| (this._validatePoint(prevPointLon,prevPointLat,this.initialConf.coords[0][i],this.initialConf.coords[1][i],psp))) {
				arrPointsLon.push(this.initialConf.coords[0][i]);
				arrPointsLat.push(this.initialConf.coords[1][i]);
				prevPointLon = this.initialConf.coords[0][i];
				prevPointLat = this.initialConf.coords[1][i];
			}
			i++;
		}
		this.aSimplifiedPoints = [arrPointsLon, arrPointsLat];
	},
	/**
	 * Découpe de points en plusieurs sous ensemble de points
	 * @param {ViaMichelin.Api.Map} mapObj
	 * @return {Array} 
	 * 
	 * Exemple de retour : 
	 * Indice 0 contient toutes les longitude
	 * Indice 1 contient toutes les latitudes
	 * [
	 *    [
	 *       ["longitudes du tonçon 1"], ["longitudes du tonçon 2"], etc.
	 *    ],
	 *    [
	 *       ["latitudes du tonçon 1"], ["latitudes du tonçon 2"], etc.
	 *    ]
	 * ]
	 */
	_clipPoints : function(mapObj) {
		var arrPointsLon = new Array();
		var arrPointsLat = new Array();
		var onw = mapObj.getNorthWest();
		var one = mapObj.getNorthEast(); 
		var osw = mapObj.getSouthWest();
		var ose = mapObj.getSouthEast();
		var northExtract = Math.max(one.lat, onw.lat);
		var southExtract = Math.min(ose.lat, osw.lat);
		var eastExtract = Math.max(one.lon, ose.lon);
		var westExtract = Math.min(onw.lon, osw.lon);
		var deltaNorthing = (northExtract - southExtract) * 0.5;
		var deltaEasting = (eastExtract - westExtract) * 0.5;
		northExtract += deltaNorthing;
		southExtract -= deltaNorthing;
		eastExtract += deltaEasting;
		westExtract -= deltaEasting;
		var arrIdx = 0;
		var i = 0;
		var previousPointLon = null;
		var previousPointLat = null;
		var previousPointInExtract = false;
		arrPointsLon[arrIdx] = new Array();
		arrPointsLat[arrIdx] = new Array();
		if (this.aSimplifiedPoints[0].length > 0) {
			previousPointLon = this.aSimplifiedPoints[0][0];
			previousPointLat = this.aSimplifiedPoints[1][0];
		}
		while (i < this.aSimplifiedPoints[0].length) {
			var currentPointLon = this.aSimplifiedPoints[0][i];
			var currentPointLat = this.aSimplifiedPoints[1][i];
			var northSection = Math.max(previousPointLat, currentPointLat);
			var southSection = Math.min(previousPointLat, currentPointLat);
			var eastSection = Math.max(previousPointLon, currentPointLon);
			var westSection = Math.min(previousPointLon, currentPointLon);
			if ((southSection < northExtract) && (northSection > southExtract) && (eastSection > westExtract) && (westSection < eastExtract)) {
				if ((previousPointLon != null)&&(!previousPointInExtract)) {
					arrPointsLon[arrIdx].push(previousPointLon);
					arrPointsLat[arrIdx].push(previousPointLat);
				}
				arrPointsLon[arrIdx].push(currentPointLon);
				arrPointsLat[arrIdx].push(currentPointLat);
				previousPointInExtract = true;
			} else {
				if (previousPointInExtract) {
					arrPointsLon[arrIdx].push(currentPointLon);
					arrPointsLat[arrIdx].push(currentPointLat);
					arrIdx++;
					arrPointsLon[arrIdx] = new Array();
					arrPointsLat[arrIdx] = new Array();
				}
				previousPointInExtract = false;
			}
			previousPointLon = currentPointLon;
			previousPointLat = currentPointLat;
			i++;
		}
		
		return [arrPointsLon, arrPointsLat];
	},
	_traceVML : function(obj,func){
		this.mapObject = obj;
		this.HTMLElement.innerHTML = "";

		var ps = obj.getPixelSize()
		if ( (this.psp != ps) || (0 == ps)){
			this._simplifyPoints(ps);
		}
		var clippedPolys = this._clipPoints(obj);
		
		this.psp = obj.getPixelSize();
		for (var j=0; j < clippedPolys[0].length; j++) 
		{
			var arrPointsLon = clippedPolys[0][j];
			var arrPointsLat = clippedPolys[1][j];
			var strPoints = "";
			for (var i = 0; i < arrPointsLon.length; i++)
			{
				var geo = new ViaMichelin.Api.Model.GeoCoordinates(arrPointsLon[i], arrPointsLat[i]);
				var point = func.call(obj,geo);
				strPoints += point.x + "px," + point.y + "px";
				if (i < arrPointsLon.length - 1) strPoints += ",";
			}
			var HTMLNode = document.createElement("v:polyline"); 
			HTMLNode.expando = this;
			HTMLNode.id = "idPoly_"+ this.uniqueIdSuffix;
			HTMLNode.style.position = "absolute";
			HTMLNode.style.cursor = "pointer";
			HTMLNode.setAttribute("points", strPoints);
			
			if (this.initialConf.type == "polygon") {
				var fillNode = document.createElement("v:fill"); 
				fillNode.setAttribute("opacity", parseFloat(this.initialConf.fillOpacity));
				fillNode.setAttribute("color", this.initialConf.fillColor);
				HTMLNode.appendChild(fillNode);
			} else {
				HTMLNode.setAttribute("filled", "false");
			}
			
			HTMLNode.setAttribute("strokeColor", this.initialConf.strokeColor);
			HTMLNode.setAttribute("strokeWeight", this.initialConf.strokeWeight + "px");
			
			if (this.initialConf.strokeOpacity) {
				var itiStroke = document.createElement("v:stroke"); 
				itiStroke.setAttribute("opacity", parseFloat(this.initialConf.strokeOpacity));
				itiStroke.opacity = this.initialConf.strokeOpacity;
				HTMLNode.appendChild(itiStroke);
			}
			HTMLNode.oncontextmenu = function(e)
			{
				if (navigator.isIE) e = window.event;
				this.parentNode.expando.stimulate("HTMLEvent_contextmenu", e, this.parentNode.expando);
				this.parentNode.expando.cancelBubbling(e);
			};
			this.HTMLElement.appendChild(HTMLNode);
			this._plugEventListeners(HTMLNode, this.reportHTMLEvent);
		}
		
		this.HTMLElement.style.top = 0 + 'px';
		this.HTMLElement.style.left = 0 + 'px';
	},
	/**
	 * Dessine la polyline en CANVAS
	 * @param {object} obj
	 * @param {object} func
	 */
	_traceCanvas : function(obj,func) {
		this.mapObject = obj;
		this.HTMLElement.innerHTML = "";
		this.unplugWire(new JSEWire("HTMLEvent_mouseclick",this.cancelBubbling,this));
		this.unplugWire(new JSEWire("HTMLEvent_mousedown",this.cancelBubbling,this));
		this.unplugWire(new JSEWire("HTMLEvent_mousemove",this.cancelBubbling,this));
		this.unplugWire(new JSEWire("HTMLEvent_mouseup",this.cancelBubbling,this));
		
		this.HTMLElement.style.top = 0 + 'px';
		this.HTMLElement.style.left = 0 + 'px';
		if (this.psp != obj.getPixelSize()) this._simplifyPoints(obj.getPixelSize());
		var clippedPolys = this._clipPoints(obj);
		this.psp = obj.getPixelSize();
		
		for (var j=0; j < clippedPolys[0].length; j++) {
			var arrPointsLon = clippedPolys[0][j];
			var arrPointsLat = clippedPolys[1][j];
			
			if (clippedPolys[0][j][0]) {
				var HTMLNode = document.createElement("canvas"); 
				HTMLNode.expando = this;
				HTMLNode.style.position = "absolute";
				
				if ($isDefined(obj.mapContainer) && $isDefined(obj.mapContainer.style)) {
					HTMLNode.width = parseInt(obj.mapContainer.style.width);
					HTMLNode.height = parseInt(obj.mapContainer.style.height);
				}
				
				if (HTMLNode.getContext) {
					var geo = new ViaMichelin.Api.Model.GeoCoordinates(arrPointsLon[0], arrPointsLat[0]);
					var point = func.call(obj,geo);
					var minX, minY, maxX, maxY;
					minX = maxX = point.x;
					minY = maxY = point.y;
					var points = new Array();
					points.push(point);
					
					for (var i = 1; i < arrPointsLon.length; i++) {
						var geo = new ViaMichelin.Api.Model.GeoCoordinates(arrPointsLon[i], arrPointsLat[i]);
						var point = func.call(obj,geo);
						points.push(point);
						if (point.x < minX)	minX = point.x;
						if (point.y < minY)	minY = point.y;
						if (point.x > maxX)	maxX = point.x;
						if (point.y > maxY)	maxY = point.y;
					}
					
					HTMLNode.style.left = minX - parseInt(this.initialConf.strokeWeight) + 'px';
					HTMLNode.style.top = minY - parseInt(this.initialConf.strokeWeight) + 'px';
					HTMLNode.width = maxX - minX + 2*parseInt(this.initialConf.strokeWeight);
					HTMLNode.height = maxY - minY + 2*parseInt(this.initialConf.strokeWeight);
					
					var ctx = HTMLNode.getContext("2d");
					ctx.lineWidth = parseInt(this.initialConf.strokeWeight);
					ctx.strokeStyle =  this.initialConf.strokeColor;
					
					if (this.initialConf.strokeOpacity) ctx.globalAlpha = parseFloat(this.initialConf.strokeOpacity);
					ctx.lineJoin = "round";
					ctx.moveTo((points[0].x - minX),(points[0].y - minY));
					
					for (var i=0;i < points.length; i++) {
						ctx.lineTo((points[i].x - minX  + parseInt(this.initialConf.strokeWeight)),(points[i].y - minY  + parseInt(this.initialConf.strokeWeight)));
					}
					ctx.stroke();
				}
				this.HTMLElement.appendChild(HTMLNode);
				
				// Brancher les écouteurs
				/*
				HTMLNode.expando = this;
				HTMLNode.onclick = this.reportCanvaHTMLEvent;
				HTMLNode.onmousedown = this.reportCanvaHTMLEvent;
				HTMLNode.onmouseup = this.reportCanvaHTMLEvent;
				HTMLNode.onmousemove = this.reportCanvaHTMLEvent;
				HTMLNode.onmouseover = this.reportCanvaHTMLEvent;
				HTMLNode.onmouseout = this.reportCanvaHTMLEvent;
				HTMLNode.oncontextmenu = this.reportCanvaHTMLEvent;
				
				if (HTMLNode.addEventListener) {
					HTMLNode.addEventListener('DOMMouseScroll', this.reportCanvaHTMLEvent, false);
				}
				HTMLNode.onmousewheel = this.reportCanvaHTMLEvent;*/
			}
		}
	},
	/**
	 * Dessine la polyline en SVG
	 * @param {object} obj
	 * @param {object} func
	 */
	_traceSVG : function(obj,func) {
		this._pathPx = [];
		this.HTMLElement.innerHTML = "";
		this.HTMLElement.style.top = 0 + 'px';
		this.HTMLElement.style.left = 0 + 'px';
		var ps = obj.getPixelSize()
		if ( (this.psp != ps) || (0 == ps)){
			this._simplifyPoints(ps);
		}
		var clippedPolys = this._clipPoints(obj);
		this.psp = obj.getPixelSize();
		
		for (var j=0; j < clippedPolys[0].length; j++) {
			var arrPointsLon = clippedPolys[0][j];
			var arrPointsLat = clippedPolys[1][j];
			
			var geo = new ViaMichelin.Api.Model.GeoCoordinates(arrPointsLon[0], arrPointsLat[0]);
			var point = func.call(obj,geo);
			var minX, minY, maxX, maxY;
			minX = maxX = point.x;
			minY = maxY = point.y;
			
			for (var i = 1; i < arrPointsLon.length; i++) {
				var geo = new ViaMichelin.Api.Model.GeoCoordinates(arrPointsLon[i], arrPointsLat[i]);
				var point = func.call(obj,geo);
				if (point.x < minX)
					minX = point.x;
				if (point.y < minY) 
					minY = point.y;
				if (point.x > maxX) 
					maxX = point.x;
				if (point.y > maxY) 
					maxY = point.y;
			}
			
			var margin = 2 * parseInt(this.initialConf.strokeWeight);
			var geo = new ViaMichelin.Api.Model.GeoCoordinates(arrPointsLon[1], arrPointsLat[1]);
			var point = func.call(obj,geo);			
			var strPoints = "M " + parseInt(point.x-minX + margin)  + " " + (point.y-minY+margin);
			
			for (var i = 2; i < arrPointsLon.length; i++) {
				var geo = new ViaMichelin.Api.Model.GeoCoordinates(arrPointsLon[i], arrPointsLat[i]);
				var point = func.call(obj,geo);
				strPoints += " L " + parseInt(point.x-minX + margin)  + ' ' + (point.y-minY+margin);
			}
			
			var HTMLNode  = document.createElementNS("http://www.w3.org/2000/svg", "svg");
			HTMLNode.expando = this;
			HTMLNode.id = "idPoly_"+ this.uniqueIdSuffix;
			HTMLNode.setAttribute('version', '1.1');
			//HTMLNode.style.zIndex = this.zIndex;
			HTMLNode.style.left = minX - margin +"px"; 
			HTMLNode.style.top = minY - margin +"px"; 
			HTMLNode.style.position = "absolute";
			//HTMLNode.style.pointerEvents = "stroke";				
			HTMLNode.style.width = maxX - minX + 2*parseInt(this.initialConf.strokeWeight) + margin + "px";
			HTMLNode.style.height = maxY - minY + 2*parseInt(this.initialConf.strokeWeight) + margin + "px";	
			HTMLNode.setAttribute("overflow" , "hidden");
			HTMLNode.setAttribute("viewBox" , "0 0 "+ (maxX - minX + 2*parseInt(this.initialConf.strokeWeight)+margin) + " " + ( maxY - minY + 2*parseInt(this.initialConf.strokeWeight)+margin));
			var SVGPath = document.createElementNS('http://www.w3.org/2000/svg', 'path');
			SVGPath.style.cursor='pointer';
			SVGPath.setAttribute('stroke-linejoin', 'round');
			SVGPath.setAttribute('stroke-linecap', 'round');
			SVGPath.setAttribute('stroke-width', this.initialConf.strokeWeight);
			SVGPath.setAttribute('stroke', this.initialConf.strokeColor);
			SVGPath.setAttribute('stroke-opacity', this.initialConf.strokeOpacity);
			SVGPath.setAttribute('d', strPoints);
			
			if (this.initialConf.type == "polygon") {
				SVGPath.setAttribute('fill', this.initialConf.fillColor);
				SVGPath.setAttribute('fill-opacity', this.initialConf.fillOpacity);
			} else {
				SVGPath.setAttribute('fill', 'none');				
			}
			
			this._plugEventListeners(SVGPath, this.reportHTMLEvent);		
			
			SVGPath.oncontextmenu = function(e) {
				this.parentNode.expando.stimulate("HTMLEvent_contextmenu", e, this.parentNode.expando);
				this.parentNode.expando.cancelBubbling(e);
			};
			
			HTMLNode.appendChild(SVGPath);	
			this.HTMLElement.appendChild(HTMLNode);
		}
	},
	_plugEventListeners : function(node, fct) {
		node.expando = this;
		node.onclick = fct;
		node.onmousedown = fct;
		node.onmouseup = fct;
		node.onmousemove = fct;
		node.onmouseover = fct;
		node.onmouseout = fct;
		node.oncontextmenu = fct;
		if (node.addEventListener) {
			node.addEventListener('DOMMouseScroll', fct, false);
		}
		node.onmousewheel = fct;
	},
	_unplugEventListeners : function (node, fct) {
		node.onclick = "";
		node.onmousedown = "";
		node.onmouseup = "";
		node.onmousemove = "";
		node.oncontextmenu = "";
		node.onmouseover = "";
		node.onmouseout = "";
		if (node.removeEventListener) node.removeEventListener('DOMMouseScroll', fct, false);
		node.onmousewheel = "";
	},
	_bubblePosition : function(e) {
		if (this.mapObject != null && this.expandLayer != null) {
			this.coords = this.mapObject.convertMousePositionToGeo(e);
			this.expandLayer.coords = this.coords; 
		}
	},
	toString : function(){
		var value="[ViaMichelin.Api.Map.PolyLine]";
		
		for(var item in this.initialConf){
			value+= "{"+ item +" : "+ this.initialConf[item]+"} ";
		}
		return value;
	}
};