JSEPackage("ViaMichelin.Api.Model.Itinerary.RoadSheet");

JSEImportApi("JSE.HTML");
JSEImportApi("ViaMichelin.Api.Map.ComplexLayer");
JSEImportApi("ViaMichelin.Api.Map.Layer");
JSEImportApi("ViaMichelin.Api.Util");
JSEImportApi("ViaMichelin.Api.Util.HTML");
JSEImportApi("ViaMichelin.Api.Util.Other");
JSEImportApi("ViaMichelin.Api.UI.Drawing.ItineraryBuilder");
JSEImportApi("ViaMichelin.Api.Model.Poi");
JSEImportApi("ViaMichelin.Api.Model.GeoCoordinates");
JSEImportApi("ViaMichelin.Api.Model.Itinerary");
JSEImportApi("ViaMichelin.Api.Model.Itinerary.ItineraryStep");

JSEImportApi("ViaMichelin.Api.Itinerary.RoadSheetView");

/**
 * Modèle de données pour la feuille de route.
 * @class ViaMichelin.Api.Model.Itinerary.RoadSheet
 * @constructor
 * @param {String} labels
 * @param {String} imgSrc
 * @param {String} unit
 * @param {Array} steps
 * @param {Object} schema
 * @param {Object} customizedPictos	:
 * @param {Object} customizedMap	: { step : { size : { width : "", height : "" } }, full : idem }
 */
ViaMichelin.Api.Model.Itinerary.RoadSheet = function(labels, imgSrc, unit, steps, schema, customizedPictos, customizedMap, tollVehicule) {
	_VM_ROADSHEETPOIS = null;
	this.tollVehicule = "car";
	if($isNotEmpty(tollVehicule))
		this.tollVehicule = tollVehicule;
		
	this.labels = labels;
	this.imgSrc = imgSrc;
	this.imgSrc2 = ViaMichelin.Api.Constants.System.ApiImgPath + "all/iti/";
	this.unit = unit;
	this._customizedPictos = customizedPictos;
	this._customizedMap = customizedMap;
	/* On ajoute un espace insécable à l'unité de distance */
	//if(this.unit!=null && !(this.unit.indexOf("&nbsp;")>-1)){
	//	this.unit = "&nbsp;" + this.unit;
	//}
	
	this.lang = "";
	this.zone = "";
	
	this.alertes = new Object();
	this.alertes.nbAlertesSignalisations = 0;
	this.alertes.liste = new Array();
	
	this._poiList = [];
	if($isDefined(steps)) this.steps = this._transform(steps);
	
	if($isDefined(schema)){
		this._schema = this._decodeSchema(schema);
	}
};

ViaMichelin.Api.Model.Itinerary.RoadSheet.prototype = {
	/** 
	 * Liste des instructions de l'itinéraire
	 * @type Array<ViaMichelin.Api.Model.Itinerary.ItineraryStep>
	 */
	steps			: null,
	/**
	 * Liste des alertes présentent sur l'itinéraire
	 * @type Object
	 * <ul>
	 *  <li><b>Détails : </b></li>
	 * 	<li>nbAlertesSignalisations {int}</li>
	 * 	<li>liste {Array}</li>
	 * </ul>
	 */
	alertes			: null,
	tollVehicule	: null,
	labels			: null,
	imgSrc			: null,
	imgSrc2			: null,
	unit			: null,
	zone			: null,
	lang			: null,
	_VMRoadsheetsIndex : null,
	_schema			: null,
	/**
	 * Utilisé pour faire des comparaisons avec les routes parcourues.
	 * @type String
	 */
	lastStepWay		: null,
	/**
	 * Liste de POIS présent sur le parcours
	 * @param {Array}
	 */
	_poiList		: [],
	/**
	 * Retourne la liste des POIS (si une recherche de POIS a été demandé)
	 * @return {Array}
	 */
	getPoiList	: function(){
		return this._poiList;
	},
	/**
	 * Récupère le layer avec les pastilles ainsi que les étapes dessus
	 * @param {Boolean} hideCorridor
	 * @return {ViaMichelin.Api.Map.ComplexLayer}
	 */
	getLayer : function(hideCorridor) {
		var cply = new ViaMichelin.Api.Map.ComplexLayer();
		for (var i = 0; i < this.steps.length; i++) {
			if (this.steps[i].type != "M") {
				var step = this.steps[i];
				var html = typeof step.number == 'number' ? ViaMichelin.Api.Util.drawMiniPastilleBleue(step.number) : step.number;
				var vmly = typeof step.number == 'string' ?
					new ViaMichelin.Api.Map.Layer(step.coords, html, -1, -12):
					new ViaMichelin.Api.Map.Layer(step.coords, html, -5, -5);

				vmly.setExpandLayer(step.description + "<br/><img src='" + step.mapURL+"' style='width:250px;height:200px;' alt=''/>");
				
				vmly.contextMenuTemplate = "/htm/obf/iti/roadsheetContextV10.2.0.htm";
				
				if (typeof(step.number) == 'string') {
					if (typeof(step.stepIndex) != "undefined") {
						vmly.stepIndex = step.stepIndex;
						vmly.contextMenuTemplate = "/htm/obf/iti/roadsheetStepContextV10.2.0.htm";	
					}
				} 
				
				cply.addLayer(vmly);
				step.layer = vmly;
				
				/*
				if (!hideCorridor){
					
					cply.addLayer(this.getPoisLayer());
					
					// On affiche es POI corridor
					for (var j = 0; j < step.details.length; j++) {
						var sousStep = step.details[j];
						
						if (sousStep.bloc == "CORRPOI") {
							log(sousStep);
							
							var vmly = new ViaMichelin.Api.Map.Layer(sousStep.coords, sousStep.number, -15, -8);
							
							vmly.HTMLElement.style.cursor = "pointer";
							vmly.poi = sousStep.poi;
							
							// On cast le poi en objet poi
							ToJSEObject(vmly.poi);
							log(vmly.poi);
							
							vmly.poi = vmly.poi.cast(ViaMichelin.Api.Model.Poi);
							
							vmly.expando = this;
							vmly.plugWire(new JSEWire("HTMLEvent_mousedown",this._createExpand,vmly));
							vmly.HTMLElement.alt = sousStep.name;
							vmly.HTMLElement.title = sousStep.name;
							cply.addLayer(vmly);
						}
					}
				}*/
			}
		}
		
		if (!hideCorridor){			
			cply.addLayer(this.getPoisLayer());
		}
		
		return cply;
	},
	/**
	 * Retourne un layer contenant la liste des POIS
	 * @return {ViaMichelin.Api.Map.ComplexLayer}
	 */
	getPoisLayer : function(disconnectEvent){
		if(!$isDefined(disconnectEvent)) disconnectEvent = false;
		var cLayer = new ViaMichelin.Api.Map.ComplexLayer();
		
		// On affiche des POI corridor
		for (var j = 0; j < this._poiList.length; j++) {
			var poi 	= this._poiList[j];
			var pictoPoi = poi.picto, offsetX=0, offsetY=0;
			var customPicto = null;
			
			if($isDefined(poi) && $isDefined(this._customizedPictos)){
				if($isDefined(this._customizedPictos[poi.productId])){
					customPicto = this._customizedPictos[poi.productId]
				} else if($isDefined(this._customizedPictos["default"])){
					customPicto = this._customizedPictos["default"]
				}
			}
			
			if($isDefined(customPicto)){
				var picto = customPicto;
				var size = "";
				
				if($isDefined(picto.offset)){
					offsetX = - picto.offset.x;
					offsetY = - picto.offset.y;
				} else if ($isDefined(picto.size)){
					offsetX = - picto.size.width/2;
					offsetY = - picto.size.height/2;
				}
				
				if($isDefined(picto.size)){
					size = " style=\"width:"+picto.size.width+"px;height:"+picto.size.height +"px\" ";
				}
				
				pictoPoi = "<img class=\"poi\" src=\""+ picto.url +"\" "+size+"/>";
			}
			
			var vmly = new ViaMichelin.Api.Map.Layer(poi.location.coords, pictoPoi, offsetX, offsetY);
				vmly.HTMLElement.style.cursor = "pointer";
				vmly.poi 				= poi;
				vmly.expando 			= this;
				vmly.HTMLElement.alt 	= poi.name;
				vmly.HTMLElement.title 	= poi.name;
			
			var self = this;
			vmly.plugWire(new JSEWire("HTMLEvent_mousedown", function(){
				cLayer.stimulate("onClickPOI", this.poi);
				if(!disconnectEvent){
					self._createExpand(this);
				}
			}, vmly));
			
			cLayer.addLayer(vmly);
		}
		
		return cLayer;
	},
	/**
	 * Retourne le calque des evenements
	 * @return {ViaMichelin.Api.Map.ComplexLayer}
	 */
	getWarningsLayer : function () {
		var cply = new ViaMichelin.Api.Map.ComplexLayer();
		cply.expando = this;
		
		var roadsheet = this;
		var lenSteps = this.steps.length;
		for (var i = 0; i < lenSteps; i++) {
			var step = roadsheet.steps[i];
			for (var j = 0; j < step.details.length; j++) {				
				var sousStep = step.details[j];
				if ((sousStep.type == "M")					
					|| (sousStep.type == "c")					
					|| (sousStep.type == "vir")
					|| (sousStep.type == "lim")
					|| (sousStep.type == "dec")
					|| (sousStep.type == "S")
					|| (sousStep.type == "rad")
					)
				{ 
					if (sousStep.coords && sousStep.coords.lon && (sousStep.coords.lon != 0) && sousStep.coords.lat && (sousStep.coords.lat != 0) && sousStep.picto && (sousStep.picto != "")) {
						var html = sousStep.picto;
						var vmly = new ViaMichelin.Api.Map.Layer(sousStep.coords, html, -1, -12);						
						vmly.HTMLElement.style.cursor = "pointer";	
						vmly.sousStep = sousStep;
						//vmly.plugWire(new JSEWire("HTMLEvent_mousedown",this._createExpandWarningElement,vmly));	
						var expandLayer = '<div class="sPODBubble">' + vmly.sousStep.description + '</div>';
						vmly.setExpandLayer(expandLayer);
						cply.addLayer(vmly);
					}
				}
			}
		}
		cply.isItiWarnings = true;
		return cply;
	},
	/**
	 * Ajoute un POI en cours d'extraction (JSON) à la liste des POIS
	 * @param {Object} poi
	 */
	_addToPoiList : function(poi){
		this._poiList.push(poi);		
	},
	/**
	 * Transforme le json en objet Roadsheet
	 * @param {Array} roadSheet : 
	 * @param {String} str		: 
	 * @param {String} isDetail	: indique si la transformation se fait sur un detail
	 */
	_transform : function(roadSheet, str, isDetail){
		if(!$isDefined(isDetail)) isDetail = false;
		
		var clonedRS = new Array();
		for (var i=0; i < roadSheet.length; i++)
		{
			clonedRS[i] = new Array();
			for (var j=0; j < roadSheet[i].length; j++)
			{
				clonedRS[i].push(roadSheet[i][j]);
			}
		}
		roadSheet = clonedRS;
		
		//Stockage de l'objet roadsheet au niveau global pour gestion clics sur itis via/depuis/vers dans datasheet CORRPOI
		if (typeof(window._VMRoadsheets) == "undefined") {
			window._VMRoadsheets = new Array();
		}
		
		if (this._VMRoadsheetsIndex == null) {
			window._VMRoadsheets.push(this);
			this._VMRoadsheetsIndex = window._VMRoadsheets.length - 1;
		}
		
		var result = new Array();
		var firstWay = true;
		var lastWay = "";
		var etapesIds = new Array("A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z");
		var etapesIdsIndex = 0;
		var stepNumber = 1;
		var oVMStep;
		
		for (var i=0; i < roadSheet.length; i++) {
			var strSortie = "";
			
			if (roadSheet[i] != null) {
				
				if(!isDetail){
					//Dans le cas ou l'on ne serait pas dans un detail on initialise la variable
					this.lastStepWay = null;
				}
				
				oVMStep = new ViaMichelin.Api.Model.Itinerary.ItineraryStep();
				roadSheet[i].bloc = roadSheet[i][0];
				roadSheet[i].splice(0,1);
				oVMStep.type = roadSheet[i].bloc;
				
				var space = " ";
				if (!isDetail) space = "";
				
				// Type Voie
				if (roadSheet[i].bloc == "V") {
					//if(!isDetail)
						//console.log("V("+i+"):",roadSheet[i]);
						
					var obj = ["coords","tDist","tTime","lDist","lTime","inAngle","inCity","inProxy","inDirLib","type","voies","pays","inRondPoint","nameRondPoint","inStyle","isWithDirection","distPleasant","timePleasant","url","detail","schema","immediately"];
					this._configureObject(roadSheet[i], obj);
					
					roadSheet[i].picto = "";
					roadSheet[i].text = "";
					oVMStep.number = stepNumber++;
					oVMStep.panels = this._computePanneaux(roadSheet[i]);
					oVMStep.description = this._wayToHTML(roadSheet[i],firstWay,lastWay, false, isDetail);
					oVMStep.shortDescription = this._wayToHTML(roadSheet[i],firstWay,lastWay, true, isDetail);
					
					if (roadSheet[i].picto.indexOf("<br>" != -1)) roadSheet[i].picto = roadSheet[i].picto.replaceAll("<br>","");
					if (roadSheet[i].picto.indexOf("<br/>" != -1)) roadSheet[i].picto = roadSheet[i].picto.replaceAll("<br/>","");
					
					oVMStep.picto = roadSheet[i].picto;
					oVMStep.partialDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].lDist,this.unit);
					oVMStep.partialTime		= ViaMichelin.Api.Util.makeHour(roadSheet[i].lTime);
					
					if (isDetail) {
						oVMStep.totalDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].tDist - roadSheet[i].lDist, this.unit);
						oVMStep.totalTime = ViaMichelin.Api.Util.makeHour(roadSheet[i].tTime - roadSheet[i].lTime);
					} else {
						oVMStep.totalDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].tDist,this.unit);
						oVMStep.totalTime = ViaMichelin.Api.Util.makeHour(roadSheet[i].tTime);
					}
					
					oVMStep.coords = new ViaMichelin.Api.Model.GeoCoordinates(roadSheet[i].coords.lon, roadSheet[i].coords.lat);
					if (roadSheet[i].distPleasant) {
						oVMStep.distPleasant = ViaMichelin.Api.Util.makeDist(roadSheet[i].distPleasant,this.unit);
					}
					firstWay = false;
					lastWay = this.getWayListToString(roadSheet[i][10]);
					oVMStep.mapURL = roadSheet[i].url;
					
					// Analyse du schema
					if($isDefined(roadSheet[i].schema) && roadSheet[i].schema.length>=3){
						oVMStep.schema2 = roadSheet[i].schema[0];
						oVMStep.schema  = [ roadSheet[i].schema[1], roadSheet[i].schema[2] ];
					} else if($isDefined(roadSheet[i].schema) && roadSheet[i].schema.length == 2){
						oVMStep.schema2 = [];
						oVMStep.schema  = [ roadSheet[i].schema[0], roadSheet[i].schema[1] ];
					} else {
						oVMStep.schema2= [];
						oVMStep.schema = [];
					}
					
					//oVMStep.schema = roadSheet[i].schema;
					
					oVMStep.voies = roadSheet[i].voies;
					oVMStep.roadInfo = [roadSheet[i].pays].concat(this._convertRoadType(parseInt(roadSheet[i].type)));
					oVMStep.isoCountry  = roadSheet[i].pays;
				}
				
				// Type étape
				if (roadSheet[i].bloc == "C") {
					//console.log("C("+i+"):",roadSheet[i]);
					var obj = ["coords","tDist","tTime","lDist","lTime","city","cityLine","addressLine","isNeighbor","type","reflex","landing","url","detail","schema","importance"];
					this._configureObject(roadSheet[i], obj);
					
					var isStep = true;
					var blocStr = "";
					var number = stepNumber;
					var isShort = false;
					if (roadSheet[i].landing === "") {
						if (roadSheet[i].type == 0) {
							if(navigator.isIE6)
								number = "<img class=\"greenFlag\" src=\""+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/s.gif\" width=\"16\" height=\"15\" class=\"greenFlag\" style=\"filter:progid:DXImageTransform.Microsoft.AlphaImageLoader(src='"+ this.imgSrc2 + "greenFlag.png',sizingMethod='scale')+\"/>";
							else if(navigator.userAgent.indexOf('Firefox/2') != -1)
								number = '<img class=\"greenFlag\" src="'+ ViaMichelin.Api.Constants.System.ApiImgPath + 'all/data/itifdr/drapeau_depart.gif" alt="" />';
							else
								number = "<img class=\"greenFlag\" src='" + this.imgSrc2 + "greenFlag.png' width='16' height='15' alt='' />";
							blocStr += "<b class='color'>"+this.labels["lab_depart"]+"</b><b>";
							if (roadSheet[i].addressLine == "") blocStr += roadSheet[i].city + this.labels["lab_centre"];
							else blocStr += roadSheet[i].city + ", " + roadSheet[i].addressLine;
							oVMStep.picto = "start";
						}
						else if (roadSheet[i].type == 1) {
							if(navigator.isIE6)
								number = "<img class=\"redFlag\" src=\""+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/s.gif\" width=\"16\" height=\"15\" style=\"filter:progid:DXImageTransform.Microsoft.AlphaImageLoader(src='"+ this.imgSrc2 + "redFlag.png',sizingMethod='scale')+\"/>";
							else if(navigator.userAgent.indexOf('Firefox/2') != -1)
								number = '<img class="redFlag" src="'+ ViaMichelin.Api.Constants.System.ApiImgPath + 'all/data/itifdr/drapeau_arrivee.gif" alt="" />';
							else
								number = "<img class='redFlag' src='" + this.imgSrc2 + "redFlag.png' width='16' height='15' alt='' />";
								
							blocStr += "<b class='color'>"+this.labels["lab_arrivee"] +"</b><b>";
							if (roadSheet[i].addressLine == "") blocStr += roadSheet[i].city + this.labels["lab_centre"];
							else blocStr += roadSheet[i].city + ", " + roadSheet[i].addressLine;
							oVMStep.picto = "end";
						}
						else if (roadSheet[i].type == 2) {
							if(navigator.isIE6)
								number = "<img class=\"blueFlag\" src=\""+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/s.gif\" width=\"16\" height=\"15\" style=\"filter:progid:DXImageTransform.Microsoft.AlphaImageLoader(src='"+ this.imgSrc2 + "blueFlag.png',sizingMethod='scale')+\"/>";
							else if(navigator.userAgent.indexOf('Firefox/2') != -1)
								number = '<img class="blueFlag" src="'+ ViaMichelin.Api.Constants.System.ApiImgPath + 'all/data/itifdr/drapeau_etape.gif" alt="" />';
							else
								number = "<img class='blueFlag' src='" + this.imgSrc2 + "blueFlag.png' width='16' height='15' alt='' />";
								
							blocStr += "<b>"+this.labels["lab_etape"]+" " + etapesIds[etapesIdsIndex] + this.labels["lab_dots"];
							oVMStep.stepIndex = etapesIdsIndex;
							if (roadSheet[i].addressLine == "") blocStr += roadSheet[i].city + this.labels["lab_centre"];
							else blocStr += roadSheet[i].city + ", " + roadSheet[i].addressLine;
							etapesIdsIndex++;
							oVMStep.picto = "step";
						}
					}
					else {
						if (roadSheet[i].landing === 0) blocStr += "<b>"+this.labels["lab_sortir"] + roadSheet[i].city + this.labels["lab_unknown2"];
						if (roadSheet[i].landing === 1) blocStr += "<b>"+this.labels["lab_entrer"] + roadSheet[i].city + this.labels["lab_unknown2"];
						if (roadSheet[i].landing === 2) blocStr += "<b>"+this.labels["lab_iti"] + roadSheet[i].city + this.labels["lab_unknown2"];
						if (roadSheet[i].landing === 3) {
							if (roadSheet[i].lDist < 450) {
								blocStr += this.labels["lab_a"] + " <b>" + roadSheet[i].city;
								isShort = true;
							}
							else blocStr += "<b>"+ this.labels["lab_traverser"] + roadSheet[i].city;
						}
						isStep = false;
						if (((roadSheet[i].landing === 0) || (roadSheet[i].landing === 3))&&(roadSheet[i].detail.length > 0)&&(!isShort)) {
							strSortie = this.labels["lab_sortieDe"] + roadSheet[i].city;
						}
						
					}
					if (isShort) blocStr += "</b>"+ this.labels["lab_endContinue"];
					if (!isShort) blocStr += "</b>";

					if (!isStep) stepNumber++;

					oVMStep.number = number;
					oVMStep.panels = "";
					oVMStep.description = blocStr;
					oVMStep.shortDescription = oVMStep.description;
					oVMStep.partialDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].lDist,this.unit);
					//oVMStep.totalTime = ViaMichelin.Api.Util.makeHour(roadSheet[i].tTime);
					//oVMStep.totalDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].tDist,this.unit);
					
					if (isDetail) {
						oVMStep.totalDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].tDist - roadSheet[i].lDist, this.unit);
						oVMStep.totalTime = ViaMichelin.Api.Util.makeHour(roadSheet[i].tTime - roadSheet[i].lTime);
					} else {
						oVMStep.totalDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].tDist,this.unit);
						oVMStep.totalTime = ViaMichelin.Api.Util.makeHour(roadSheet[i].tTime);
					}
					
					oVMStep.coords = new ViaMichelin.Api.Model.GeoCoordinates(roadSheet[i].coords.lon, roadSheet[i].coords.lat);
					oVMStep.mapURL = roadSheet[i].url;
					oVMStep.schema = roadSheet[i].schema;
					
					// Calcul du type de route
					// on prends le type de route de la première voie rencontré
					var len = roadSheet[i].detail.length;
					if (len > 0){
						for(var k=0; k<len; k++){
							var detail = roadSheet[i].detail[k];
							if(detail[0]=="V"){
								oVMStep.roadInfo = [detail[12]].concat(this._convertRoadType(parseInt(detail[10])));
								break;
							}
						}
					}
				}
				
				if (roadSheet[i].bloc == "F") {
					var obj = ["coords","tDist","tTime","lDist","lTime","iso","id","message","url","detail"];
					this._configureObject(roadSheet[i], obj);
					var flag = "<img src='" + ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/flags/" + roadSheet[i].iso + ".png' alt='' />";
					oVMStep.number = flag;
					oVMStep.panels = "";
					oVMStep.description = "<b><span class=\"frontierText\">"+ roadSheet[i].message +"</span><a class='sCountryLink frontierLink' href='javascript:;' onclick='window.open(\"/web/CityToCityFrontierXML?msgId="+roadSheet[i].id+"&lang="+this.lang+"&zone="+this.zone+"\",\"\",\"status=no,scrollbars=yes,width=375,height=400\")'>" + roadSheet[i].message + "</a></b>";
					oVMStep.description += "<span class='sCountryMessage'>" + roadSheet[i].message + "</span>";
					oVMStep.shortDescription = oVMStep.description;
					oVMStep.partialDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].lDist,this.unit);
					oVMStep.totalTime = ViaMichelin.Api.Util.makeHour(roadSheet[i].tTime);
					oVMStep.totalDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].tDist,this.unit);
					oVMStep.coords = new ViaMichelin.Api.Model.GeoCoordinates(roadSheet[i].coords.lon, roadSheet[i].coords.lat);
					oVMStep.mapURL = roadSheet[i].url;
				}
				
				if (roadSheet[i].bloc == "POI") {
					var obj = ["coords","tDist","tTime","lDist","lTime","nom","addressLine","cityLine","logo","productId","id","url","detail"];
					this._configureObject(roadSheet[i], obj);
					oVMStep.number = "<img src='"+roadSheet[i].logo+"' alt='' />";
					oVMStep.panels = "";
					oVMStep.description = "<b>" + roadSheet[i].nom + " - " + roadSheet[i].addressLine + ", " + roadSheet[i].cityLine + "</b>";
					oVMStep.shortDescription = oVMStep.description;
					oVMStep.partialDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].lDist,this.unit);
					oVMStep.totalTime = ViaMichelin.Api.Util.makeHour(roadSheet[i].tTime);
					oVMStep.totalDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].tDist,this.unit);
					oVMStep.coords = new ViaMichelin.Api.Model.GeoCoordinates(roadSheet[i].coords.lon, roadSheet[i].coords.lat);
					oVMStep.mapURL = roadSheet[i].url;
				}
				
				// Type POI CORRIDOR
				if (roadSheet[i].bloc == "CORRPOI") {
					var obj = ["coords","tDist","tTime","lDist","lTime","nom","logo","logoId", "productId","id","poiId","addressLine","city","countryISOCode","photo"];
					this._configureObject(roadSheet[i], obj);
					var poiTemp = new ViaMichelin.Api.Model.Poi();
					
					poiTemp.id 			= roadSheet[i].id;
					poiTemp.productId	= roadSheet[i].productId;
					poiTemp.name		= roadSheet[i].nom;
					poiTemp.location	= {
						coords 				: new ViaMichelin.Api.Model.GeoCoordinates(roadSheet[i].coords.lon, roadSheet[i].coords.lat),
						formattedAddressLine: roadSheet[i].addressLine,
						formattedCityLine 	: roadSheet[i].city,
						streetNumber 		: null,
						streetLabel 	: null,
						city 			: roadSheet[i].city,
						postalCode 		: null,
						reflexId 		: null,
						area 			: null,
						countryISO 		: roadSheet[i].countryISOCode,
						countryOfficial : null,
						countryLabel 	: null
					};
					poiTemp.photo = roadSheet[i].photo;
					
					// on détermine le type de POI
					poiTemp.principalType = ViaMichelin.Api.Util.Poi.getType(poiTemp.productId);
					poiTemp.poipko = roadSheet[i].poiId;
					
					this._addToPoiList(poiTemp);
					
					var _tmpHTML = "";
					//_tmpHTML += "<div class='hand bleu' onclick='window._VMRoadsheets["+this._VMRoadsheetsIndex+"].stimulate(\"onClickPoiInternal\",\""+ poiTemp.id +"-"+ poiTemp.productId +"\" )'><div style='float:left;'>"+oVMStep.number+"</div>&nbsp;<b>" + roadSheet[i].nom + "</b></div>"
					
					var temp = poiTemp.id+"-"+poiTemp.productId+"-"+poiTemp.poipko;
					_tmpHTML += "<div class='hand bleu poiToClick' param='"+ temp +"'><div style='float:left;'>"+oVMStep.number+"</div>&nbsp;<b>" + roadSheet[i].nom + "</b></div>";
					_tmpHTML += "<div id='VMRoadSheetDetail_"+roadSheet[i].poiId+"' style='display:none;'>" + roadSheet[i].addressLine + "</div>";
					
					var logoSrc = roadSheet[i].logo.replace("../../..", ViaMichelin.Api.Constants.System.ApiPath);
					logoSrc = ViaMichelin.Api.Util.Other.protocolAdapter(logoSrc);
					
					poiTemp.picto 			= oVMStep.picto = oVMStep.number = '<img src="'+ logoSrc +'" class="poi" alt="" />';
					oVMStep.pictoUrl 		= logoSrc;
					oVMStep.panels 			= "";
					oVMStep.bloc 			= "CORRPOI";
					oVMStep.name 			= roadSheet[i].nom;
					oVMStep.description 	= _tmpHTML;
					oVMStep.shortDescription = oVMStep.description;
					oVMStep.poi 			= poiTemp;
					
					if ((typeof(window._VMRoadsheets[this._VMRoadsheetsIndex]._VM_ROADSHEETPOIS) == "undefined")||(window._VMRoadsheets[this._VMRoadsheetsIndex]._VM_ROADSHEETPOIS == null)) _VMRoadsheets[this._VMRoadsheetsIndex]._VM_ROADSHEETPOIS = new Array();
					window._VMRoadsheets[this._VMRoadsheetsIndex]._VM_ROADSHEETPOIS.push(poiTemp);
					
					oVMStep.partialDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].lDist,this.unit);
					oVMStep.totalTime 		= ViaMichelin.Api.Util.makeHour(roadSheet[i].tTime);
					oVMStep.totalDistance 	= ViaMichelin.Api.Util.makeDist(roadSheet[i].tDist,this.unit);
					oVMStep.coords 			= new ViaMichelin.Api.Model.GeoCoordinates(roadSheet[i].coords.lon, roadSheet[i].coords.lat);
				}
				
				// Normalement n'est plus utilisé avec l'ITI 2 de l'API REST
				if (roadSheet[i].bloc == "CP") {
					var obj = ["tDist","tTime","url","detail"];
					this._configureObject(roadSheet[i], obj);
					var dirLab = this.labels["lab_dirCP"];
					if ((roadSheet[i+1])&&(roadSheet[i+1][0]=="V")) dirLab += " " + this.labels["lab_dirVers"] + " <b>" + this.getWayListToString(roadSheet[i+1][10]) + "</b>";

					oVMStep.number = stepNumber++;
					oVMStep.panels = "";
					oVMStep.picto = "<img class='itifdr18' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/sortie.png' alt='' />";
					oVMStep.description = dirLab;
					oVMStep.shortDescription = oVMStep.description;
					oVMStep.partialDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].lDist,this.unit);
					oVMStep.totalTime = ViaMichelin.Api.Util.makeHour(roadSheet[i].tTime);
					oVMStep.totalDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].tDist,this.unit);
					oVMStep.mapURL = roadSheet[i].url;
				}
				
				// Type : sortie
				if (roadSheet[i].bloc == "S") {
					var obj = ["coords","tDist","tTime","lDist","lTime","type","panels","direction","style","pays","wayLabel","schema","immediately"];
					this._configureObject(roadSheet[i], obj);
					var dirLab = "";
					var classname = "dirPan";
					if (isDetail) classname+= " isDetail";
					
					// Sans panneau
					if (roadSheet[i].panels instanceof Array && roadSheet[i].panels.length == 0){
						if (roadSheet[i].immediately)
							dirLab = this.labels["lab_dirCP"].toLowerCase();
						else 
							dirLab = this.labels["lab_dirCP"];
					// Avec panneau
					} else {
						var text = this.labels["lab_dirLab"];
						if (roadSheet[i].immediately) {
							text = this.labels["lab_dirLab"].toLowerCase();
						}
						
						dirLab = text
							+"<div class='"+classname+"' schema='"+ roadSheet[i].panels[1] +"' distance='"+ ViaMichelin.Api.Util.makeDist(roadSheet[i].tDist,this.unit) +"'>"
							+ this._buildPanneaux("S", roadSheet[i].panels[0], roadSheet[i].style, roadSheet[i].pays, roadSheet[i].wayLabel)
							+"</div>";
					}
					
					// Ajout de la mention "Puis immédiatement" si-besoin
					if (roadSheet[i].immediately){
						dirLab = ViaMichelin.Api.Util.Other.formattingPunctuation(",", this.labels["lab_immediately"], dirLab, ViaMichelin.Api.Constants.System.ApiLang);
					}
					
					oVMStep.number = stepNumber++;
					oVMStep.panels = "";
					oVMStep.picto  = "<img class='itifdr18' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/sortie.png' alt='' />";
					oVMStep.description = dirLab;
					oVMStep.shortDescription = oVMStep.description;
					oVMStep.partialDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].lDist,this.unit);
					oVMStep.totalTime = ViaMichelin.Api.Util.makeHour(roadSheet[i].tTime);
					oVMStep.totalDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].tDist,this.unit);
					oVMStep.coords = new ViaMichelin.Api.Model.GeoCoordinates(roadSheet[i].coords.lon, roadSheet[i].coords.lat);
					oVMStep.schema = roadSheet[i].schema;				
				}
				
				// Type : Passage à proximité, tunnel, pont
				// ["c", "", 309477.0, 10422, 0, 0, "Beaune", "", "", true, "", 0, [[194, 1081, 0.0, 0.0], [119, 1075, 149, 1087], 1]]
				if (roadSheet[i].bloc == "c") {
					var obj = ["coords","tDist","tTime","lDist","lTime","city","cityLine","addressLine","isNeighbor","type","reflex","schema"]; //,"landing","url","detail"];
					this._configureObject(roadSheet[i], obj);
					var picto = "";
					
					if (roadSheet[i].type == 1) {
						picto = "<img class='itifdr18' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/pont.png' alt='' />";
						dirLab = "<b> "+roadSheet[i].city + "</b>";
						oVMStep.className = 'pont';
						oVMStep.model = "bridge";
					} else if (roadSheet[i].type == 2) {
						picto = "<img class='itifdr18' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/tunnel.png' alt='' />";
						dirLab = "<b> "+roadSheet[i].city + "</b>";
						oVMStep.className = 'tunnel';
						oVMStep.model = "tunnel";
					} else {
						if (roadSheet[i].isNeighbor) dirLab = this.labels["lab_proxPassage"]+" <b>" + roadSheet[i].city + "</b>";
						else dirLab = this.labels["lab_traverserDe"]+" <b>" + roadSheet[i].city + "</b>";
						oVMStep.className 	= 'proximity';
						oVMStep.model 		= "proximity";
					}
					
					oVMStep.number		= stepNumber++;
					oVMStep.panels		= "";
					oVMStep.picto		= picto;
					oVMStep.city		= roadSheet[i].city;
					oVMStep.description = dirLab;
					oVMStep.shortDescription	= oVMStep.description;
					oVMStep.partialDistance 	= ViaMichelin.Api.Util.makeDist(roadSheet[i].lDist,this.unit);
					oVMStep.totalTime	= ViaMichelin.Api.Util.makeHour(roadSheet[i].tTime);
					oVMStep.totalDistance		= ViaMichelin.Api.Util.makeDist(roadSheet[i].tDist,this.unit);
					oVMStep.coords		= new ViaMichelin.Api.Model.GeoCoordinates(roadSheet[i].coords.lon, roadSheet[i].coords.lat);
					//oVMStep.mapURL	= roadSheet[i].url;
					oVMStep.schema		= roadSheet[i].schema;
				}
				
				if (roadSheet[i].bloc == "M") {
					var obj = ["coords","tDist","tTime","message","type","url","detail"];
					this._configureObject(roadSheet[i], obj);
					var picto = "";
					if ((roadSheet[i].type != "")&&(roadSheet[i].type != 0)) picto = "<img class='itifdr18' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/msg"+roadSheet[i].type+".png' alt='' />";

					oVMStep.number = "";
					oVMStep.panels = "";
					oVMStep.picto = picto;
					if(roadSheet[i].message == "lab_security" || roadSheet[i].message==""){
						oVMStep.description = this.labels["lab_security"];
					} else {
						oVMStep.description = roadSheet[i].message;
					}
					//oVMStep.description = (roadSheet[i].message == "lab_security") ? this.labels["lab_security"] : roadSheet[i].message;
					oVMStep.shortDescription = oVMStep.description;
					oVMStep.partialDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].lDist,this.unit);
					oVMStep.totalTime = ViaMichelin.Api.Util.makeHour(roadSheet[i].tTime);
					oVMStep.totalDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].tDist,this.unit);
					oVMStep.coords = new ViaMichelin.Api.Model.GeoCoordinates(roadSheet[i].coords.lon, roadSheet[i].coords.lat);
					oVMStep.mapURL = roadSheet[i].url;
				}
				
				// Type Radar
				if (roadSheet[i].bloc == "rad") {
					var obj = ["coords","tDist","tTime","message","speed","schema"];
					this._configureObject(roadSheet[i], obj);
					var picto = "<img class='itifdr18' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/msg13.png' alt='' />";
					
					oVMStep.number = "";
					oVMStep.panels = "";
					oVMStep.picto = picto;
					oVMStep.description = (roadSheet[i].message == "lab_security") ? this.labels["lab_security"] : roadSheet[i].message;
					oVMStep.shortDescription = oVMStep.description;
					oVMStep.partialDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].lDist,this.unit);
					oVMStep.totalTime = ViaMichelin.Api.Util.makeHour(roadSheet[i].tTime);
					oVMStep.totalDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].tDist,this.unit);
					oVMStep.coords = new ViaMichelin.Api.Model.GeoCoordinates(roadSheet[i].coords.lon, roadSheet[i].coords.lat);
					oVMStep.mapURL = roadSheet[i].url;
					oVMStep.className = 'radar';
					oVMStep.speed  = roadSheet[i].speed;
					oVMStep.schema = roadSheet[i].schema;
					
					this.alertes.nbAlertesSignalisations++;
					var item = new Object();
						item.picto = oVMStep.picto;
						item.description = oVMStep.description;
						item.totalDistance = oVMStep.totalDistance;
					this.alertes.liste.push(item);
				}
				
				// Type RoadNumber
				if (roadSheet[i].bloc == "rnu") {
					var obj = ["coords","tDist","tTime","road","pays","schema"];
					this._configureObject(roadSheet[i], obj);
					oVMStep.number		= stepNumber++;
					oVMStep.coords = new ViaMichelin.Api.Model.GeoCoordinates(roadSheet[i].coords.lon, roadSheet[i].coords.lat);
					oVMStep.road		= roadSheet[i].road;
					oVMStep.countryISO	= roadSheet[i].pays;
					oVMStep.schema		= roadSheet[i].schema[0];
				}
				
				// Type Franchissement
				if (roadSheet[i].bloc == "fra") {
					var obj = ["coords","tDist","tTime","model","label","langue","etage","importance","pays","schema"];
					this._configureObject(roadSheet[i], obj);
					oVMStep.className	= "franchissement";
					oVMStep.number		= stepNumber++;
					oVMStep.coords		= new ViaMichelin.Api.Model.GeoCoordinates(roadSheet[i].coords.lon, roadSheet[i].coords.lat);
					oVMStep.label		= roadSheet[i].label;
					oVMStep.lang		= roadSheet[i].lang;
					oVMStep.importance	= roadSheet[i].importance;
					oVMStep.countryISO	= roadSheet[i].pays;
					oVMStep.schema		= roadSheet[i].schema;
					oVMStep.model		= roadSheet[i].model;
				}
				
				// Type virage
				if (roadSheet[i].bloc == "vir") {
					//Virage à gauche ISOLE_GAUCHE = 0;
					//Virage à droite ISOLE_DROITE = 1;
					//Zone de virage gauche ZONE_GAUCHE = 2;
					//Zone de virage gauche ZONE_DROITE = 3;
					var obj = ["coords","tDist","tTime","longueur","num","schema"];
					this._configureObject(roadSheet[i], obj);
					var picto = "";
					if (typeof(roadSheet[i].num) != "undefined" && roadSheet[i].num != 4){
						 picto = '<img class="itifdr18" src="'+ ViaMichelin.Api.Constants.System.ApiImgPath + 'all/data/itifdr/vir'+roadSheet[i].num+'.png\"  alt="" />';
					}
					oVMStep.number = "";
					oVMStep.panels = "";
					oVMStep.picto  = picto;
					oVMStep.num	   = roadSheet[i].num;
					if(roadSheet[i].num == 0){
						oVMStep.description = this.labels['lab_virage_gauche'];
					}
					else if(roadSheet[i].num == 1){
						oVMStep.description =  this.labels['lab_virage_droite'];
					}
					else if(roadSheet[i].num == 2){
						if(roadSheet[i].longueur > 150)
							oVMStep.description = this.labels['lab_zone_virage_large_g'].replaceAll('$L',ViaMichelin.Api.Util.makeDist(roadSheet[i].longueur,this.unit));
						else
							oVMStep.description =  this.labels['lab_zone_virage_small_g'];
					}
					else if(roadSheet[i].num == 3){
						if(roadSheet[i].longueur > 150)
							oVMStep.description = this.labels['lab_zone_virage_large_d'].replaceAll('$L',ViaMichelin.Api.Util.makeDist(roadSheet[i].longueur,this.unit));
						else
							oVMStep.description =  this.labels['lab_zone_virage_small_d'];
					}
					oVMStep.shortDescription = oVMStep.description;
					oVMStep.totalTime = ViaMichelin.Api.Util.makeHour(roadSheet[i].tTime);
					oVMStep.totalDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].tDist,this.unit);
					oVMStep.coords = new ViaMichelin.Api.Model.GeoCoordinates(roadSheet[i].coords.lon, roadSheet[i].coords.lat);
					oVMStep.mapURL = "";
					oVMStep.className="virage";
					oVMStep.schema = roadSheet[i].schema;
					
					this.alertes.nbAlertesSignalisations++;
					var item = new Object();
					item.picto = oVMStep.picto;
					item.description = oVMStep.description;
					item.totalDistance = oVMStep.totalDistance;
					this.alertes.liste.push(item);
				}
				
				// Type Limitation
				if (roadSheet[i].bloc == "lim") {
					var obj = ["coords","tDist","tTime","longueur","lim","schema"];
					this._configureObject(roadSheet[i], obj);
					if(roadSheet[i].lim%5 == 0)
					{
						oVMStep.number = "";
						oVMStep.panels = "";
						oVMStep.picto = '<img class="itifdr19" src="'+ ViaMichelin.Api.Constants.System.ApiImgPath + 'all/data/itifdr/vitesse'+roadSheet[i].lim+'.png\"  alt="" />';
						if(roadSheet[i].longueur > 150)
							oVMStep.description = this.labels['lab_vitesse_large'].replaceAll('$V',roadSheet[i].lim).replaceAll('$L',ViaMichelin.Api.Util.makeDist(roadSheet[i].longueur,this.unit));
						else
							oVMStep.description = this.labels['lab_vitesse_small'].replaceAll('$V',roadSheet[i].lim);
						oVMStep.shortDescription = oVMStep.description;
						oVMStep.partialDistance = "";
						oVMStep.totalTime = ViaMichelin.Api.Util.makeHour(roadSheet[i].tTime);
						oVMStep.totalDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].tDist,this.unit);
						oVMStep.coords = new ViaMichelin.Api.Model.GeoCoordinates(roadSheet[i].coords.lon, roadSheet[i].coords.lat);
						oVMStep.mapURL = "";
						oVMStep.className="limitation";
						oVMStep.schema = roadSheet[i].schema;
						
						this.alertes.nbAlertesSignalisations++;
						var item = new Object();
						item.picto = oVMStep.picto;
						item.description = oVMStep.description;
						item.totalDistance = oVMStep.totalDistance;
						this.alertes.liste.push(item);
					}
				}
				
				// Type descente
				if (roadSheet[i].bloc == "dec") {
					var obj = ["coords","tDist","tTime","longueur","pente","schema"];
					this._configureObject(roadSheet[i], obj);
					oVMStep.number = "";
					oVMStep.panels = "";
					oVMStep.picto = '<img class="itifdr18" src="'+ ViaMichelin.Api.Constants.System.ApiImgPath + 'all/data/itifdr/pente.png\" alt="" />';
					if(roadSheet[i].longueur > 150)
						oVMStep.description = this.labels['lab_pente_large'].replaceAll('$L',ViaMichelin.Api.Util.makeDist(roadSheet[i].longueur,this.unit));
					else
						oVMStep.description = this.labels['lab_pente_small'];
					oVMStep.shortDescription = oVMStep.description;
					oVMStep.partialDistance = "";
					oVMStep.totalTime = ViaMichelin.Api.Util.makeHour(roadSheet[i].tTime);
					oVMStep.totalDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].tDist,this.unit);
					oVMStep.coords = new ViaMichelin.Api.Model.GeoCoordinates(roadSheet[i].coords.lon, roadSheet[i].coords.lat);
					oVMStep.mapURL = "";
					oVMStep.className="pente";
					oVMStep.schema = roadSheet[i].schema;
					
					this.alertes.nbAlertesSignalisations++;
					var item = new Object();
					item.picto = oVMStep.picto;
					item.description = oVMStep.description;
					item.totalDistance = oVMStep.totalDistance;
					this.alertes.liste.push(item);
				}
				
				// TODO : ou est-ce c'est utilisé ?
				if (roadSheet[i].bloc == "pan") {
					var obj = ["coords","tDist","tTime","lDist","lTime","panneau","style","pays","infoRoute"];
					this._configureObject(roadSheet[i], obj);

					var message = this.labels["lab_suivreDir"] + "<div class='dirPan' schema'"+ roadSheet[i].panneau[1] +"'>"+ this._buildPanneaux("pan", roadSheet[i].panneau,roadSheet[i].style,roadSheet[i].pays, roadSheet[i].infoRoute)+"</div>";
					oVMStep.number = stepNumber++;
					oVMStep.panels = "";
					oVMStep.picto = "";
					oVMStep.description = message;
					oVMStep.shortDescription = oVMStep.description;
					oVMStep.partialDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].lDist,this.unit);
					oVMStep.totalTime = ViaMichelin.Api.Util.makeHour(roadSheet[i].tTime);
					oVMStep.totalDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].tDist,this.unit);
					oVMStep.coords = new ViaMichelin.Api.Model.GeoCoordinates(roadSheet[i].coords.lon, roadSheet[i].coords.lat);
				}
				
				// Type : Péage
				// Exemple : ["P", {"lon": 4.732077, "lat": 45.97347}, 434765.0, 14237, 0.0, 0, "Villefranche-Limas", "EUR", 3150, 4920, 7680, 10550, 10550, 10550, 1880, [[125, 688, 125, 700]]]
				if (roadSheet[i].bloc == "P") {
					var obj = ["coords","tDist","tTime","lDist","lTime","nom","monnaie","car","caravan","pl2","pl3","pl4","pl5","moto","url","detail"];
					this._configureObject(roadSheet[i], obj);

					var picto = "";
					if (roadSheet[i].monnaie == "") picto = "<img class='itifdr18' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/peage.png' alt=''/>";
					else picto = "<img class='itifdr18' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/peageP.png' alt='' />";
					var message = "";
					
					if (roadSheet[i].monnaie != "") {
						oVMStep.cost = parseInt(roadSheet[i][this.tollVehicule]);
						message = roadSheet[i].nom + this.labels["lab_peage"] + this._formatNumber(oVMStep.cost/100) + " " + roadSheet[i].monnaie;
					}
					else message = roadSheet[i].nom + this.labels["lab_ticket"];
					
					oVMStep.number 		= stepNumber++;
					oVMStep.panels 		= "";
					oVMStep.picto 		= picto;
					oVMStep.description = message;
					oVMStep.shortDescription 	= oVMStep.description;
					oVMStep.partialDistance 	= ViaMichelin.Api.Util.makeDist(roadSheet[i].lDist,this.unit);
					oVMStep.totalTime 	= ViaMichelin.Api.Util.makeHour(roadSheet[i].tTime);
					oVMStep.totalDistance 		= ViaMichelin.Api.Util.makeDist(roadSheet[i].tDist,this.unit);
					oVMStep.coords 		= new ViaMichelin.Api.Model.GeoCoordinates(roadSheet[i].coords.lon, roadSheet[i].coords.lat);
					oVMStep.mapURL 		= roadSheet[i].url;
					oVMStep.className 	= "toll";
				}
				
				if (roadSheet[i].bloc == "D") {
					var obj = ["coords","tDist","tTime","lDist","lTime","libelle","side","type","style","pays","url","detail"];
					this._configureObject(roadSheet[i], obj);

					var message = this.labels["lab_suivreDir"] + this._buildStyledDirections(roadSheet[i].libelle,roadSheet[i].style,roadSheet[i].pays) + "";

					oVMStep.number = stepNumber++;
					oVMStep.panels = "";
					oVMStep.picto = "";
					oVMStep.description = message;
					oVMStep.shortDescription = oVMStep.description;
					oVMStep.partialDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].lDist,this.unit);
					oVMStep.totalTime = ViaMichelin.Api.Util.makeHour(roadSheet[i].tTime);
					oVMStep.totalDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].tDist,this.unit);
					oVMStep.coords = new ViaMichelin.Api.Model.GeoCoordinates(roadSheet[i].coords.lon, roadSheet[i].coords.lat);
					oVMStep.mapURL = roadSheet[i].url;
				}
				
				
				if (roadSheet[i].bloc == "R") {
					var obj = ["coords","tDist","tTime","lDist","lTime","nbVoie","name","pays","url","detail"];
					this._configureObject(roadSheet[i], obj);

					var dirLab = "";
					if (roadSheet[i].nameRondPoint != "") dirLab = this.labels["lab_rond1"] + " " + roadSheet[i].nameRondPoint+", " +this.labels["lab_rond2"] +" " + this._computeRoundWording(roadSheet[i].nbVoie);
					else dirLab = this.labels["lab_rondFull"] + roadSheet[i].nbVoie + this._computeRoundWording(roadSheet[i].nbVoie);
					var strPicto = "";
					if ((roadSheet[i].pays == "GBR")||(roadSheet[i].pays == "IRL")) strPicto = "GBR";

					oVMStep.number = stepNumber++;
					oVMStep.panels = "";
					oVMStep.picto = "<img class='itifdr18' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/rondPoint"+strPicto+".png'  alt='' />";
					oVMStep.description = dirLab;
					oVMStep.shortDescription = oVMStep.description;
					oVMStep.partialDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].lDist,this.unit);
					oVMStep.totalTime = ViaMichelin.Api.Util.makeHour(roadSheet[i].tTime);
					oVMStep.totalDistance = ViaMichelin.Api.Util.makeDist(roadSheet[i].tDist,this.unit);
					oVMStep.coords = new ViaMichelin.Api.Model.GeoCoordinates(roadSheet[i].coords.lon, roadSheet[i].coords.lat);
					oVMStep.mapURL = roadSheet[i].url;
				}
				
				// Type voies privées
				if (roadSheet[i].bloc == "priv") {
					var obj = ["coords","tDist","tTime","distance"];
					this._configureObject(roadSheet[i], obj);
					var distance = ViaMichelin.Api.Util.makeDist(roadSheet[i].distance, this.unit);
					oVMStep.number 		= "";
					oVMStep.panels 		= "";
					oVMStep.picto 		= "<img class=\"itifdr18\" src=\""+ ViaMichelin.Api.Constants.System.ApiImgPath +"all/data/itifdr/msg3.png\"/>";
					oVMStep.description = this.labels["lab_private_road"] + distance;
					oVMStep.shortDescription 	= oVMStep.description;
					oVMStep.partialDistance 	= "";
					oVMStep.totalTime 			= "";
					oVMStep.totalDistance 		= distance;
					oVMStep.coords 				= new ViaMichelin.Api.Model.GeoCoordinates(roadSheet[i].coords.lon, roadSheet[i].coords.lat);
					oVMStep.mapURL 				= "";
				}
				
				// Type voies fermées
				// 0=jamais ferme, 1=ferme une fois par jour 2=une fois par mois 3=une fois par an
				if (roadSheet[i].bloc == "clos") {
					var obj = ["coords","tDist","tTime","distance","closeIndex"];
					this._configureObject(roadSheet[i], obj);
					
					if (roadSheet[i].closeIndex == 0) {
						continue;
					}
					
					oVMStep.panels 			= "";
					oVMStep.coords 			= new ViaMichelin.Api.Model.GeoCoordinates(roadSheet[i].coords.lon, roadSheet[i].coords.lat);
					oVMStep.totalDistance 	= ViaMichelin.Api.Util.makeDist(roadSheet[i].tDist,this.unit);
					oVMStep.totalTime 		= ViaMichelin.Api.Util.makeHour(roadSheet[i].tTime);
					oVMStep.partialDistance = "";
					oVMStep.closeIndex 		= roadSheet[i].closeIndex;
					oVMStep.description 	= this.labels["lab_closed_road"];
					oVMStep.shortDescription = oVMStep.description;
					oVMStep.number 			= "";
					oVMStep.panels 			= "";
					oVMStep.picto 			= '<img class="itifdr18" src="'+ ViaMichelin.Api.Constants.System.ApiImgPath +'all/data/itifdr/msg3.png\"/>';
					oVMStep.mapURL 			= "";
				}
				
				if ((roadSheet[i].detail)&&(roadSheet[i].detail.length > 0)) oVMStep.details = this._transform(roadSheet[i].detail, strSortie, true);
				result.push(oVMStep);
			}
		}
		if (str) {
			var sVMStep = new ViaMichelin.Api.Model.Itinerary.ItineraryStep();
			sVMStep.number = "";
			sVMStep.panels = "";
			sVMStep.picto = "";
			sVMStep.description = str;
			sVMStep.shortDescription = str;
			sVMStep.partialDistance = "";
			sVMStep.totalTime = "";
			sVMStep.totalDistance = "";
			sVMStep.coords = "";
			sVMStep.mapURL = "";
			result.push(sVMStep);
		}
		
		return result;
	},
	/**
	 * Convertie une type de route en un type compatible pour l'ItineraryBuilder
	 * @param {String} roadType
	 * @return {Array} 
	 */
	_convertRoadType : function(type){
		var roadType = [0, 9];
		
		switch(parseInt(type)){
			// Route
			case 0 : 
				roadType = [5, 13];
				break;
			// Piétons
			case 4 :
				roadType = [4, 9];
				break;
			// Routes pour automobiles
			case 5 :
				roadType = [0, 18];
				break;
			// Autoroute
			case 6 :
				roadType = [6, 20];
				break;
			// Bretelle autoroute
			case 7 :
				roadType = [0, 18];
				break;
			default :
				roadType = [0, 9];
				break;
		}
		
		return roadType;
	},
	_configureObject : function(json, obj) {
		for (var i=0; i < obj.length; i++) {
			if ((!json[obj[i]])||(obj[i] == "detail")) json[obj[i]] = json[i];
		}
	},
	_createExpandWarningElement : function () {
		var expandLayer = '<div class="sPODBubble">' + this.sousStep.description + '</div>';
		this.setExpandLayer(expandLayer);
	},
	/**
	 * Crée le contenu de la bubble H,R,T
	 */
	_createExpand : function(layer) {
		var oDiv = document.createElement("div");
			oDiv.innerHTML = 	'<div style="width:200px;background-color:#FFF;">' +
									layer.poi.name + 
								'</div>';
		
		layer.setExpandLayer(oDiv);
	},
	
	_createExpand2 : function() {
		var oDiv = document.createElement("div");
		oDiv.innerHTML =	'<div style="width:300px;height:140px;background-color:#FFF;">' +
								'<div style="width:300px;height:40px;">' + this.poi.bubbleLogo +
									'<div style="height:20px;font-size:14px;font-weight:bold;" class="hand txtLink">' + this.poi.name + '</div>' +
								'</div>' +
								'<div style="width:200px;height:100px;float:left;">' +
									'<div style="width:200px;height:50px;font-size:12px;font-weight:100;padding-top:10px;">' + this.poi.displayAddress + '</div>' +
									'<div><span class="hand poiIti">' + _application.trads["DEPUIS"] + '</span> | <span class="hand poiIti">' + _application.trads["VERS"] + '</span> | <span class="hand poiIti">' + _application.trads["VIA"] + '</span> ' + _application.trads["CETTE_ADRESSE"] + '</div>' +
									'<div style="width:200px;height:40px;"><button class="btnButton btnMoreInfo">' + _application.trads["MORE_INFO"] + '</button></div>' +
								'</div>' +
								(this.poi.photo ?
								('<div style="overflow:hidden;width:95px;height:95px;float:right;padding-top:15px;">' +
									'<img id="poiPic" width="95" height="95" src="' + this.poi.photo + '"  alt="" />' +
								'</div>') : "") +
							'</div>';
		
		var o3 = oDiv.getElementsByTagName("div")[3];
		o3.expando = this.expando;
		o3.poi = this.poi;
		o3.onclick = (this.poi.isPoiBrandPlacement()) ? this.expando._clickOnBrand : this.expando._clickOnPoi;
		
		var o31 = oDiv.getElementsByTagName("button")[0];
		o31.expando = this.expando;
		o31.poi = this.poi;
		o31.onclick = (this.poi.isPoiBrandPlacement()) ? this.expando._clickOnBrand : this.expando._clickOnPoi;
		
		var o6 = oDiv.getElementsByTagName("div")[6];
		var o6s0 = o6.getElementsByTagName("span")[0];
		o6s0.poi = this.poi;
		o6s0.expando = this.expando;
		o6s0.onclick = this.expando._clickOnFromIti;
		
		var o6s1 = o6.getElementsByTagName("span")[1];
		o6s1.poi = this.poi;
		o6s1.expando = this.expando;
		o6s1.onclick = this.expando._clickOnToIti;
		
		var o6s2 = o6.getElementsByTagName("span")[2];
		o6s2.expando = this.expando;
		o6s2.poi = this.poi;
		o6s2.onclick = this.expando._clickOnViaIti;
		
		var imgs = oDiv.getElementsByTagName("img");
		for (var i=0; i<imgs.length; i++){
			if (imgs[i].id == "poiPic") {
				imgs[i].expando = this.expando;
				imgs[i].poi = this.poi;
				imgs[i].onclick = (this.poi.isPoiBrandPlacement() ? this.expando._clickOnBrand : this.expando._clickOnPoi);
			}
		}
		
		this.setExpandLayer(oDiv);
	},
	_createDivNode : function(className, inner){
		var node = document.createElement("DIV");
		if (className && className != ""){
			node.className = className;
		}
		if (inner && inner != ""){
			node.innerHTML = inner;
		}
		return node;
	},
	_createImgNode : function(className, src){
		var node = document.createElement("img");
		if (className && className != ""){
			node.className = className;
		}
		if (src && src != ""){
			node.src = src;
		}
		return node;
	},
	_getMapBlockElement : function(url) {
		var divElement = this._createDivNode('mapBlock');
		var spanElement = document.createElement('SPAN');
		var imgElement = this._createImgNode('',url);
		spanElement.innerHTML = this.labels['lab_map'];
		spanElement.className = 'txtLink';
		spanElement.onclick = function(){
			this.style.display = 'none';
		};
		
		imgElement.style.display = 'none';
		imgElement.span = spanElement;
		imgElement.onclick = function(){
			this.span.style.display = 'inl'+'ine';
			this.style.display = 'none';
		};
		divElement.appendChild(imgElement);
		divElement.appendChild(spanElement);
		
		return divElement;
	},
	/**
	 * Construit les panneaux
	 * @param {String} type
	 * @param {Array} panneau
	 * @param {String} style
	 * @param {String} pays
	 * @param {Array} infoRoute
	 * @return {String}
	 */
	_buildPanneaux : function(type, panneau, style, pays, voies, options){
		var importantSortie, inter, sVoies, sSorties, separator, listeVoiesSeul, voiesSeul, item2, i, nbsLiPanneau, nbsLiVoies, j, libelle, obj, stringVoie, classe, biDimPanneau, biDimPanneauLength, stringPanneau, j, item, panneau2, suppr;
		
		//options
		options = options !== undefined ? options : {};
		options.displayVoies = options.displayVoies !== undefined ? options.displayVoies : Number.MAX_VALUE;
		options.voiesAndPanneau = options.voiesAndPanneau !== undefined ? options.voiesAndPanneau : "associate";
		options.displaySortie = options.displaySortie !== undefined ? options.displaySortie : true;
		
		options.displayLocalite = options.displayLocalite !== undefined ? options.displayLocalite : Number.MAX_VALUE;
		options.displayLocalite ++;//car le premier indice du tableau MultiDi est toujours undefined
		if(type == "S" && options.displaySortie == false){//on ne compte pas une sortie qui n'est pas affiché
			options.displayLocalite ++; 
		}
		
		//développé pour la france, fonctionne dans les autres pays mais dans une logique france.
		//on ne traite que les type V et S
		
		//voies = (voies && typeof voies == 'object' && voies.length > 0) ? voies : [] ;
		voies = [];
		
		nbsLiPanneau = panneau.length;
		nbsLiVoie = voies.length;
		
		//on fait la correspodance indice => attribut
		for(i = 0 ; i < nbsLiPanneau ; i ++){
			item = panneau[i];
			
			obj = ["libelle","connexiontype","mentionType","routeType","picto","direction","primarySequence","secondarySequence"];
			this._configureObject(item, obj);
		}
		
		for(i = 0 ; i < nbsLiVoie ; i ++){
			item = voies[i];
			
			obj = ["libelle","typeHierarchique","style"];
			this._configureObject(item, obj);
		}
		voies = [];
		nbsLiVoie = 0;
		
		nbsLiPanneau = panneau.length;
			
		for(i = 0 ; i < nbsLiPanneau ; i++ ){
			item = panneau[i];
			if(item.mentionType == 3){
				voiesSeul = true;
				for(j = i + 1 ; j < nbsLiPanneau ; j++ ){
					item2 = panneau[j];
					if(item.primarySequence == item2.primarySequence){
						voiesSeul = false;
					}
				}
				if(voiesSeul == true){
					
					item["display"] = false;
					voies.push([item.libelle, item.routeType]);
					
					voies[nbsLiVoie]["libelle"] = item.libelle;
					voies[nbsLiVoie]["typeHierarchique"] = item.routeType;
					
					
					nbsLiVoie ++;
				}
			}
		}
		nbsLiVoies = voies.length;

		//on traite les voies
		stringVoie = "";
		sVoies = "";
		if(nbsLiVoie > 0 && options.displayVoies > 0){
			for(i = 0 ; i < nbsLiVoie && i < options.displayVoies ; i ++){
				item = voies[i];
				
				if(item.typeHierarchique != 0){//nom de rue ou nom de route donc on affiche pas
					
					libelle = item.libelle;
					classe = "pan pan" + item.typeHierarchique;
					inter = '<span class="' + classe + '">' + libelle + '</span>';
					stringVoie += inter;
					sVoies += inter;
				}
			}
		}
		stringVoie = '<div class="voieBlock">' + stringVoie + '</div>';
		
		
		//on transforme en tableau bidimensionnelle grace au primary sequence
		biDimPanneau = [];
		
		for(i = 0 ; i < nbsLiPanneau ; i ++){
			if(!biDimPanneau[panneau[i].primarySequence]){
				biDimPanneau[panneau[i].primarySequence] = [];
			}
			biDimPanneau[panneau[i].primarySequence].push(panneau[i]);
		}
		
		//on traite le panneau en lui meme
		nbsBiDimPanneau = biDimPanneau.length;
		stringPanneau = "";
		importantSortie = false;
		sSortie = "";
		
		for(i = 0 ; i < options.displayLocalite && i < nbsBiDimPanneau; i ++){
			var isSortie = false;
			if(biDimPanneau[i]){//peut etre a undefined
				libelle = "";
				for(j = 0 ; j < biDimPanneau[i].length; j ++){
					item = biDimPanneau[i][j];
					
					if(item.display !== false){
						//on traite simplement mentionType = sortie / picto / tout les autre
						if(j < biDimPanneau[i].length - 1){
							separator = " - ";
						}
						else{
							separator = "";
						}
						
						//console.log("style:"+ style +"," , item, item.mentionType);
						if(item.mentionType == 8){//pictogramme
							libelle += '<img src="'+ ViaMichelin.Api.Constants.System.ApiImgPath + 'all/data/itifdr/picto_' + item.picto + '.png"  alt="" /> ';
						} else if(item.mentionType == 1 || item.mentionType == 2){//sortie
							isSortie = true;
							var exitImg = ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/exit_black.png";
							
							if((style == 2 && pays == "FRA") || (style == 1 && pays == "CHE") || (style==1&&pays=="ITA")){
								exitImg = ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/exit_white.png";
							}
								
							if(options.displaySortie === true){
								libelle += '<span class="mention' + item.mentionType + '"><img src="'+ exitImg +'"  alt="" />&nbsp;&nbsp;' + item.libelle + "<span> ";
							} else {
								sortie = '<span class="mention' + item.mentionType + '"><img src="'+ exitImg +'"  alt="" />&nbsp;&nbsp;' + item.libelle + "<span> ";
							}
						}
						else{
							libelle += '<span class="mention' + item.mentionType + ' typeRoute' + item.routeType + '">' + item.libelle + "</span>" + separator;
						}
					}
				}
				
				if(isSortie) classSortie= " itemSortie";
				else classSortie= "";
				
				stringPanneau += '<p class="itemPanneau'+classSortie+'" >' + libelle + '</p>';
			}
		}
		
		classe = 'panBlock panneauStyle' + style;
		stringPanneau = '<div class="' + pays + '"><div class="' + classe + '">' + stringPanneau + '</div></div>';
		
		if(options.voiesAndPanneau == "associate"){
			return '<div class="' + pays + '">' + stringVoie + stringPanneau + '</div>';
		}
		if(options.voiesAndPanneau == "not_associate"){
			
			return {
				voies	: stringVoie,
				panneau	: stringPanneau,
				sortie	: sSortie
			};
		}
	},
	_formatNumber : function(num)
	{
		num = num.toString();
		if (num.indexOf(".") != -1) 
		{
			var entier = num.substring(0, num.indexOf("."));
			var decimal = num.substr(num.indexOf(".")+1);
			if (decimal.length == 1) decimal = decimal + "" + "0";
			if (decimal.length > 2) decimal = Math.round(parseInt(decimal) / Math.pow(10,decimal.length-2));
			return entier + "." + decimal;
		} else num = num + ".00";
		return num;
	},
	_computeRoundWording : function(nSortie)
	{
		if (nSortie == 1) return this.labels["lab_one"];
		if (nSortie == 2) return this.labels["lab_two"];
		if (nSortie == 3) return this.labels["lab_three"];
		if (nSortie > 3) return this.labels["lab_more"].replace("$N", nSortie);
	},
	/**
	 * Retourne la liste des voies sur une ligne séparé par des "/"
	 * @param {Array} voies
	 * @return {String}
	 */
	getWayListToString : function(voies) {
		var w = 0;
		var value = "";
		while (w < voies.length){
			if (voies[w][0] != ""){
				if (w > 0){
					value += " / ";
				}
				value += voies[w][0];
			}
			w++;
		}
		return value;
	},
	/**
	 * @param {Object}	wayBlock
	 * @param {Boolean}	firstWay
	 * @param {String}	lastWay
	 * @param {Boolean}	dontShowPanels	:
	 * @param {Boolean}	isDetail		: indique si la représentation est celle d'un detail
	 * @return {String}
	 */
	_wayToHTML : function(wayBlock,firstWay,lastWay,dontShowPanels,isDetail) {
		var imgIti = this.imgSrc;
		var text = "";
		
		// Calcul sur les panneaux : on affiche un panneau que s'il a au moins une ville
		var showPanel = false;
		var wayListToString = this.getWayListToString(wayBlock[10]);
		var wayList = wayBlock[10];
		
		if (wayBlock.inDirLib.length>0) {
			for(var cpt=0;cpt<wayBlock.inDirLib[0].length;cpt++){
				var pan = wayBlock.inDirLib[0][cpt];
				
				if(pan[2]==4 || pan[2]==6) {
					showPanel = true;
					break;
				}
			}
		}
		
		// Si on est en ville
		if ($isNotEmpty(wayBlock.inCity)) {
			if (wayBlock.inProxy)
				text += this.labels["lab_prox"] + " <b>" + wayBlock.inCity + "</b>"+this.labels["lab_unknown"]+", ";
			else
				text += this.labels["lab_a"] + " <b>" + wayBlock.inCity + "</b>, ";
		}
		
		// Continuer, continuer tout droit, etc.
		if (wayBlock.inAngle == 0) {
			if (wayListToString == "") {
				text += this.labels["lab_continue_droit"];
			} else if (!firstWay) {
				if (lastWay == wayListToString) {
					var labContinue = this.labels["lab_continue"];
					if($isNotEmpty(text))
						labContinue = this.labels["lab_continue"].toLowerCase();
						
					text += labContinue +" " + this._mkWay(wayBlock," "+this.labels["lab_sur"]+" ")+ "<b>" + wayListToString + "</b>";
				} else {
					if (wayBlock.isWithDirection){
						if (wayList[0][0] == this.lastStepWay)
							text+= this.labels["lab_continue"];
						else
							text+= this.labels["lab_prendre"];
					} else
						text+= this.labels["lab_rond_point_continue"];
					text += this._mkWay(wayBlock,"")+ "<b>" + wayListToString + "</b>";
				}
			} else {
				if (wayBlock.isWithDirection) {
					if (wayList[0][0] == this.lastStepWay)
						text+= this.labels["lab_rond_point_continue"];
					else
						text+= this.labels["lab_prendre"];
				} else 
					text+= this.labels["lab_rond_point_continue"];
				
				text += this._mkWay(wayBlock,"") +"<b>" + wayListToString + "</b>";
			}
		}
		
		// Demi tour
		if (wayBlock.inAngle == -18 || wayBlock.inAngle == 18) {
			text += this.labels["lab_demitour"];
			text += this._mkWay(wayBlock,this.labels["lab_dots"]) + " <b>" + wayListToString + "</b>";
		}
		
		// Tourner à droite
		if (wayBlock.inAngle > 0 && wayBlock.inAngle <= 17) {
			// si pas de voies
			if (wayListToString == ""){
				if(showPanel) {
					text += this.labels["lab_takeRight"];
				} else {
					text += this.labels["lab_turnRight"];
				}
			} else {
				var immediately = this.labels["lab_immediately"];
				var label = "";
				if(text != "")
					immediately = this.labels["lab_immediately"].toLowerCase();
				
				if (wayBlock.immediately){
					text += ViaMichelin.Api.Util.Other.formattingPunctuation(",", immediately, "", ViaMichelin.Api.Constants.System.ApiLang);					
					label = this.labels["lab_takeRight"].toLowerCase();
				} else {
					if (wayList[0][0] == this.lastStepWay) {
						label = this.labels["lab_continueRight"];
					} else {
						label = this.labels["lab_takeRight"];
					}
				}
				text += ViaMichelin.Api.Util.Other.formattingPunctuation(":", label, " <b>"+ wayListToString +"</b>", ViaMichelin.Api.Constants.System.ApiLang);
			}
			wayBlock.picto = "<img class='itifdr18' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/right.png' alt='' /><br/>";
		}
		
		// Tourner à gauche
		if (wayBlock.inAngle >= -17 && wayBlock.inAngle < 0) {
			if (wayListToString == "") {
				if (showPanel) {
					text += this.labels["lab_takeLeft"];
				} else {
					text += this.labels["lab_turnLeft"];
				}
			} else {
				var immediately = this.labels["lab_immediately"];
				var label = "";
				if (text != "")
					immediately = this.labels["lab_immediately"].toLowerCase();
				
				if (wayBlock.immediately) {
					text += ViaMichelin.Api.Util.Other.formattingPunctuation(",", immediately, "" , ViaMichelin.Api.Constants.System.ApiLang);
					label = this.labels["lab_takeLeft"].toLowerCase();
				} else {
					if (wayList[0][0] == this.lastStepWay) {
						label = this.labels["lab_continueLeft"];
					} else {
						label = this.labels["lab_takeLeft"];
					}
				}
				text += ViaMichelin.Api.Util.Other.formattingPunctuation(":", label, " <b>"+ wayListToString +"</b>", ViaMichelin.Api.Constants.System.ApiLang);
			}
			wayBlock.picto = "<img class='itifdr18' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/left.png' alt='' /><br/>";
		}
		
		// Rond point
		if (wayBlock.inRondPoint != -1) {
			text = "";
			if (wayBlock.nameRondPoint!="")
				text+= this.labels["lab_rond1"]+" " + wayBlock.nameRondPoint+", " +this.labels["lab_rond2"];
			else 
				text+= this.labels["lab_rondFull"];
			
			text+= " " +this._computeRoundWording(wayBlock.inRondPoint) + this._mkWay(wayBlock, ": ") + "<b>" + wayListToString + "</b>";
			
			
			if (wayBlock.inCity != "") {
				if (wayBlock.inProxy) { 
					text = this.labels["lab_prox"]+" <b>" + wayBlock.inCity + "</b>"+", ";
					
					if (wayBlock.nameRondPoint!="")
						text+= this.labels["lab_rond1"]+" " + wayBlock.nameRondPoint+", " +this.labels["lab_rond2"];
					else
						text+= this.labels["lab_rondFull"];
					
					text+= " " + this._computeRoundWording(wayBlock.inRondPoint) +" "+ this._mkWay(wayBlock,this.labels["lab_rond_point_continue"]+" ") + "<b>" + wayListToString + "</b>";
				} else {
					text = this.labels["lab_a"] +" <b>" + wayBlock.inCity + "</b>, ";
					
					if(wayBlock.nameRondPoint!="")
						text+=this.labels["lab_rond1"]+" " + wayBlock.nameRondPoint+", " +this.labels["lab_rond2"];
					else
						text+= this.labels["lab_rondFull"];
						
					text+=" " + this._computeRoundWording(wayBlock.inRondPoint) +" "+ this._mkWay(wayBlock, ": ") + "<b>" + wayListToString + "</b>";
				}
			}
			
			strPicto = "";
			if ((wayBlock.pays == "GBR")||(wayBlock.pays == "IRL")) strPicto = "GBR";
			wayBlock.picto = "<img class='itifdr18' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/rondPoint"+strPicto+".png'  alt='' /><br/>";
		}
		
		if (wayBlock.type == 4) wayBlock.picto = "<img class='itifdr18' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/fdr_pieton.png'  alt=''/><br/>";
		
		
		if (showPanel && (!dontShowPanels)) {
			text += "<span class=noMap> "+this.labels["lab_direction"]+" </span>";
		}
		
		if (wayBlock.distPleasant) {
			text += "<img class='itifdr18' style='float:right;' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/tourism.png' alt='' />";
		}
		
		if (showPanel && (!dontShowPanels)) {
			var distance = wayBlock.tDist, classname = "dirPan";
			
			if(isDetail){
				distance = wayBlock.tDist - wayBlock.lDist;
			}
			
			if (isDetail) classname+= " isDetail";
			text += "<div class=\""+classname+"\" schema=\""+ wayBlock.inDirLib[1] +"\" distance=\""+ ViaMichelin.Api.Util.makeDist(distance,this.unit) +"\">" + this._buildPanneaux("V", wayBlock.inDirLib[0],wayBlock.inStyle,wayBlock.pays,(wayBlock.coords != "" ? wayBlock.voies : "")) + "</div>";
		}
		
		if(wayList.length > 0){
			this.lastStepWay = wayList[0][0];
		}
		
		return text;
	},
	_buildStyledDirections : function(blocStr,style,pays)
	{
		while (blocStr.indexOf(",") != -1) blocStr = blocStr.substring(0,blocStr.indexOf(",")) + "<br/>" + blocStr.substr(blocStr.indexOf(",")+1);
		if ((pays == "FRA")||(pays == "ITA")) blocStr = blocStr.toUpperCase();
		var classContainer = "pan" + style;
		return "<div class='"+classContainer+"' style='padding:5px;width:200px;text-align:center;margin:5px;margin-left:0px;'>"+blocStr+"</div>";
	},
	_mkWay : function(wayBlock,str) {
		return ((this.getWayListToString(wayBlock[10])!="")?str:"");
	},
	/**
	 * Affichage des petits panneaux
	 */
	_computePanneaux : function(wayBlock,imgIti){
		var strPanneaux = "";
		var lastType = "";
		var imgIti = this.imgSrc;
		var p = 0;
		while (p < wayBlock[10].length) {
			if ((wayBlock[10][p]!="")&&(wayBlock[10][p][1] != lastType)&&(wayBlock[10][p][0].length<7)) {
				strPanneaux += '<div class="pan' + wayBlock[10][p][1] + '" style="text-align:center">' + wayBlock[10][p][0] + '</div>';
				lastType = wayBlock[10][p][1];
			}
			p++;
		}
		if		(wayBlock.type == 6) strPanneaux = "<img class='itifdr18' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/highway.png' alt='' />" + strPanneaux;
		else if (wayBlock.type == 5) strPanneaux = "<img class='itifdr18' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/express.png' alt='' />" + strPanneaux;
		else if (wayBlock.type == 3) strPanneaux = "<img class='itifdr18' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/liaisF.png' alt='' />" + strPanneaux;
		else if (wayBlock.type == 1) strPanneaux = "<img class='itifdr18' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/liaisM.png' alt='' />" + strPanneaux;
		else if (wayBlock.type == 2) strPanneaux = "<img class='itifdr18' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/liaisM.png' alt='' />" + strPanneaux;
		else if (wayBlock.type == 4) strPanneaux = "<img class='itifdr18' src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/fdr_pieton.png' alt='' />" + strPanneaux;
		return strPanneaux;
	},
	_checkClassified : function(productId){
		if(productId == "125197" || productId == "125196" || productId == "125198"
		|| productId == "125201" || productId == "125199"){
			return true;
		}
		return false;
	},
	/**
	 * Décode les éléments du schema
	 * @param {Object} schema
	 */
	_decodeSchema : function(schema){
		// Traitements des tronçons adjacents
		var items = schema.sections.adj, len = items.length;
		for(var i=0; i<len; i++){
			var item = items[i];
			if (item[0] && item[0].length > 0)
				item[0] = this._decodeLine(item[0][0].replace(/\\\\/g, "\\"), true);
		}
		
		// Traitements des tronçons de l'itineraire
		var items = schema.sections.iti, len = items.length;
		for(var i=0; i<len; i++){
			var item = items[i];
			if (item[0] && item[0].length > 0)
				item[0] = this._decodeLine(item[0][0].replace(/\\\\/g, "\\"), true);
		}
		
		// Traitements des instructions
		var items = schema.sections.ins, len = items.length;
		for(var i=0; i<len; i++){
			var item = items[i];
			if (item[0] && item[0].length > 0)
				item[0] = this._decodeLine(item[0][0].replace(/\\\\/g, "\\"), true);
		}
		
		return schema;
	},
	// These functions decode a polyline pointstring.
	// The first is mine and simply links the second function to the form.
	_decode : function () {
		var instring;
		var outstring;
		var points;
		  
		instring = document.getElementById("polylineDecoder").encodedPolylineIn.value;
		instring = instring.replace(/\\\\/g, "\\");
		points = this._decodeLine(instring);
		outstring = "";
		for(i=0; i < points.length; i++) {
			outstring = outstring + points[i][0] + ", " + points[i][1] + "\n";
		}
		document.getElementById("polylineDecoder").decodedPolylineOut.value = outstring;
	},
	
	// This function is from Google's polyline utility.
	_decodeLine : function (encoded, pixel) {
		var isPixel = ($isDefined(pixel)) ? pixel : false;
		var len = encoded.length;
		var index = 0;
		var array = [];
		var lat = 0;
		var lng = 0;
		
		while (index < len) {
			var b;
			var shift = 0;
			var result = 0;
			do {
				b = encoded.charCodeAt(index++) - 63;
				result |= (b & 0x1f) << shift;
				shift += 5;
			} while (b >= 0x20);
			var dlat = ((result & 1) ? ~(result >> 1) : (result >> 1));
			lat += dlat;
			
			shift = 0;
			result = 0;
			do {
				b = encoded.charCodeAt(index++) - 63;
				result |= (b & 0x1f) << shift;
				shift += 5;
			} while (b >= 0x20);
			var dlng = ((result & 1) ? ~(result >> 1) : (result >> 1));
			lng += dlng;
			
			var multi = 1;
			if (isPixel) {
				multi = 10000;
			}
			array.push([parseInt(multi*(lng * 1e-5)), parseInt(multi*(lat * 1e-5))]);
		}
		
		return array;
	},
	/**
	 * Méthode qui convertie le format actuel en le futur format json
	 * @param {Object} data
	 * @deprecated
	 */
	_convertJSON : function(data) {
		var ndata = {};
			ndata.size = data["taille_image"];
			ndata.sections = {
				"itinerary"		: [],
				"adjacent"		: [],
				"instructions"	: []
			};
			ndata.roadsLabel = [];
			ndata.puces = [];
		
		// Tronçons itinerary
		for(var i=0 ;i<data["troncons_svg"]["troncons_its"].length; i++){
			var item = new Array();
				item[0] = data["troncons_svg"]["troncons_its"][i]["lst_pts"];
				item[1] = data["troncons_svg"]["troncons_its"][i]["atts"]["iso_pays"];
				
			var splitItem = data["troncons_svg"]["troncons_its"][i]["atts"]["classe"].split("_");
				item[2] = parseInt(splitItem[1]);
				item[3] = parseInt(splitItem[2]);
			
			ndata.sections.itinerary.push(item);
		}
		
		// Tronçons adjacent
		for(var i=0 ;i<data["troncons_svg"]["troncons_adj"].length; i++){
			var item = new Array();
				item[0] = data["troncons_svg"]["troncons_adj"][i]["lst_pts"];
				item[1] = parseInt(data["troncons_svg"]["troncons_adj"][i]["atts"]["roadtype"]);
				item[2] = parseInt(data["troncons_svg"]["troncons_adj"][i]["atts"]["importance"]);
				item[3] = parseInt(data["troncons_svg"]["troncons_adj"][i]["atts"]["sens"]);
			
			ndata.sections.adjacent.push(item);
		}
		
		// Tronçons instructions
		for(var i=0 ;i<data["troncons_svg"]["troncons_inst"].length; i++){
			var item = new Array();
				item[0] = data["troncons_svg"]["troncons_inst"][i]["lst_pts"];
				
				if(data["troncons_svg"]["troncons_inst"][i]["atts"]["classe"] == "majeure"){
					item[1] = 1;
				} else {
					item[1] = 0;
				}
			
			ndata.sections.instructions.push(item);
		}
		
		// Road Label
		for(var i=0 ;i<data["etiquettes_route"].length; i++){
			var item = new Array();
				item[0] = data["etiquettes_route"][i]["pos"];
				item[1] = data["etiquettes_route"][i]["num"];
				item[2] = data["etiquettes_route"][i]["pays"];
				item[3] = data["etiquettes_route"][i]["typ"];
				
			ndata.roadsLabel.push(item);
		}
		
		// Puces
		for(var i=0 ;i<data["puces"].length; i++){
			var puce = new Array();
			
			if(data["puces"][i]["type_check"] == "depart"){
				puce[0] = data["puces"][i]["type_check"];
				puce[1] = data["puces"][i]["type_puce"];
				puce[2] = data["puces"][i]["indice_puce"];
				puce[3] = data["puces"][i]["indice_majeur"];
				puce[4] = data["puces"][i]["x_pix"];
				puce[5] = data["puces"][i]["y_pix"];
				ndata.puces.push(puce);
			}
		}
		
		return ndata;
	},
	/**
	 * Clone Roadsheet
	 * @return {ViaMichelin.Api.Model.Itinerary.RoadSheet}
	 */
	clone : function(){
		var cloneComponent = new ViaMichelin.Api.Model.Itinerary.RoadSheet();
		for (var item in this) {
			if (item == "steps") {
				cloneComponent["steps"] = [];
				for(var i=0;i<this.steps.length;i++){
					cloneComponent["steps"].push(this.steps[i]);
				}
			} else {
				cloneComponent[item] = this[item];
			}
		}
		
		// ajout de roadsheet au tableau global.
		if (window._VMRoadsheets != null) {
			window._VMRoadsheets.push(cloneComponent);
			cloneComponent._VMRoadsheetsIndex = window._VMRoadsheets.length - 1;
		}
		
		return cloneComponent;
	},
	release : function() {
		this._poiList = [];
		
		if (this._VMRoadsheetsIndex != null && window._VMRoadsheets != null){
			window._VMRoadsheets[this._VMRoadsheetsIndex]._poiList = [];
			window._VMRoadsheets[this._VMRoadsheetsIndex] = null;
		}
	},
	toString : function(){
		return "[ViaMichelin.Api.Model.Itinerary.RoadSheet]";	
	}
};

ViaMichelin.Api.Model.Itinerary.RoadSheet.prototype.schema = {
	"size"		: [ "276", "905" ],
	"emprise"	: [ -3.5, 2.4, 15.9, 30.05 ],
	"sections"	: {
		"itinerary" : [
			[
				[
					[ 125, 73 , "2.35", "48.8" ],
					[ 126, 74 , "2.35", "48.8" ],
					[ 127, 75 , "2.35", "48.8" ]
					// etc...
				],
				"FRA",
				"iti_0_15"
			],
			[
				[
					[ 125, 73 , "2.35", "48.8" ]
					// etc.
				],
				"FRA",
				"iti_7_15"
			]
			// etc.
		],
		"adjacent" : [],
		"instructions" : []
	}
};